/**
 * @class Sb.Editor
 * <p>This singleton object, which is represent core of Sitebuilder.</p>
 * @singleton
 */
Sb.Editor = function() {
	return {
		/**
		 * Initialize the Editor and its components
		 */
		init: function() {
			Ext.BLANK_IMAGE_URL = Sb.Config.blankImageUrl;
			Ext.QuickTips.init();

			Ext.state.Manager.setProvider(
				new Ext.state.CookieProvider({
					path: "/",
					expires: new Date(new Date().getTime() + (60*60*24*30*12*10*1000)) //10 years
				})
			);

			this.windowManager = new Sb.WindowManager();
			/**
			 * The site model
			 * @type Sb.model.Site
			 * @property model
			 */
			this.model = new Sb.model.Site();
			this.model.on('beforeload', function() {
				Sb.Loading.updateMessage(SbGetLocaleByKey('loaderMsgInitialize'));
			});
			this.model.on('load', function() {
				Sb.Editor.Ui.init();
				Sb.Loading.hide();

				if (Sb.Config.gettingStartedDemoEnabled && Ext.state.Manager.get('showGettingStarted') != false) {
					this.showGettingStarted();
				}
			}, this);
			this.siteView = new Sb.SiteView();
			Sb.Loading.updateMessage(SbGetLocaleByKey('loaderMsgSiteStructure'));
			this.model.load();
		},

		/**
		 * Open help window
		 */
		openHelp: function(articleUrl) {
			if (undefined == articleUrl) {
				articleUrl = '';
			}
			var openResult = window.open(Sb.Config.helpUrl + encodeURIComponent(articleUrl), '_blank');
			if (!openResult) {
				Sb.Editor.showLocalizedWarning('helpPopupBlocked');
			}
		},

		showLocalizedWarning: function (msgLocaleKey) {
			Ext.Msg.show({
				title	: SbGetLocaleByKey('warningMessageBoxTitle'),
				msg		: SbGetLocaleByKey(msgLocaleKey),
				closable: false,
				buttons	: Ext.Msg.OK
			});
		},

		showPublishErrorDetails: function() {
			var publishResult = Sb.Editor.Ui.mainWindow.progressBar.publishResult;
			var msg = publishResult._statusText + '\n';
			Ext.each(publishResult._messagesDetailed, function(item) {
				msg += item.message + '\n';
			});

			var win = new Sb.Window({
				layout	: 'fit',
				width	: 500,
				height	: 300,
				plain	: true,
				title	: SbGetLocaleByKey('publishDetailsTitle'),
				modal	: true,

				items: {
					xtype: 'textarea',
					id: 'sb-publish-details-text',
					value: msg,
					readOnly: true
				},

				buttons: [{
					text: SbGetLocaleByKey('sendToAdmin'),
					handler: function() {
						this.disable();
						Sb.SiteService.msgToAdmin(Ext.getCmp('sb-publish-details-text').getValue(), function(result) {
							win.close();
							Sb.Editor.Ui.mainWindow.statusBar.setMessageInfo(SbGetLocaleByKey('emailSentToAdmin'));
						});
					}
				}]
			});
			win.show(this);
		},

		showGettingStarted: function() {
			var protocol = Ext.isSecure ? 'https' : 'http';
			var win =  new Sb.Window({
				modal: true,
				resizable: false,
				cls: 'sb-getting-started',
				width: 599,
				autoHeight: true,
				defaults: {
					border: false
				},
				items: [{
					xtype: 'box',
					cls: 'sb-getting-started-greeting-block',
					html: SbGetLocaleByKey('gettingStartedMessage')
				}, {
					xtype: 'box',
					cls: 'sb-getting-started-player-block',
					html: '<object id="csSWF" classid="clsid:d27cdb6e-ae6d-11cf-96b8-444553540000" width="480" height="288" codebase="' + protocol + '://download.macromedia.com/pub/shockwave/cabs/flash/swflash.cab#version=9,0,115,0">'
						+ '<param name="src" value="' + Sb.Config.gettingStartedDemoUrl + '"/>'
						+ '<param name="bgcolor" value="#1a1a1a"/>'
						+ '<param name="quality" value="best"/>'
						+ '<param name="allowScriptAccess" value="always"/>'
						+ '<param name="allowFullScreen" value="true"/>'
						+ '<param name="scale" value="showall"/>'
						+ '<param name="flashVars" value="autostart=false#&color=0x1A1A1A,0x1A1A1A"/>'
						+ '<embed name="csSWF" src="' + Sb.Config.gettingStartedDemoUrl + '" width="480" height="288" bgcolor="#1a1a1a" quality="best" allowScriptAccess="always" allowFullScreen="true" scale="showall" flashVars="autostart=false&color=0x1A1A1A,0x1A1A1A" pluginspage="' + protocol + '://www.macromedia.com/shockwave/download/index.cgi?P1_Prod_Version=ShockwaveFlash"></embed>'
					+ '</object>'
				}, {
					cls: 'sb-getting-started-tools-block',
					defaults: {
						border: false
					},
					items: [{
						cls: 'sb-getting-started-tools-checkbox-block',
						items: [{
							xtype: 'checkbox',
							id: 'showGettingStarted',
							boxLabel: SbGetLocaleByKey('showGettingStarted'),
							checked: (Ext.state.Manager.get('showGettingStarted')) ? true : false
						}]
					}, {
						cls: 'sb-getting-started-tools-button-block',
						items: [{
							xtype: 'button',
							cls: 'sb-getting-started-close',
							width: 75,
							text: SbGetLocaleByKey('closeGettingStarted'),
							handler: function(){
								win.close();
							}
						}]
					}]
				}],
				listeners: {
					scope: this,
					close: function() {
						Ext.state.Manager.set('showGettingStarted', Ext.getCmp('showGettingStarted').checked);
					}
				}
			});
			win.show();
		}
	};
}();