/**
 * @class Sb.Utils
 * <p>Contains a bunch of usable utils.</p>
 * @singleton
 */
Sb.Utils = function() {
	return {
		/**
		 * Generate uuid
		 * @return {String} uuid
		 */
		generateUuid: function() {
			var chars = '0123456789abcdef'.split(''), uuid = [], r;
			// rfc4122 requires these characters
			uuid[8] = uuid[13] = uuid[18] = uuid[23] = '-';
			uuid[14] = '4';

			// Fill in random data.  At i==19 set the high bits of clock sequence as
			// per rfc4122, sec. 4.1.5
			for (var i = 0; i < 36; i++) {
				if (!uuid[i]) {
					r = 0 | Math.random() * 16;
					uuid[i] = chars[(i == 19) ? (r & 0x3) | 0x8 : r];
				}
			}
			return uuid.join('');
		},

		/**
		 * Generates random number between 0 and "to" param inclusive.
		 * When to is array, it returns random value from it.
		 *
		 * @param {Number|Array|Ext.util.MixedCollection} to Upper limit
		 * @return {Number|Object}
		 */
		random: function(to) {
			if (Ext.isArray(to)) {
				return to[this.randomIndex(to)];
			}
			if (to instanceof Ext.data.ArrayStore) {
				return to.getAt(this.randomIndex(to.data));
			}
			return Math.floor(Math.random()*to);
		},

		/**
		 * Returns random index from input array.
		 *
		 * @param {Array|Ext.util.MixedCollection} arrayValue
		 * @return {Number}
		 */
		randomIndex: function(arrayValue) {
			return this.random(arrayValue.length);
		}
	};
}();