/**
 * @class Sb.WidgetTypeManager
 * <p>Manages types of widgets, when they are not created yet, helps in buttons and DnD.</p>
 * @singleton
 */
Sb.WidgetTypeManager = function() {
	var types = new Ext.util.MixedCollection(false,
		/**
		 * Provide key for collection
		 * @param {Sb.widgetType.Base} widgetType
		 * @return {String}
		 */
		function(widgetType) {
			return widgetType.getKey();
		}
	);

	return {

		/**
		 * Widget factory by type.
		 *
		 * @param {Object} config The configuration options for the widget model
		 * @return {Sb.model.Widget}
		 */
		create: function(config) {
			config = Ext.apply({
				type: ''
			}, config);
			return this.get(config.type.toString()).createInstance(config);
		},

		/**
		 * Widget cleaner by type
		 *
		 * @param {Sb.model.Widget} widget
		 */
		remove: function(widget) {
			this.get(widget.data.type.toString()).removeInstance(widget);
		},

		/**
		 * Restore widget in cache
		 *
		 * @param {Sb.model.Widget} widget
		 */
		restore: function(widget) {
			this.get(widget.data.type.toString()).restoreInstance(widget);
		},

		/**
		 * Creates type once and return it
		 * @param {String} key Name of the type
		 * @return {Sb.widgetType.Base}
		 */
		get: function(key) {
			key = key.toString().camelize();
			typeName = key.toUpperCaseFirst();
			if (!types.map[key]) {
				if (!Sb.widgetType[typeName]) {
					typeName = 'Base';
				}
				types.add(
					new Sb.widgetType[typeName](key)
				);
			}
			return types.map[key];
		},

		/**
		 * Register for type it`s component(-s)
		 *
		 * @param {Ext.Component} component
		 * @param {Sb.widgetType.Base} widgetType
		 */
		registerComponent: function(component, widgetType) {
			this.get(widgetType.toString()).components.add(component);
		}
	};
}();