Ext.onReady(function() {
	Ext.BLANK_IMAGE_URL = Sb.Config.blankImageUrl;

	var selectedTemplateId = null;

	var templatesStore = new Ext.data.JsonStore({
		proxy: new Ext.data.HttpProxy({
			method: 'POST',
			url: Sb.Config.apiUrl + '/GetSiteTemplateList'
		}),
		baseParams: {foo: 'bar'}, // :INFO: bug ~35217
		autoLoad: true,
		fields: ['id', 'title', 'description', {name: 'templates', type: 'object'}],
		listeners: {
			load: function() {
				doLayout();
				categoriesView.select(0);
				templatesView.select(0);
				Sb.Loading.hide();
			}
		},
		filterByKeywords: function(query) {
			this.snapshot = this.snapshot || this.data;
			var data = this.snapshot || this.data;

			var sep = new RegExp('[~!@#%^&:%_, <>\\-\\/]');
			query = query.trim().split(sep);
			Ext.iterate(query, function(word, index) {
				query[index] = Ext.escapeRe(word);
			}, query);
			var queryRegExp = new RegExp('^(' + query.join('|') + ')', 'ig');

			var r = new Ext.util.MixedCollection();
			r.getKey = data.getKey;
			var keys = data.keys, it = data.items;
			var count = 0;
			for (var i = 0, len = it.length; i < len; i++) {
				var templates = it[i].get('templates');
				var matchedTemplates = [];
				for (var j = 0, lenj = templates.length; j < lenj; j++) {
					var match = false;

					for (var k = 0, lenk = templates[j].keywords.length; k < lenk; k++) {
						if (queryRegExp.test(templates[j].keywords[k])) {
							match = true;
							break;
						}
					}

					var words = templates[j].description.split(sep).concat(templates[j].title.split(sep));
					for (var idescr = 0; idescr < words.length; idescr++) {
						if (queryRegExp.test(words[idescr])) {
							match = true;
							break;
						}
					}

					if (match) {
						matchedTemplates.push(templates[j]);
					}
				}
				count = count + matchedTemplates.length;
				r.add(keys[i], new Ext.data.Record({
					id: it[i].get('id'),
					title: it[i].get('title'),
					description: it[i].get('description'),
					templates: matchedTemplates
				}));
			}

			this.data = r;
			this.fireEvent('datachanged', this);
			return count;
		}
	});

	var categoriesView = new Ext.DataView({
		store: templatesStore,
		singleSelect: true,
		renderTo: 'wizard-categories-box',
		tpl: new Ext.XTemplate(
			'<ul class="wizard-categories-box">',
				'<tpl for=".">',
				'<li class="cat-{id} {[values.templates.length > 0 ? "" : "empty"]}"><a href="#"><span><b>{title}{[values.templates.length > 0 ? " (" + values.templates.length + ")" : ""]}</b></span></a></li>',
				'</tpl>',
			'</ul>'
		),
		itemSelector: 'li',
		selectedClass: 'active',
		listeners: {
			selectionchange: function(dv, nodes) {
				if (dv.getSelectionCount() < 1) {
					return;
				}

				if (bound) {
					var index = dv.getSelectedIndexes()[0];
					var categoryElement = Ext.fly('category-' + (index + 1));
					categoryElement.removeClass('collapsed');
					var top = categoryElement.dom.offsetTop + 1;
					unbindScroll();
					templatesView.container.scrollTo('top', top, {callback: bindScroll});
				}
			},
			beforeclick: function(dv, index, node, e) {
				if (Ext.get(node).hasClass('empty')) {
					return false;
				}
			}
		}
	});

	var templatesView = new Ext.DataView({
		store: templatesStore,
		singleSelect: true,
		renderTo: 'wizard-templates-box',
		tpl: new Ext.XTemplate(
			'<tpl for=".">',
			'<tpl if="templates.length &gt; 0">', // :NOTE: that the > is encoded
			'<div class="b-category" id="category-{#}">',
				'<h3 class="b-category-title">',
					'<span onClick="Ext.get(this).up(\'.b-category\').toggleClass(\'collapsed\');"><em></em>{title}</span>',
				'</h3>',
				'<div class="b-category-description">{description}</div>',

				'<ul class="wizard-templates-box">',
				'<tpl for="templates">',
					'<li class="" id="{id}">',
						'<div class="tpl-wrap-1"><div class="tpl-wrap-2"><div class="tpl-wrap-3"><div class="tpl-wrap-4">',
							'<div class="tpl-content clearfix">',
								'<div class="tpl-img"><img src="', Sb.Config.siteTemplatesUrl ,'/{id}/icon.png" alt="" /></div>',
								'<div class="tpl-title"><h3><a href="#"><span>{title:wbrBySlash}</span></a></h3></div>',
								'<div class="tpl-description">{description:defaultValue("This template does not have a description yet.")}</div>',
							'</div>',
						'</div></div></div></div>',
					'</li>',
				'</tpl>',
				'</ul>',
			'</div>',
			'</tpl>',
			'</tpl>'
		),
		itemSelector: 'li',
		selectedClass: 'tpl-selected',
		overClass: 'tpl-hover',
		listeners: {
			containerclick: function() {
				return false;
			},
			selectionchange: function(dv, nodes) {
				if (dv.getSelectionCount() < 1) {
					return;
				} else {
					selectedTemplateId = nodes[0].id;
				}
			},
			beforeclick: function(dv, index, node, e) {
				var el = Ext.get(node).findParent('div.b-category');
				var lastId = parseInt(el.id.substr(9)) - 1;
				categoriesView.suspendEvents();
				categoriesView.select(lastId);
				categoriesView.resumeEvents();

				var top,
					offsetTop = Ext.get(node).getOffsetsTo(dv.container)[1],
					scrollTop = dv.container.getScroll().top,
					areaHeight = dv.container.getHeight();
				if (offsetTop + Ext.get(node).getHeight() + 30 > areaHeight) {
					top = offsetTop + Ext.get(node).getHeight() - areaHeight + scrollTop + 50;
					dv.container.scrollTo('top', top, true);
				} else if (offsetTop < 0) {
					top = offsetTop + scrollTop - 10;
					dv.container.scrollTo('top', top, true);
				}

				if (Ext.get(node).hasClass('tpl-hover')) {
					Ext.get(node).removeClass('tpl-hover');
					delete dv.lastItem;
				}
			}
		}
	});

	var searchField = new Ext.form.TextField({
		applyTo: 'search-field',
		emptyText: SbGetLocaleByKey('EmptySearch'),
		value: '',
		lengthToSearch: 0,
		enableKeyEvents: true,
		listeners: {
			keyup: function(tf, e) {
				if (e.ESC == e.getKey()) {
					tf.setValue('');
					templatesStore.clearFilter();
				}

				if (tf.getValue().trim().length > this.lengthToSearch) {
					if (!templatesStore.filterByKeywords(tf.getValue())) {
						templatesView.update('<div class="b-category-empty">' + SbGetLocaleByKey('NoMatch') + '</div>');
					}
				} else {
					templatesStore.clearFilter();
				}

				Ext.each(templatesView.getNodes(), function(node) {
					if (node.id == selectedTemplateId) {
						templatesView.select(node);
						return false;
					}
				});
				calcScrollPosition();
			}
		}
	});

	var siteNameField = new Ext.form.TextField({
		maxLength: 255,
		applyTo: 'sb-wizard-site-name'
	});

	var localeField = new Ext.ux.IconCombo({
		applyTo: 'sb-wizard-site-language',
		editable: false,
		triggerAction: 'all',
		mode: 'local',
		shadow: false,
		store: new Ext.data.ArrayStore({
			fields: [
				'code',
				'name',
				'iconCls'
			],
			data: localesData,
			sortInfo: {
				field: 'name'
			}
		}),
		hiddenName: 'locale',
		valueField: 'code',
		displayField: 'name',
		iconClsField: 'iconCls',
		value: Sb.Config.locale,
		width: 175,
		getListParent: function() {
			return this.container;
		}
	});

	function onCreateSite(){
		if (null === selectedTemplateId) {
			return;
		}

		var selectedCategory = null;
		templatesStore.each(function(record) {
			Ext.each(record.get('templates'), function(template) {
				if (template.id == selectedTemplateId) {
					selectedCategory = record.get('id');
					return false;
				}
			});
			if (selectedCategory) {
				return false;
			}
		});

		if (selectedCategory == 'Other') {
			var win = new Ext.Window({
				id: 'submit-website-topic-suggest-win',
				cls: 'x-window-dlg',
				width: 400,
				title: SbGetLocaleByKey('SubmitWebsiteTopicSuggestTitle'),
				renderTo: Ext.getBody(),
				modal: true,
				draggable: false,
				resizable: false,
				shadow: false,
				items: {
					xtype: 'form',
					border: false,
					items: [{
						xtype: 'displayfield',
						hideLabel: true,
						html: '<p>' + SbGetLocaleByKey('SubmitWebsiteTopicSuggestDescription') + '</p>'
					}, {
						emptyText: SbGetLocaleByKey('SubmitWebsiteTopicSuggestEmptyField'),
						id: 'submit-website-topic-suggest-text',
						hideLabel: true,
						xtype: 'textarea',
						maxLength: 1000,
						anchor: '100%'
					}]
				},
				buttonAlign: 'center',
				fbar: [{
					cls: 'sb-btn-action',
					text: SbGetLocaleByKey('SubmitAndCreateSite'),
					handler: function() {
						Sb.SiteTemplateService.submitSuggest(Ext.getCmp('submit-website-topic-suggest-text').getValue().substring(0, 1000));
						showUserInfoDialog();
						win.close();
					}
				}]
			});
			win.show();
			return;
		}

		showUserInfoDialog();
	}

	var createSiteButton = new Ext.Button({
		cls: 'sb-btn-action',
		iconCls: 'sb-ok-icon',
		text: SbGetLocaleByKey('CreateSite'),
		renderTo: 'sb-wizard-create-site-btn',
		tooltip: SbGetLocaleByKey('CreateSiteTooltip'),
		tooltipType: 'title',
		disabled: true,
		handler: onCreateSite
	});

	var isSiteDeploymentStarted = false;

	function preloadResources() {
		var preload = Ext.emptyFn;
		Ext.each(Sb.cssListForPreloading.concat(Sb.jsListForPreloading), function(fileUrl) {
			var preloadSequence = preload;
			preload = function() {
				if (isSiteDeploymentStarted) {
					return;
				}
				Ext.Ajax.request({
					url: fileUrl,
					disableCaching: false,
					callback: preloadSequence
				});
			};
		});
		preload();
	}

	var siteOwnerInfo = {};
	Sb.SiteService.getOwnerInfo(
		function(result, response) {
			if (response.status) {
				siteOwnerInfo = result;
				createSiteButton.enable();
			}
			Sb.SiteService.preparePersistentResources();
			preloadResources.defer(2000);
		}
	);

	function showUserInfoDialog() {
		var win = new Ext.Window({
			cls: 'x-window-dlg',
			width: 400,
			title: SbGetLocaleByKey('UserEditInfoTitle'),
			renderTo: Ext.getBody(),
			modal: true,
			draggable: false,
			resizable: false,
			shadow: false,
			keys: {
				key: Ext.EventObject.ENTER,
				stopEvent: true,
				fn: function() {
					deploySiteTemplate();
					win.close();
				}
			},
			items: {
				xtype: 'form',
				border: false,
				id: 'userEditInfoForm',
				labelAlign: 'right',
				labelWidth: 150,
				defaults: {
					xtype: 'textfield',
					maxLength: 255
				},
				items: [{
					xtype: 'displayfield',
					hideLabel: true,
					html: '<p>' + SbGetLocaleByKey('UserEditInfoDescription') + '</p>'
				}, {
					fieldLabel: SbGetLocaleByKey('UserEditInfoPersonalName'),
					name: 'personalName',
					value: siteOwnerInfo.personalName
				}, {
					fieldLabel: SbGetLocaleByKey('UserEditInfoCompanyName'),
					name: 'companyName',
					value: siteOwnerInfo.companyName
				}, {
					fieldLabel: SbGetLocaleByKey('UserEditInfoPhone'),
					name: 'phone',
					value: siteOwnerInfo.phone
				}, {
					fieldLabel: SbGetLocaleByKey('UserEditInfoEmail'),
					name: 'email',
					value: siteOwnerInfo.email
				}, {
					fieldLabel: SbGetLocaleByKey('UserEditInfoAddress'),
					name: 'address',
					value: siteOwnerInfo.address
				}, {
					fieldLabel: SbGetLocaleByKey('UserEditInfoCity'),
					name: 'city',
					value: siteOwnerInfo.city
				}, {
					fieldLabel: SbGetLocaleByKey('UserEditInfoCountry'),
					name: 'country',
					value: siteOwnerInfo.country
				}]
			},
			buttonAlign: 'center',
			fbar: [{
				cls: 'sb-btn-action',
				text: SbGetLocaleByKey('SubmitAndCreateSite'),
				handler: function() {
					deploySiteTemplate();
					win.close();
				}
			}]
		});
		win.show();
	}

	var deploySiteTemplate = function() {
		Sb.Loading.updateMessage(SbGetLocaleByKey('loaderMsgSiteCreated'));
		Sb.Loading.show();
		isSiteDeploymentStarted = true;

		var formValues = Ext.getCmp('userEditInfoForm').getForm().getFieldValues();
		var contentParameters = [{
			name: 'siteName', value: siteNameField.getValue()
		}];
		Ext.iterate(formValues, function(name, value) {
			if ('undefined' == name) {
				return;
			}
			contentParameters.push({
				name: name,
				value: value
			});
		});

		Sb.SiteService.deploySiteTemplate(
			selectedTemplateId,
			localeField.getValue(),
			contentParameters,
			function(result, response) {
				if (response.status) {
					window.location = Sb.Config.baseUrl + '/Editor';
					return;
				}
			}
		);
	};

	var doLayout = function() {
		var scrollArea = Ext.get('wizard-templates-box');
		var distanceToTop = scrollArea.getOffsetsTo(Ext.getBody());
		var buttonAreaHeight = Ext.get('btns-box').getHeight();
		var footerAreaHeight = Ext.get('footer-wrapper').getHeight();
		var bodyHeight = Ext.getBody().getHeight();

		var minScrollAreaHeight = Ext.fly('wizard-categories-box').getHeight();
		var newScrollAreaHeight = parseInt(bodyHeight) - parseInt(distanceToTop[1]) - parseInt(buttonAreaHeight) - parseInt(footerAreaHeight) - 20;
		if (newScrollAreaHeight < minScrollAreaHeight) {
			newScrollAreaHeight = minScrollAreaHeight;
		}

		if (scrollArea.getHeight() != newScrollAreaHeight) {
			scrollArea.setHeight(newScrollAreaHeight);
		}
	};
	Ext.EventManager.onWindowResize(doLayout);

	var bound = true;
	function bindScroll() {
		templatesView.container.on('scroll', calcScrollPosition, templatesView.container, {buffer: 250});
		bound = true;
	}
	function unbindScroll() {
		templatesView.container.un('scroll', calcScrollPosition, templatesView.container);
		bound = false;
	}

	function calcScrollPosition() {
		var lastId;
		templatesView.container.select('.b-category').each(function(el) {
			lastId = parseInt(el.id.substr(9)) - 1;
			if (el.getOffsetsTo(templatesView.container)[1] > -5) {
				return false;
			}
		});
		categoriesView.suspendEvents();
		categoriesView.select(lastId);
		categoriesView.resumeEvents();
	}

	doLayout();
	new Ext.KeyMap(document, {
		key: Ext.EventObject.ENTER,
		stopEvent: true,
		fn: function() {
			if (createSiteButton.disabled) {
				return;
			}
			onCreateSite();
		}
	});
});