/**
 * @class Sb.editor.BaseFormPanel
 * @extends Ext.form.FormPanel
 */
Sb.editor.BaseFormPanel = Ext.extend(Ext.form.FormPanel, {
	// private
	initComponent: function() {
		Ext.apply(this, {
			footerCssClass: 'sb-action-footer',
			buttons: [{
				text: SbGetLocaleByKey('ok'),
				handler: this.okHandler,
				scope: this
			}, {
				text: SbGetLocaleByKey('cancelButtonTitle'),
				handler: this.cancelHandler,
				scope: this
			}]
		});

		Sb.editor.BaseFormPanel.superclass.initComponent.apply(this, arguments);
	},

	initEvents: function() {
		Sb.editor.BaseFormPanel.superclass.initEvents.call(this);
		this.on('activate', this.onActivate, this);
	},

	/**
	 * Activate handler
	 */
	onActivate: function() {
		Sb.editor.UiContext.setNewContext({
			yesCallback: this.uiContextYesHandler.createDelegate(this),
			isYesDeferred: true,
			noCallback: this.uiContextNoHandler.createDelegate(this)
		});
	},

	/**
	 * OK button handler.
	 */
	okHandler: function() {
		var mask = Ext.getBody().mask();
		mask.addClass('sb-editor-mask');
		this.buttons[0].setIconClass('sb-loading-icon');
		this.saveHandler({
			success: this.afterOkSuccessHandler.createDelegate(this),
			failure: this.afterOkFailureHandler.createDelegate(this)
		});
	},

	/**
	 * Handler, that will fire after saving of model as success callback.
	 */
	afterOkSuccessHandler: function() {
		this.enableOkButton();
		Sb.editor.UiContext.setNewContext();
		this.closeDialog();
	},

	/**
	 * Handler, that will fire after saving of model as failure callback.
	 */
	afterOkFailureHandler: function() {
		this.enableOkButton();
	},

	/**
	 * Enables OK button.
	 */
	enableOkButton: function() {
		Ext.getBody().unmask();
		this.buttons[0].setIconClass('');
	},

	/**
	 * UI context common handler for yes button.
	 */
	uiContextYesHandler: function() {
		this.saveHandler({
			success: function() {
				Sb.editor.UiContext.finishDeferredAction();
			},
			failure: function() {
				Sb.editor.UiContext.cancelDeferredAction();
			}
		});
	},

	/**
	 * Save handler. Must be overriden in child classes with calling in the end
	 * afterSaveCallback that arrive to it always.
	 */
	saveHandler: Ext.emptyFn,

	/**
	 * Get all active panel`s form errors.
	 *
	 * @return {Array}
	 */
	getActiveErrors: function() {
		var errorMsg = [];
		this.form.items.each(function(formItem){
			if (formItem.isComposite) {
				formItem.eachItem(function(compositeChild) {
					if (compositeChild.activeError) {
						errorMsg.push(compositeChild.activeError);
					}
				});
			} else {
				if (formItem.activeError) {
					errorMsg.push(formItem.activeError);
				}
			}
		});
		return Ext.unique(errorMsg);
	},

	/**
	 * Shows all active panel`s form errors if they exists.
	 *
	 * @return {Boolean} Error show result
	 */
	showActiveErrors: function() {
		var errorMsg = this.getActiveErrors();
		Sb.Editor.Ui.mainWindow.statusBar.clear();
		if (errorMsg.length > 0) {
			Ext.each(errorMsg, function(msg) {
				Sb.Editor.Ui.mainWindow.statusBar.addMessageError(msg);
			});
			return true;
		}
		return false;
	},

	/**
	 * Cancel button handler.
	 */
	cancelHandler: function() {
		this.discardHandler();
		Sb.editor.UiContext.setNewContext();
		this.closeDialog();
	},

	/**
	 * UI context common handler for no button.
	 */
	uiContextNoHandler: function() {
		this.discardHandler();
	},

	/**
	 * Discard handler for undo and another actions.
	 */
	discardHandler: Ext.emptyFn,

	/**
	 * Close dialog by restoring tabs in it`s initial state, when all was hidden.
	 */
	closeDialog: function() {
		Sb.Editor.Ui.mainWindow.tabs.setActiveTab(null);
	}
});