/**
 * @class Sb.editor.LayoutPanel
 * @extends Sb.editor.BaseFormPanel
 * Layout editor panel
 * @constructor
 * @param {Object} config The config object
 * @xtype layoutpanel
 */
Sb.editor.LayoutPanel = Ext.extend(Sb.editor.BaseFormPanel, {

	/**
	 * initial content width
	 */
	contentWidth: 0,

	/**
	 * initial value for lock width
	 */
	lockWidth: true,

	/**
	 * initial value of columns
	 */
	columns: 1,

	/**
	 * initial value for margin type
	 */
	marginType: 'vertical',

	/**
	 * Design proeperties
	 */
	designProperties: null,

	/**
	 * Width of external borders
	 */
	externalBordersWidth: 120,

	/**
	 * Object for saving some temporaries properties
	 */
	tmpProperty: {
		sidebarFirst: false,
		sidebarSecond: false,
		columnSizeEdit: 'width'
	},

	// private
	initComponent: function() {
		this.designProperties = Sb.Editor.model.design.serialize();

		if (this.getColumns() == 2) {
			this.tmpProperty.sidebarFirst = this.designProperties.layout.sidebarFirst;
			this.tmpProperty.sidebarSecond = this.designProperties.layout.sidebarSecond;
		} else {
			this.tmpProperty.sidebarFirst = !Math.round(Math.random());
			this.tmpProperty.sidebarSecond = !this.tmpProperty.sidebarFirst;
		}

		var layoutParams = {
			layoutTypeAndWidth: {
				autoHeight: true,
				cls: 'sb-settings-layout-type',
				items: [{
					xtype: 'box',
					html: SbGetLocaleByKey('layoutType'),
					cls: 'sb-layout-settings-name'
				}, {
					xtype: 'radiogroup',
					fieldLabel: SbGetLocaleByKey('layoutSwitchType'),
					defaults: {
						xtype: 'radio',
						name: 'layoutSwitchType'
					},
					items: [{
						boxLabel: SbGetLocaleByKey('fixed'),
						inputValue: 'fixed',
						checked: this.designProperties.layout.type == 'fixed'
					}, {
						boxLabel: SbGetLocaleByKey('elastic'),
						inputValue: 'elastic',
						checked: this.designProperties.layout.type != 'fixed'
					}],
					listeners: {
						scope: this,
						change: function(group, radio) {
							this.setLayoutType(radio.getGroupValue());
						}
					},
					changeLayout: function(el) {
						el.setValue(this.getLayoutType());
					}.createDelegate(this)
				}, {
					xtype: 'container',
					cls: 'sb-layout-setting-layout-width',
					items: [{
						xtype: 'label',
						html: SbGetLocaleByKey('layoutWidth')
					}, {
						xtype: 'numberfield',
						name: 'width',
						width: 40,
						allowNegative: false,
						enableKeyEvents: true,
						changeLayout: function(el) {
							var width = (this.getLayoutType() == 'fixed')
								? this.getWatermarkWidth()
								: this.getWrapperWidth();

							width = Math.round(width * 100) / 100;
							if (width != el.getValue()) {
								el.setValue(width);
							}
						}.createDelegate(this),
						listeners: {
							scope: this,
							keyup: function(field, event) {
								var width = field.getValue();
								if (this.getLayoutType() == 'fixed') {
									width += this.externalBordersWidth;
								}
								this.setLayoutWidth(width);
							}
						}
					}, {
						xtype: 'box',
						autoEl: 'span',
						text: this.getUnitMeasurement(),
						changeLayout: function(el) {
							el.update(this.getUnitMeasurement());
						}.createDelegate(this)
					}]
				}]
			},
			margins: {
				autoHeight: true,
				cls: 'sb-settings-layout-margins',
				items: [{
					xtype: 'box',
					html: SbGetLocaleByKey('margins'),
					cls: 'sb-layout-settings-name'
				}, {//vertical
					xtype: 'container',
					cls: 'sb-layout-setting-margins-vertical',
					items: [{
						xtype: 'label',
						text: SbGetLocaleByKey('vertical')
					}, {
						xtype: 'numberfield',
						name: 'marginVertical',
						decimalPrecision: 0,
						width: 32,
						enableKeyEvents: true,
						changeLayout: function(el) {
							if (this.getMarginWidth(this.getMarginType()) != el.getValue()) {
								el.setValue(this.getMarginWidth('vertical'));
							}
						}.createDelegate(this),
						listeners: {
							scope: this,
							keyup: function(field, event) {
								this.setMarginWidth(field.getValue(), 'vertical');
							}
						}
					}, {
						xtype: 'box',
						autoEl: 'span',
						html: SbGetLocaleByKey('px')
					}]
				}, {//horizontal
					xtype: 'container',
					cls: 'sb-layout-setting-margins-horizontal',
					items: [{
						xtype: 'label',
						text: SbGetLocaleByKey('horizontal')
					}, {
						xtype: 'numberfield',
						name: 'marginHorizontal',
						decimalPrecision: 0,
						width: 32,
						enableKeyEvents: true,
						changeLayout: function(el) {
							if (this.getMarginWidth(this.getMarginType()) != el.getValue()) {
								el.setValue(this.getMarginWidth('horizontal'));
							}
						}.createDelegate(this),
						listeners: {
							scope: this,
							keyup: function(field, event) {
								this.setMarginWidth(field.getValue(), 'horizontal');
							}
						}
					}, {
						xtype: 'box',
						autoEl: 'span',
						html: SbGetLocaleByKey('px')
					}]
				}]
			},
			columnSizes: {
				autoHeight: true,
				cls: 'sb-settings-layout-column-width',
				items: [{
					xtype: 'box',
					html: SbGetLocaleByKey('layoutColumnTitle'),
					cls: 'sb-layout-settings-name'
				}, {
					xtype: 'label',
					cls: 'sb-layout-settings-label',
					html: SbGetLocaleByKey('layoutColumnMinHeight'),
					changeLayout: function(el) {
						el[this.getColumns() != 1 ? 'hide' : 'show']();
					}.createDelegate(this)
				}, {
					xtype: 'radiogroup',
					fieldLabel: SbGetLocaleByKey('layoutSwitchSize'),
					defaults: {
						xtype: 'radio',
						name: 'layoutSwitchSize'
					},
					items: [{
						boxLabel: SbGetLocaleByKey('layoutColumnWidth'),
						inputValue: 'width',
						checked: this.tmpProperty.columnSizeEdit == 'width'
					}, {
						boxLabel: SbGetLocaleByKey('layoutColumnMinHeight'),
						inputValue: 'height',
						checked: this.tmpProperty.columnSizeEdit != 'width'
					}],
					listeners: {
						scope: this,
						change: function(group, radio) {
							this.tmpProperty.columnSizeEdit = radio.getGroupValue();
							var el = Ext.getCmp('sidebar-left-size-filed');
							el.changeLayout(el);
							el = Ext.getCmp('sidebar-right-size-filed');
							el.changeLayout(el);
							el = Ext.getCmp('content-size-filed');
							el.changeLayout(el);
						}
					},
					changeLayout: function(el) {
						el[this.getColumns() == 1 ? 'hide' : 'show']();
					}.createDelegate(this)
				}, {
					xtype: 'container',
					cls: 'sb-layout-settings-columns-size',
					border: false,
					items: [{
						xtype: 'container',
						autoEl: 'span',
						items: [{
							xtype: 'numberfield',
							width: 40,
							decimalPrecision: 0,
							name: 'sidebarLeft',
							id: 'sidebar-left-size-filed',
							enableKeyEvents: true,
							msgTarget: 'sbtitle',
							minText: SbGetLocaleByKey('layoutColumnWidthMoreThan'),
							minValue: 100,
							changeLayout: function(el) {
								el.minText = this.tmpProperty.columnSizeEdit == 'width' ?
									SbGetLocaleByKey('layoutColumnWidthMoreThan') :
									SbGetLocaleByKey('layoutColumnHeightMoreThan');
								el.setValue(this.getSidebarSize('left'));
								this.showHideSidebarElement(el, 'left');
							}.createDelegate(this),
							listeners: {
								scope: this,
								keyup: function(field, event) {
									if (this.getSidebarSize('left') != field.getValue()) {
										this.setSidebarSize(field.getValue(), 'left');
									}
								}
							}
						}, {
							xtype: 'box',
							autoEl: 'span',
							html: SbGetLocaleByKey('px'),
							changeLayout: function(el) {
								this.showHideSidebarElement(el, 'left');
							}.createDelegate(this)
						}]
					}, {
						xtype: 'container',
						autoEl: 'span',
						items: [{
							xtype: 'numberfield',
							width: 40,
							decimalPrecision: 0,
							name: 'content',
							id: 'content-size-filed',
							enableKeyEvents: true,
							msgTarget: 'sbtitle',
							minValue: 100,
							editable: false,
							disabled: this.tmpProperty.columnSizeEdit == 'width',
							changeLayout: function(el) {
								var type = (this.getColumns() == 1) ? 'height' : this.tmpProperty.columnSizeEdit;
								el.setValue(this.getSidebarSize('center', type));
								el[this.tmpProperty.columnSizeEdit == 'width' && this.getColumns() != 1? 'disable' : 'enable']();
							}.createDelegate(this),
							listeners: {
								scope: this,
								keyup: function(field, event) {
									if (this.getSidebarSize('center') != field.getValue()) {
										var type = (this.getColumns() == 1) ? 'height' : this.tmpProperty.columnSizeEdit;
										this.setSidebarSize(field.getValue(), 'center', type);
									}
								}
							}
						}, {
							xtype: 'box',
							autoEl: 'span',
							html: SbGetLocaleByKey('px')
						}]
					}, {
						xtype: 'container',
						autoEl: 'span',
						items: [{
							xtype: 'numberfield',
							width: 40,
							decimalPrecision: 0,
							name: 'sidebarRight',
							id: 'sidebar-right-size-filed',
							enableKeyEvents: true,
							msgTarget: 'sbtitle',
							minText: SbGetLocaleByKey('layoutColumnWidthMoreThan'),
							minValue: 100,
							changeLayout: function(el) {
								el.minText = this.tmpProperty.columnSizeEdit == 'width' ?
									SbGetLocaleByKey('layoutColumnWidthMoreThan') :
									SbGetLocaleByKey('layoutColumnHeightMoreThan');
								el.setValue(this.getSidebarSize('right'));
								this.showHideSidebarElement(el, 'right');
							}.createDelegate(this),
							listeners: {
								scope: this,
								keyup: function(field, event) {
									if (this.getSidebarSize('right') != field.getValue()) {
										this.setSidebarSize(field.getValue(), 'right');
									}
								}
							}
						}, {
							xtype: 'box',
							autoEl: 'span',
							html: SbGetLocaleByKey('px'),
							changeLayout: function(el) {
								this.showHideSidebarElement(el, 'right');
							}.createDelegate(this)
						}]
					}]
				}]
			}
		};

		Ext.apply(this, {
			cls: 'sb-editor-layout-panel',
			title: SbGetLocaleByKey('layoutTitle'),
			helperMessage: SbGetLocaleByKey('layoutDescription'),
			tabTip: SbGetLocaleByKey('layoutDescriptionOnHover'),
			labelAlign: 'top',
			autoHeight: true,
			items: {
				xtype: 'tabpanel',
				id: 'sb-layout-tabpanel',
				baseCls: 'sb-editor-layout-layout-panel',
				border: false,
				autoWidth:true,
				deferredRender: false,
				defaults: {
					autoHeight: true,
					defaults: {
						autoHeight: true,
						border: false,
						baseCls: 'sb-box'
					}
				},
				items: [{
					title: SbGetLocaleByKey('1Column'),
					tabCls: 'sb-editor-layout-panel-1col-tab',
					baseCls: 'sb-editor-layout-cols-panel',
					columns: 1,
					id: 'sb-layout-tabpanel-tab1',
					items: [layoutParams.layoutTypeAndWidth,
						layoutParams.margins,
						layoutParams.columnSizes]
				}, {
					title: SbGetLocaleByKey('2Column'),
					tabCls: 'sb-editor-layout-panel-2col-'
						+ ((this.designProperties.layout.sidebarFirst) ?
								this.designProperties.layout.sidebarFirstPosition :
								this.designProperties.layout.sidebarSecondPosition) + '-tab',
					columns: 2,
					id: 'sb-layout-tabpanel-tab2'
				}, {
					title: SbGetLocaleByKey('3Column'),
					tabCls: 'sb-editor-layout-panel-3col-tab',
					columns: 3,
					id: 'sb-layout-tabpanel-tab3'
				}],
				listeners: {
					scope: this,
					tabchange: this.onTabChange,
					beforetabchange: this.onBeforeTabChange,
					afterrender: function() {
						var tab2 = Ext.get('sb-layout-tabpanel__sb-layout-tabpanel-tab2');
						var tab3 = Ext.get('sb-layout-tabpanel__sb-layout-tabpanel-tab3');

						new Ext.BoxComponent({
							id: 'sb-layout-tabpanel-tab2-sidebar',
							renderTo: tab2.select('.x-tab-strip-inner').first(),
							autoEl: 'span',
							hidden: this.getColumns() != 2,
							cls: 'sb-editor-layout-sidebar sb-editor-layout-sidebar-' + (this.tmpProperty.sidebarFirst ?
								this.designProperties.layout.sidebarFirstPosition :
								this.designProperties.layout.sidebarSecondPosition),
							html: this.tmpProperty.sidebarFirst ? '1' : '2',
							changeLayout: function(el) {
								el.getEl().removeClass('sb-editor-layout-sidebar-left');
								el.getEl().removeClass('sb-editor-layout-sidebar-right');
								el.getEl().addClass('sb-editor-layout-sidebar-' +
									(this.tmpProperty.sidebarFirst ?
										this.designProperties.layout.sidebarFirstPosition :
										this.designProperties.layout.sidebarSecondPosition)
								);
								el.update(this.tmpProperty.sidebarFirst ? '1' : '2');
								el[this.getColumns() == 2 ? 'show' : 'hide']();
							}.createDelegate(this)
						});

						new Ext.BoxComponent({
							id: 'sb-layout-tabpanel-tab3-sidebar-left',
							renderTo: tab3.select('.x-tab-strip-inner').first(),
							autoEl: 'span',
							hidden: this.getColumns() != 3,
							cls: 'sb-editor-layout-sidebar sb-editor-layout-sidebar-left',
							html: this.designProperties.layout.sidebarFirstPosition == 'left' ?
								'1' : '2',
							changeLayout: function(el) {
								el.update(this.designProperties.layout.sidebarFirstPosition == 'left' ? '1' : '2');
								el[this.getColumns() == 3 ? 'show' : 'hide']();
							}.createDelegate(this)
						});

						new Ext.BoxComponent({
							id: 'sb-layout-tabpanel-tab3-sidebar-right',
							renderTo: tab3.select('.x-tab-strip-inner').first(),
							autoEl: 'span',
							hidden: this.getColumns() != 3,
							cls: 'sb-editor-layout-sidebar sb-editor-layout-sidebar-right',
							html: this.designProperties.layout.sidebarFirstPosition == 'right' ?
								'1' : '2',
							changeLayout: function(el) {
								el.update(this.designProperties.layout.sidebarFirstPosition == 'right' ? '1' : '2');
								el[this.getColumns() == 3 ? 'show' : 'hide']();
							}.createDelegate(this)
						});

						new Ext.Button({
							id: 'sb-layout-tabpanel__switch-column2',
							iconCls: 'sb-switch-icon',
							hidden: true,
							renderTo: tab2,
							text: SbGetLocaleByKey('switchSidebar'),
							handler: function(btn, evt) {
								this.switchColumns();
							}.createDelegate(this),
							changeLayout: function(el) {
								el[this.getColumns() == 2 ? 'show' : 'hide']();
							}.createDelegate(this)
						});
						new Ext.Button({
							id: 'sb-layout-tabpanel__switch-column3',
							iconCls: 'sb-switch-icon',
							hidden: true,
							renderTo: tab3,
							text: SbGetLocaleByKey('switchSidebar'),
							handler: function(btn, evt) {
								this.switchColumns();
							}.createDelegate(this),
							changeLayout: function(el) {
								el[this.getColumns() == 3 ? 'show' : 'hide']();
							}.createDelegate(this)
						});
					}
				}
			}
		});
		Sb.editor.LayoutPanel.superclass.initComponent.apply(this, arguments);
	},

	/**
	 * Update form element from layout dialog
	 */
	onUpdateDialog: function(registerChanges) {
		var xtypes = ['textfield', 'box', 'label', 'radiogroup'];
		Ext.each(xtypes, function(xtype) {
			Ext.each(this.findByType(xtype), function(item){
				if (item.changeLayout) {
					item.changeLayout(item);
				}
			});
		}, this);
		Sb.editor.UiContext.registerChanges();
	},

	/**
	 * Detecting a content in sidebar on all pages
	 * @sidebar {String} sidebar first|second
	 * @return (Boolean)
	 */
	isContentInSidebar: function(sidebar) {
		var isContentSidebar = false;
		sidebar = sidebar == 'first' ? 'column1' : 'column2';
		Sb.Editor.model.rootPage.pages.each(function(page) {
			page.widget.widgets.each(function(widget) {
				if (widget.data.region == sidebar && widget.widgets.getCount()) {
					isContentSidebar = true;
					return false;
				}
			});
			if (isContentSidebar) {
				return false;
			}
		});
		return isContentSidebar;
	},

	/**
	 * Get size (width or height) for sidebar defined by param position
	 * @param {String} position left|center|right
	 * @param {String} type width|height By default defined inner protected property (columnSizeEdit) of this object
	 * @return {Number}
	 */
	getSidebarSize: function(position, type) {
		if (!type || (type !== 'width' && type != 'height')) {
			type = this.tmpProperty.columnSizeEdit;
		}
		if (position == 'center') {
			return type == 'width' ? this.getContentWidth() : this.designProperties.layout.contentMinHeight;
		}
		return (this.designProperties.layout.sidebarFirstPosition == position) ?
			(type == 'width' ?
				this.getSidebarFirstWidth() :
				this.designProperties.layout.sidebarFirstMinHeight) :
			(type == 'width' ?
				this.getSidebarSecondWidth() :
				this.designProperties.layout.sidebarSecondMinHeight);
	},

	/**
	 * Set size (width or height) for sidebar defined by param position
	 * @param {Number} sizeValue
	 * @param {String} position left|center|right
	 * @param {String} type By default defined protected property (columnSizeEdit) of tmpProperty object
	 * @return {Sb.editor.LayoutPanel} this
	 */
	setSidebarSize: function(sizeValue, position, type) {
		if (!type || (type !== 'width' && type != 'height')) {
			type = this.tmpProperty.columnSizeEdit;
		}
		if (position == 'center') {
			if (type == 'height') {
				this.setContentMinHeight(sizeValue);
			}
			//else {
				//NOTE: smikhalyev: 07.07.2010: Place for setWidth for content area if is need
			//}
			return this;
		}
		if (this.designProperties.layout.sidebarFirstPosition == position) {
			type == 'width' ?
				this.setSidebarFirstWidth(sizeValue):
				this.setSidebarFirstMinHeight(sizeValue);
		} else {
			type == 'width' ?
				this.setSidebarSecondWidth(sizeValue) :
				this.setSidebarSecondMinHeight(sizeValue);
		}
		return this;
	},

	/**
	 * Show or hide sidebar size input depending on the position and number of columns
	 * @param {Ext.element} el
	 * @param {String} position left|right
	 * @param {Number} columns By default this.getColumns()
	 */
	showHideSidebarElement: function(el, position, columns) {
		if (!columns) {
			columns = this.getColumns();
		}
		if (this.designProperties.layout.sidebarFirstPosition == position) {
			el[this.designProperties.layout.sidebarFirst ? 'show' : 'hide']();
		} else {
			el[this.designProperties.layout.sidebarSecond ? 'show' : 'hide']();
		}
	},

	/**
	 * Get number of columns in design
	 */
	getColumns: function() {
		return 1 +
			(this.designProperties.layout.sidebarFirst ? 1 : 0) +
			(this.designProperties.layout.sidebarSecond ? 1 : 0);
	},

	/**
	 * Switch position of columns
	 */
	switchColumns: function() {
		var tmpPosition = this.designProperties.layout.sidebarFirstPosition;
		this.designProperties.layout.sidebarFirstPosition = this.designProperties.layout.sidebarSecondPosition;
		this.designProperties.layout.sidebarSecondPosition = tmpPosition;
		Sb.Editor.model.design.unserialize(this.designProperties);

		Ext.getCmp('sb-layout-tabpanel-tab2-sidebar').changeLayout(Ext.getCmp('sb-layout-tabpanel-tab2-sidebar'));
		Ext.getCmp('sb-layout-tabpanel-tab3-sidebar-left').changeLayout(Ext.getCmp('sb-layout-tabpanel-tab3-sidebar-left'));
		Ext.getCmp('sb-layout-tabpanel-tab3-sidebar-right').changeLayout(Ext.getCmp('sb-layout-tabpanel-tab3-sidebar-right'));

		if (this.getColumns() == 2) {
			var tab2Id = 'sb-layout-tabpanel__' + Ext.getCmp('sb-layout-tabpanel').get(1).id;
			Ext.get(tab2Id).toggleClass('sb-editor-layout-panel-2col-left-tab');
			Ext.get(tab2Id).toggleClass('sb-editor-layout-panel-2col-right-tab');
		}
		this.onUpdateDialog();
	},

	/**
	 * Lock width layout
	 * @param {Boolean} lock
	 * @return {Sb.editor.LayoutPanel} this
	 */
	setLockLayoutWidth: function(lock) {
		this.lockWidth = lock;
		this.onUpdateDialog();
		return this;
	},

	/**
	 * Check layout width is lock
	 * @return {Boolean} State of lock width or not
	 */
	isLockLayoutWidth: function() {
		return this.lockWidth;
	},

	/**
	 * Set selected margin type
	 * @param {String} type
	 * @return {Sb.editor.LayoutPanel} this
	 */
	setMarginType: function(type) {
		this.marginType = type;
		this.onUpdateDialog();
		return this;
	},

	/**
	 * Get selected margin type
	 * @return {String}
	 */
	getMarginType: function() {
		return this.marginType;
	},

	/**
	 * Set margin for selected margin type
	 * @param {Number} width
	 * @param {String} type
	 * @return {Sb.editor.LayoutPanel} this
	 */
	setMarginWidth: function(width, type) {
		if (width === undefined) {
			return this;
		}
		width = Math.abs(width);
		if (type == 'vertical') {
			if (this.designProperties.layout.verticalMargin == width) {
				return this;
			}
			this.designProperties.layout.verticalMargin = width;
		}
		if (type == 'horizontal') {
			if (this.designProperties.layout.horizontalMargin == width) {
				return this;
			}
			this.designProperties.layout.horizontalMargin = width;
		}
		Sb.Editor.model.design.unserialize(this.designProperties);
		this.onUpdateDialog();
		return this;
	},

	/**
	 * Get margin for margin type
	 * @param {String} type
	 * @return {Number}
	 */
	getMarginWidth: function(type) {
		if (type == 'vertical') {
			return this.designProperties.layout.verticalMargin;
		}
		if (type == 'horizontal') {
			return this.designProperties.layout.horizontalMargin;
		}
		return 0;
	},

	/**
	 * Get localized text for unit of measurement of width
	 * @return {String}
	 */
	getUnitMeasurement: function() {
		return (this.designProperties.layout.type == 'elastic') ? '%' : SbGetLocaleByKey('px');
	},

	/**
	 * Set layout type
	 * @param {String} type
	 * @return {Sb.editor.LayoutPanel} this
	 */
	setLayoutType: function(type) {
		if (this.designProperties.layout.type == type) {
			return this;
		}

		var bodyWidth = Ext.getBody().getWidth();
		width = (type == 'elastic')
			? this.getWrapperWidth() * 100 / bodyWidth
			: Math.round(this.getWrapperWidth() * bodyWidth / 100);

		this.designProperties.layout.type = type;
		Sb.Editor.model.design.unserialize(this.designProperties);
		this.setLayoutWidth(width);
		return this;
	},

	/**
	 * Get layuot type
	 * @return {String}
	 */
	getLayoutType: function() {
		return this.designProperties.layout.type;
	},

	/**
	 * Set width for layout
	 * @param {Number} width
	 * @return {Sb.editor.LayoutPanel} this
	 */
	setLayoutWidth: function(width) {
		if (width === undefined || this.designProperties.layout.width == width) {
			return this;
		}
		this.designProperties.layout.width = width;
		Sb.Editor.model.design.unserialize(this.designProperties);
		this.onUpdateDialog();
		return this;
	},

	/**
	 * Get width of watermark
	 * @return {Number}
	 */
	getWatermarkWidth: function() {
		var externalBordersWidth = this.externalBordersWidth;
		if (this.getLayoutType() == 'elastic') {
			externalBordersWidth = this.externalBordersWidth * 100 / Ext.getBody().getWidth();
		}
		return this.getWrapperWidth() - externalBordersWidth;
	},

	/**
	 * Get width of wrapper
	 * @return {Number}
	 */
	getWrapperWidth: function() {
		return this.designProperties.layout.width;
	},

	/**
	 * Set width for first sidebar
	 * @param {Number} width
	 * @return {Sb.editor.LayoutPanel} this
	 */
	setSidebarFirstWidth: function(width) {
		if (width === undefined || this.designProperties.layout.sidebarFirstWidth == width || width < 50) {
			return this;
		}
		width = Math.abs(width);
		if (!this.isLockLayoutWidth()) {
			this.designProperties.layout.width = this.getWatermarkWidth() + width - this.getSidebarFirstWidth();
		}
		this.designProperties.layout.sidebarFirstWidth = width;
		Sb.Editor.model.design.unserialize(this.designProperties);
		this.onUpdateDialog();
		return this;
	},

	/**
	 * Get width first sidebar
	 * @return {Number}
	 */
	getSidebarFirstWidth: function() {
		return this.designProperties.layout.sidebarFirstWidth;
	},

	/**
	 * Get width content area in pixel
	 * @return {Number} Return calculated width for content area
	 */
	getContentWidth: function() {
		var firstWidth = this.designProperties.layout.sidebarFirst ? this.getSidebarFirstWidth() : 0;
		var secondWidth = this.designProperties.layout.sidebarSecond ? this.getSidebarSecondWidth() : 0;

		var width = this.getWatermarkWidth();
		var margins = parseInt(this.designProperties.layout.horizontalMargin) * (this.getColumns() - 1);
		this.contentWidth = width - firstWidth - secondWidth - margins;
		if (this.contentWidth < 0) {
			this.contentWidth = 0;
		}
		return Math.round(this.contentWidth);
	},

	/**
	 * Set width for second sidebar
	 * @param {Number} width
	 * @return {Sb.editor.LayoutPanel} this
	 */
	setSidebarSecondWidth: function(width) {
		if (width === undefined || this.designProperties.layout.sidebarSecondWidth == width || width < 50) {
			return this;
		}
		width = Math.abs(width);
		if (!this.isLockLayoutWidth()) {
			this.designProperties.layout.width = this.getWatermarkWidth() + width - this.getSidebarSecondWidth();
		}
		this.designProperties.layout.sidebarSecondWidth = width;
		Sb.Editor.model.design.unserialize(this.designProperties);
		this.onUpdateDialog();
		return this;
	},

	/**
	 * Set minimal height for first sidebar
	 * @param {Number} height
	 * @return {Sb.editor.LayoutPanel} this
	 */
	setSidebarFirstMinHeight: function(height) {
		if (height === undefined || this.designProperties.layout.sidebarFirstMinHeight == height || height < 100) {
			return this;
		}
		height = Math.abs(height);
		this.designProperties.layout.sidebarFirstMinHeight = height;
		Sb.Editor.model.design.unserialize(this.designProperties);
		this.onUpdateDialog();
		return this;
	},

	/**
	 * Set minimal height for second sidebar
	 * @param {Number} height
	 * @return {Sb.editor.LayoutPanel} this
	 */
	setSidebarSecondMinHeight: function(height) {
		if (height === undefined || this.designProperties.layout.sidebarSecondMinHeight == height || height < 100) {
			return this;
		}
		height = Math.abs(height);
		this.designProperties.layout.sidebarSecondMinHeight = height;
		Sb.Editor.model.design.unserialize(this.designProperties);
		this.onUpdateDialog();
		return this;
	},

	/**
	 * Set minimal height for second sidebar
	 * @param {Number} height
	 * @return {Sb.editor.LayoutPanel} this
	 */
	setContentMinHeight: function(height) {
		if (height === undefined || this.designProperties.layout.contentMinHeight == height || height < 100) {
			return this;
		}
		height = Math.abs(height);
		this.designProperties.layout.contentMinHeight = height;
		Sb.Editor.model.design.unserialize(this.designProperties);
		this.onUpdateDialog();
		return this;
	},

	/**
	 * Get width second sidebar
	 * @return {Number}
	 */
	getSidebarSecondWidth: function() {
		return this.designProperties.layout.sidebarSecondWidth;
	},

	/**
	 * Activate handler
	 */
	onActivate: function() {
		this.memento = Sb.Editor.model.design.saveToMemento();
		this.designProperties = Sb.Editor.model.design.serialize();
		this.onUpdateDialog();
		Ext.getCmp('sb-layout-tabpanel').setActiveTab(this.getColumns() - 1);
		Sb.editor.LayoutPanel.superclass.onActivate.apply(this, arguments);
	},

	/**
	 * OK button handler.
	 */
	okHandler: function() {
		if (!this.form.isValid()) {
			this.form.items.each(function(field) {
				if (!field.validate()) {
					var error = this.tmpProperty.columnSizeEdit == 'width' ? 'minColumnWidth' : 'minColumnHeight';
					Sb.Editor.Ui.mainWindow.statusBar.setMessageError(SbGetLocaleByKey(error));
					return;
				}
			}, this);
		} else {
			Sb.editor.LayoutPanel.superclass.okHandler.call(this);
		}
	},

	/**
	 * Save handler.
	 * @param {Function} afterSaveCallbacks Handler, that will be run after save
	 */
	saveHandler: function(afterSaveCallbacks) {
		Sb.Editor.model.design.save({
			success: function() {
				afterSaveCallbacks.success.call();
				Sb.Editor.Ui.mainWindow.statusBar.setMessage(
					SbGetLocaleByKey('messageDesignLayoutSettingsSaved')
				);
			},
			failure: afterSaveCallbacks.failure
		});
	},

	/**
	 * Discard handler
	 */
	discardHandler: function() {
		Sb.Editor.model.design.restoreFromMemento(this.memento);
	},

	/**
	 * Update state of tab 2
	 * this method update tab2 image, show a position of contents
	 */
	onUpdateTab2: function() {
		Ext.getCmp('sb-layout-tabpanel-tab2-sidebar').changeLayout(Ext.getCmp('sb-layout-tabpanel-tab2-sidebar'));
		Ext.getCmp('sb-layout-tabpanel__switch-column2').changeLayout(Ext.getCmp('sb-layout-tabpanel__switch-column2'));

		if (this.getColumns() == 2) {
			var tabPanel = Ext.getCmp('sb-layout-tabpanel');
			var tab2Id = tabPanel.id + '__' + tabPanel.get(1).id;
			Ext.get(tab2Id).removeClass('sb-editor-layout-panel-2col-left-tab');
			Ext.get(tab2Id).removeClass('sb-editor-layout-panel-2col-right-tab');
			Ext.get(tab2Id).addClass('sb-editor-layout-panel-2col-' +
				(this.designProperties.layout.sidebarFirst ?
					this.designProperties.layout.sidebarFirstPosition :
					this.designProperties.layout.sidebarSecondPosition) +
				'-tab');
		}
	},

	/**
	 * Update state of tab 3
	 * this method update tab3 show a position of contents
	 */
	onUpdateTab3: function() {
		Ext.getCmp('sb-layout-tabpanel-tab3-sidebar-left').changeLayout(Ext.getCmp('sb-layout-tabpanel-tab3-sidebar-left'));
		Ext.getCmp('sb-layout-tabpanel-tab3-sidebar-right').changeLayout(Ext.getCmp('sb-layout-tabpanel-tab3-sidebar-right'));
		Ext.getCmp('sb-layout-tabpanel__switch-column3').changeLayout(Ext.getCmp('sb-layout-tabpanel__switch-column3'));
	},

	/**
	 * call when tab was switched
	 */
	onTabChange: function(tabPanel, newTab) {
		Ext.get(tabPanel.id + '-tab1')[newTab.columns == 1 ? 'addClass' : 'removeClass']('sb-editor-1col-layout-panel');
		tabPanel.items.each(function(tab, index) {
			tab[index == 0 ? 'removeClass' : 'addClass']('x-hide-display');
		});
		this.onUpdateTab2();
		this.onUpdateTab3();
		this.onUpdateDialog();
	},

	/**
	 * Call befor switch tab
	 */
	onBeforeTabChange: function(tabPanel, newTab, currentTab) {
		switch (newTab.columns) {
			case 1:
				this.designProperties.layout.sidebarFirst = false;
				this.designProperties.layout.sidebarSecond = false;
				break;
			case 2:
				//switch from 3 => 2 columns
				//all sidebars has content
				if (this.designProperties.layout.sidebarFirst &&
					this.designProperties.layout.sidebarSecond &&
					this.isContentInSidebar('first') &&
					this.isContentInSidebar('second')) {
					this.showDialog(tabPanel, newTab, currentTab);
					return false;
				}

				var tabDoll = Ext.get(tabPanel.id + '__' + newTab.id).hasClass('sb-editor-layout-panel-2col-left-tab') ?
					'left' : 'right';

				//switch from 3 => 2 columns
				//only one of the sidebars has content
				if (this.designProperties.layout.sidebarFirst &&
					this.designProperties.layout.sidebarSecond &&
					(this.isContentInSidebar('first') || this.isContentInSidebar('second'))) {

					//column which should be hide
					this.tmpProperty.sidebarFirst = this.isContentInSidebar('first');
					this.tmpProperty.sidebarSecond = this.isContentInSidebar('second');

					this.designProperties.layout.sidebarFirstPosition = this.tmpProperty.sidebarFirst ?
						tabDoll :
						(tabDoll == 'left' ? 'right' : 'left');
					this.designProperties.layout.sidebarSecondPosition = this.tmpProperty.sidebarSecond ?
						tabDoll :
						(tabDoll == 'left' ? 'right' : 'left');
					this.designProperties.layout.sidebarFirst = this.tmpProperty.sidebarFirst;
					this.designProperties.layout.sidebarSecond = this.tmpProperty.sidebarSecond;
					break;
				}

				if (currentTab && currentTab.columns != 3 ||
					(currentTab && currentTab.columns == 3 &&
					!this.isContentInSidebar('first') &&
					!this.isContentInSidebar('second'))) {
					this.tmpProperty.sidebarFirst = this.designProperties.layout.sidebarFirstPosition == tabDoll;
					this.tmpProperty.sidebarSecond = this.designProperties.layout.sidebarSecondPosition == tabDoll;
				}

				this.designProperties.layout.sidebarFirst = this.tmpProperty.sidebarFirst;
				this.designProperties.layout.sidebarSecond = this.tmpProperty.sidebarSecond;
				break;
			case 3:
				this.designProperties.layout.sidebarFirst = true;
				this.designProperties.layout.sidebarSecond = true;
				break;
		}
		Sb.Editor.model.design.unserialize(this.designProperties);
	},

	/**
	 * Action for hide left or right sidebar after switching 3 to 2 columns
	 */
	yesButtonClicked: function() {
		if (Ext.getCmp('idSelectSideBar').getValue().getGroupValue() == 'hideRight') {
			this.tmpProperty.sidebarFirst = this.designProperties.layout.sidebarFirstPosition == 'left';
		} else {
			this.tmpProperty.sidebarFirst = this.designProperties.layout.sidebarFirstPosition == 'right';
		}
		this.tmpProperty.sidebarSecond = !this.tmpProperty.sidebarFirst;
		this.designProperties.layout.sidebarFirst = this.tmpProperty.sidebarFirst;
		this.designProperties.layout.sidebarSecond = this.tmpProperty.sidebarSecond;
		Sb.Editor.model.design.unserialize(this.designProperties);
		Ext.getCmp('sb-layout-tabpanel').setActiveTab(1);
		this.dialog.destroy();
	},

	/**
	 * show dialog when switch from 3 to 2 columns
	 */
	showDialog: function(tabPanel, newTab, currentTab) {
		this.dialog = new Ext.Window({
			manager:	Sb.Editor.windowManager.getManager(9),
			renderTo:	Ext.getBody(),
			modal:		true,
			constrain:	true,
			draggable:	false,
			resizable:	false,
			closable:	false,
			shadow:		false,
			stateful:	false,
			shim:		true,
			buttonAlign:'center',
			width:		400,
			cls:		'x-window-dlg sb-layout-dialog-switch',
			plain:		true,
			items : 	[{
				xtype: 'box',
				autoEl: 'p',
				html: SbGetLocaleByKey('switchFrom3To2Columns')
			}, {
				xtype: 'radiogroup',
				id:	'idSelectSideBar',
				columns: 1,
				defaults: {
					xtype: 'radio',
					name: 'selectSideBar'
				},
				items: [{
					boxLabel: (this.designProperties.layout.sidebarFirstPosition == 'left' ?
						SbGetLocaleByKey('sidebar1Sidebar2Hidden') :
						SbGetLocaleByKey('sidebar2Sidebar1Hidden')),
					inputValue: 'hideRight',
					itemCls: 'sb-layout-dialog-sidebar' + (this.designProperties.layout.sidebarFirstPosition == 'left' ?
						'1-2hidden' :
						'2-1hidden') + ' ' +
						(Ext.get(tabPanel.id + '__' + newTab.id).hasClass('sb-editor-layout-panel-2col-left-tab') ?
							'sb-layout-dialog-sidebar' + (this.designProperties.layout.sidebarFirstPosition == 'left' ?
								'1-2hidden' :
								'2-1hidden') + '-selected' : ''),
					checked: Ext.get(tabPanel.id + '__' + newTab.id).hasClass('sb-editor-layout-panel-2col-left-tab'),
					listeners: {
						check: function(radio, checked) {
							var cls = 'sb-layout-dialog-sidebar' + (this.designProperties.layout.sidebarFirstPosition == 'left' ?
								'1-2hidden' :
								'2-1hidden');
							var wrapperEl = radio.getEl().up('.' + cls);
							if (checked) {
								wrapperEl.addClass(cls + '-selected');
							} else {
								wrapperEl.removeClass(cls + '-selected');
							}
						}.createDelegate(this)
					}
				}, {
					boxLabel: (this.designProperties.layout.sidebarFirstPosition == 'left' ?
						SbGetLocaleByKey('sidebar2Sidebar1Hidden') :
						SbGetLocaleByKey('sidebar1Sidebar2Hidden')),
					inputValue: 'hideLeft',
					itemCls: 'sb-layout-dialog-sidebar' + (this.designProperties.layout.sidebarFirstPosition == 'left' ?
							'1hidden-2' :
							'2hidden-1') + ' ' +
							(Ext.get(tabPanel.id + '__' + newTab.id).hasClass('sb-editor-layout-panel-2col-right-tab') ?
								'sb-layout-dialog-sidebar' + (this.designProperties.layout.sidebarFirstPosition == 'left' ?
									'1hidden-2' :
									'2hidden-1') + '-selected' : ''),
					checked: Ext.get(tabPanel.id + '__' + newTab.id).hasClass('sb-editor-layout-panel-2col-right-tab'),
					listeners: {
						check: function(radio, checked) {
							var cls = 'sb-layout-dialog-sidebar' + (this.designProperties.layout.sidebarFirstPosition == 'left' ?
								'1hidden-2' :
								'2hidden-1');
							var wrapperEl = radio.getEl().up('.' + cls);
							if (checked) {
								wrapperEl.addClass(cls + '-selected');
							} else {
								wrapperEl.removeClass(cls + '-selected');
							}
						}.createDelegate(this)
					}
				}]
			}],
			footer:		true,
			onEsc:		function() {
				this.dialog.destroy();
			}.createDelegate(this),
			buttonAlign: 'center',
			fbar: [{
				iconCls: 'sb-ok-icon',
				text: SbGetLocaleByKey('ok'),
				handler: this.yesButtonClicked.createDelegate(this)
			}, {
				iconCls: 'sb-cancel-icon',
				text: SbGetLocaleByKey('cancelButtonTitle'),
				handler: function() {
					this.dialog.destroy();
				}.createDelegate(this)
			}]
		});
		this.dialog.show();
	}
});
Ext.reg('layoutpanel', Sb.editor.LayoutPanel);