/**
 * @class Sb.editor.MainWindow
 * @extends Ext.Window
 */
Sb.editor.MainWindow = Ext.extend(Ext.Window, {
	// private
	initComponent: function() {

		/**
		 * Link to status bar
		 * @type Sb.editor.StatusBar
		 * @property statusBar
		 */
		this.statusBar = new Sb.editor.StatusBar();

		/**
		 * How many pixels of window will be always visible in X axis
		 * @type Number
		 * @property visibleOffsetX
		 */
		this.visibleOffsetX = 45;

		/**
		 * How many pixels of window will be always visible in Y axis
		 * @type Number
		 * @property visibleOffsetY
		 */
		this.visibleOffsetY = 27;

		/**
		 * Permanent helper message
		 * @type String
		 * @property permanentHelperMessage
		 */
		this.permanentHelperMessage = '';

		/**
		 * Helper message height
		 * @type Int
		 * @property helperMessageHeight
		 */
		this.helperMessageHeight = 0;

		Ext.apply(this, {
			baseCls: 'sb-editor-panel',
			manager: Sb.Editor.windowManager.getManager(2),
			title: SbGetLocaleByKey('productName') + ' ' + Sb.Config.version,
			width: 626,
			resizable: false,
			closable: false,
			shadow: false,
			modal: true,
			items: {
				xtype: 'tabpanel',
				headerCfg: {
					cls: 'sb-editor-main-navigation'
				},
				border: false,
				items: [{
					xtype: 'widgetspanel',
					tabCls: 'sb-editor-widgets-tab',
					items: [
						'text',
						'-',
						'navigation',
						'sharedText',
						'header'
					]
				}, {
					xtype: 'layoutpanel',
					tabCls: 'sb-editor-layout-tab'
				}, {
					xtype: 'stylepanel',
					tabCls: 'sb-editor-style-tab'
				}, {
					xtype: 'pagespanel',
					tabCls: 'sb-editor-pages-tab'
				}, {
					xtype: 'settingspanel',
					tabCls: 'sb-editor-settings-tab'
				}],
				onStripMouseDown: function(e) {
					if (e.button !== 0) {
						return;
					}
					e.preventDefault();
					var t = this.findTargets(e);
					if (!t.item) {
						return;
					}
					if (t.item != this.activeTab) {
						this.setActiveTab(t.item);
					} else {
						this.setActiveTab(null);
					}
				},
				listeners: {
					scope: this,
					tabchange: this.onTabChange,
					beforetabchange: this.onBeforeTabChange
				}
			},
			tbar: this.statusBar,
			bbar: [{
				xtype: 'tbtext',
				cls: 'sb-helper-text'
			}],
			buttonAlign: 'left',
			fbar: [{
				text: SbGetLocaleByKey('gettingStarted'),
				tooltip: SbGetLocaleByKey('gettingStartedDescription'),
				handler: Sb.Editor.showGettingStarted,
				hidden: !Sb.Config.gettingStartedDemoEnabled
			}, {
				xtype: 'tbseparator',
				hidden: !Sb.Config.gettingStartedDemoEnabled
			}, {
				id: 'sb-editor-help-btn',
				text: SbGetLocaleByKey('help'),
				tooltip: SbGetLocaleByKey('helpDescription'),
				handler: function () {
					var articleUrl		= '';
					var activeWidget	= Sb.widget.Widget.editCurrent;
					if (null != activeWidget) {
						// Active widget
						articleUrl		= 'Widgets/' + activeWidget.xtype.toUpperCaseFirst();
					} else {
						// Panel and it`s tabs
						articleUrl		= 'Panel';
						var activeTab	= this.tabs.getActiveTab();
						if (null != activeTab) {
							var activeTabKey	= activeTab.itemId;
							if (null != activeTabKey) {
								articleUrl		+= '/' + activeTabKey.toUpperCaseFirst();
							}
							if (null != activeTab.tabs) {
								activeTabKey	= activeTab.tabs.getActiveTab().itemId;
								if (null != activeTabKey) {
									articleUrl	+= '/' + activeTabKey.toUpperCaseFirst();
								}
							}
						}
					}
					Sb.Editor.openHelp(articleUrl);
				}.createDelegate(this)
			}, '-', {
				text: SbGetLocaleByKey('startOver'),
				tooltip: SbGetLocaleByKey('startOverDescription'),
				handler: function() {
					Sb.editor.UiContext.tryToSwitch(
						function() {
							Sb.Editor.Ui.mainWindow.tabs.setActiveTab(null);
							Ext.Msg.show({
								title    : SbGetLocaleByKey('titleConfirmStartingOver'),
								msg      : SbGetLocaleByKey('msgConfirmStartingOver'),
								closable : false,
								buttons  : Ext.Msg.YESNO,
								fn       : function(btn) {
									if (btn == 'yes') {
										Sb.Loading.updateMessage(SbGetLocaleByKey('loaderMsgStartingOver'));
										Sb.Loading.show();
										Sb.SiteService.backToStub(function() {
											window.location = Sb.Config.baseUrl + '/Start';
										});
									}
								}
							});
						}
					);
				}
			}, '->', {
				id: 'sb-editor-minimize-btn',
				iconCls: 'sb-editor-minimize-btn-icon',
				text: SbGetLocaleByKey('hidePanel').toUpperCase(),
				tooltip: SbGetLocaleByKey('hidePanelDescription'),
				scope: this,
				handler: this.toggleMinimize
			}]
		});

		Ext.each(this.items.items, function(item) {
			if (item.xtype) {
				item.itemId = item.xtype.replace('panel', '');
			}
		});

		Sb.editor.MainWindow.superclass.initComponent.apply(this, arguments);

		/**
		 * Main navigation
		 * @type Ext.TabPanel
		 * @property tabs
		 */
		this.tabs = this.items.item(0);

		// Override quick tips to show it in helper
		Ext.override(Ext.QuickTip, {
			show: function() {
				Sb.Editor.Ui.mainWindow.setHelperMessage(this.activeTarget.text);
			},
			delayShow: function() {
				this.show();
			},
			hide: function() {
				if (!this.activeTarget) {
					return;
				}
				Sb.Editor.Ui.mainWindow.restoreHelperMessage();
				delete this.activeTarget;
			},
			delayHide: function() {
				this.hide();
			}
		});
	},

	// private
	onRender: function(ct, position) {
		Sb.editor.MainWindow.superclass.onRender.call(this, ct, position);

		this.publishBtn = new Ext.Button({
			renderTo: this.header,
			cls: 'sb-btn-action',
			iconCls: 'sb-publish-icon',
			text: SbGetLocaleByKey('publish'),
			tooltip: SbGetLocaleByKey('publishDescription'),
			scope: this,
			handler: function() {
				this.publishSite();
			}
		});
	},

	// private
	afterRender: function() {
		Sb.editor.MainWindow.superclass.afterRender.call(this);

		if (Ext.isIE7) {
			this.tabs.body.dom.innerHTML = '<span />';
		}

		var tabsCount = this.tabs.items.length;
		var	tabElements = this.tabs.el.select('li:not(.x-tab-edge)');
		var firstTabId = tabElements.first().getAttribute('id');
		var lastTabId = tabElements.last().getAttribute('id');

		var tabsMargins = -20 * (tabsCount - 1); // 4 tabs by -20px margin-left
		var delta = this.tabs.header.dom.offsetWidth - tabsMargins + 12; //12 - width of shadows of side tabs to hide
		tabElements.each(function(tab, index) {
			delta -= tab.getWidth();
		});
		tabElements.each(function(tab) {
			var tab = Ext.get(tab.id);
			tab.setStyle({
				width: (tab.getWidth() + delta / tabsCount) + 'px',
				marginLeft: '-20px'
			});

			tab.on('mouseenter', function(e) {
				var target = e.getTarget('li');

				if (target.id == firstTabId) {
					tabElements.setStyle({
						marginLeft: '-22px'
					});
					target.style.marginRight = '8px';
				} else if (target.id == lastTabId) {
					tabElements.setStyle({
						marginLeft: '-22px'
					});
					target.style.marginLeft = '-14px';
				} else {
					tabElements.setStyle({
						marginLeft: '-23px',
						marginRight: 0
					});
					target.style.marginLeft = '-16px';
					target.style.marginRight = '5px';
				}
			}, this);

			tab.on('mouseleave', function(e) {
				tabElements.setStyle({
					marginLeft: '-20px',
					marginRight: 0
				});
			}, this);

			tab.on('mousedown', function(e) {
				tabElements.setStyle({
					marginLeft: '-20px',
					marginRight: 0
				});
			}, this);
		}, this);

		this.tabs.items.each(function(tab) {
			tab.un('titlechange', this.tabs.onItemTitleChanged, this.tabs);
		}, this);

		this.mask.setOpacity(0);
		this.alignTo(Ext.getBody(), 'tr-tr', [11, -7]);

		this.clearHelperMessage();

		this.mask.on('click', function() {
			this.tabs.setActiveTab(null);
		}, this);

		/**
		 * Width for restoring
		 * @type Number
		 * @property restoreWidth
		 */
		this.restoreWidth = this.getWidth();
	},

	beforeShow: function() {
		Sb.editor.MainWindow.superclass.beforeShow.call(this);
		this.mask[this.tabs.getActiveTab() instanceof Sb.editor.BaseFormPanel ? 'show' : 'hide']();
	},

	/**
	 * Shortcut for performing an {@link #minimize} or {@link #restore} based on the current state of the panel.
	 * @return {Sb.editor.MainWindow} this
	 */
	toggleMinimize: function() {
		this[this.minimized ? 'restore' : 'minimize']();
		return this;
	},

	/**
	 * Minimize a panel.
	 * @return {Sb.editor.MainWindow} this
	 */
	minimize: function() {
		if (!Sb.editor.UiContext.isMayBeSwitchedImmediately()) {
			Sb.editor.UiContext.tryToSwitch(this.minimize.createDelegate(this));
			return this;
		}

		this.tabs.setActiveTab(null);
		this.minimized = true;
		this.addClass('sb-editor-panel-minimized');
		this.restoreWidth = this.getWidth();
		this.setWidth(this.header.child('.sb-editor-panel-header-text').getWidth() + 60);
		this.setPosition(this.getPosition()[0] + this.restoreWidth - this.getWidth());
		Ext.getCmp('sb-editor-minimize-btn').setText(SbGetLocaleByKey('showPanel').toUpperCase());

		this.fireEvent('minimize', this);
		return this;
	},

	/**
	 * Restores a panel back to its original size and position prior to being minimized.
	 * @return {Sb.editor.MainWindow} this
	 */
	restore: function() {
		this.removeClass('sb-editor-panel-minimized');
		this.minimized = false;
		this.setPosition(this.getPosition()[0] - this.restoreWidth + this.getWidth());
		this.setWidth(this.restoreWidth);
		Ext.getCmp('sb-editor-minimize-btn').setText(SbGetLocaleByKey('hidePanel').toUpperCase());

		this.doConstrain(true);
		this.fireEvent('restore', this);
		return this;
	},

	/**
	 * Fires before the active tab changes. Handlers return false in order to cancel the tab change.
	 * @param {TabPanel} tabPanel Active tabpanel
	 * @param {Panel} newTab The tab being activated
	 * @param {Panel} currentTab The current active tab
	 */
	onBeforeTabChange: function(tabPanel, newTab, currentTab) {
		if (Sb.editor.UiContext.isMayBeSwitchedImmediately()) {
			return true;
		}
		Sb.editor.UiContext.tryToSwitch(tabPanel.setActiveTab.createDelegate(tabPanel, [newTab]));
		return false;
	},

	/**
	 * Fires after the active tab has changed.
	 * @param {TabPanel} tabPanel
	 * @param {Panel} tab
	 */
	onTabChange: function(tabPanel, tab) {
		this.mask[tab instanceof Sb.editor.BaseFormPanel ? 'show' : 'hide']();

		this.setHelperMessageHeight(0, true);

		if (tab) {
			this.statusBar.clear();
			if (tab.hidden) {
				tab.show();
			}
			if (tab.helperMessage) {
				this.setHelperMessage(tab.helperMessage, true);
			}
		} else {
			tabPanel.layout.activeItem.hide();
			tabPanel.layout.activeItem = null;
			this.clearHelperMessage();
		}

		this.restore();
	},

	/**
	 * Publish site.
	 */
	publishSite: function() {
		if (!Sb.editor.UiContext.isMayBeSwitchedImmediately()) {
			Sb.editor.UiContext.tryToSwitch(this.publishSite.createDelegate(this));
			return;
		}

		this.tabs.setActiveTab(null);
		var mask = Ext.getBody().mask();
		mask.addClass('sb-editor-mask');
		this.statusBar.clear();

		if (!this.progressBar || this.progressBar.isDestroyed) {
			this.progressBar = new Sb.editor.MainWindow.FakeProgressBar({
				renderTo: this.getTopToolbar().container
			});
			this.getTopToolbar().add(this.progressBar);
		}

		this.progressBar.wait({
			duration: 10000,
			increment: 80,
			text: String.format(SbGetLocaleByKey('publishingSite'), '<b>0%</b>')
		});

		Sb.SiteService.publish(function(data, e) {
			var count = Sb.Editor.model.rootPage.pages.getCount();
			Sb.Editor.model.rootPage.pages.each(function(page) {
				if (page.data.hidden) {
					count--;
				}
			});
			this.progressBar.gotoEnd({
				fn: function() {
					this.progressBar.destroy();
					mask.remove();
					var message = '', errLevel = '';
					if (data) {
						switch (data._statusLevel) {
							case 0:
							case 1:
								message = SbGetLocaleByKey('siteSuccessfullyPublished') + ' '
									+ '<a href="' + Sb.Editor.model.data.publishWebSiteUrl + '" target="_blank">'
									+ Sb.Editor.model.data.publishWebSiteUrl + '</a>';
								break;
							case 2:
								this.progressBar.publishResult = data;
								message = data._statusText + ' '
									+ '<a href="#" onclick="javascript: Sb.Editor.showPublishErrorDetails();">'
									+ SbGetLocaleByKey('publishSeeDetails') + '</a>';
								errLevel = 'Error';
								break;
							case 3:
								message = SbGetLocaleByKey('sitePublishedWithErrors');
								errLevel = 'Warning';
								break;
						}
						Sb.Editor.Ui.mainWindow.statusBar['setMessage' + errLevel](message);
					}
				},
				scope: this
			});
		}, this);
	},

	/**
	 * Update helpers's message.
	 * @param {String} msg The text to display in helper.
	 * @param {Boolean} isPermanent Is the text will be permanent?
	 */
	setHelperMessage: function(msg, isPermanent) {
		this.setHelperMessageHeight(0);

		if (isPermanent) {
			this.permanentHelperMessage = msg;
		}
		this.bottomToolbar.items.items[0].setText(msg);

		var height = this.getHelperMessageHeight();
		if (height > this.helperMessageHeight) {
			this.helperMessageHeight = height;
		}
		this.setHelperMessageHeight(this.helperMessageHeight);
	},

	/**
	 * Restore helper message from previous message
	 */
	restoreHelperMessage: function() {
		this.setHelperMessage(this.permanentHelperMessage);
	},

	/**
	 * Clear helper's message.
	 */
	clearHelperMessage: function() {
		this.permanentHelperMessage = SbGetLocaleByKey('helperDefaultMessage');
		this.setHelperMessage(this.permanentHelperMessage);
	},

	/**
	 * Get helpers's message area height.
	 * @return {Int} Height of message area.
	 */
	getHelperMessageHeight: function() {
		return this.bottomToolbar.el.child('table').getHeight();
	},

	/**
	 * Set helpers's message area height.
	 * @param {Int} height Height of message area.
	 * @param {Bool} setModelHeight If set height to model height too.
	 */
	setHelperMessageHeight: function(height, setModelHeight) {
		if (setModelHeight == true) {
			this.helperMessageHeight = height;
		}
		this.bottomToolbar.el.child('table').setHeight((height == 0) ? '' : height);
	},

	initDraggable: function() {
		this.dd = new Sb.editor.MainWindow.DD(this);
	},

	/**
	 * Make editor panel always visible, even after browser resize
	 *
	 * @param {Boolean} isWholeWindowVisible Do we force to show whole window on screen?
	 */
	doConstrain: function(isWholeWindowVisible) {
		if (undefined == isWholeWindowVisible) {
			isWholeWindowVisible = false;
		}

		var containerSize		= this.container.getSize();
		var windowSize			= this.getSize();
		var windowPosition		= newWindowPosition = this.getPosition();
		var delta				= {
			x			: containerSize.width - (isWholeWindowVisible ? windowSize.width : this.visibleOffsetX),
			y			: containerSize.height - (isWholeWindowVisible ? windowSize.height : this.visibleOffsetY)
		};
		delta.x					= delta.x < 0 ? 0 : delta.x;
		delta.y					= delta.y < 0 ? 0 : delta.y;

		var distanceToVisible	= {
			fromRight	: windowPosition[0] - delta.x,
			fromBottom	: windowPosition[1] - delta.y
		};

		var isNeedToMove = {
			toLeft		: (distanceToVisible.fromRight > 0),
			toTop		: (distanceToVisible.fromBottom > 0)
		};

		if (isNeedToMove.toLeft || isNeedToMove.toTop) {
			if (isNeedToMove.toLeft) {
				newWindowPosition[0] = delta.x;
			}
			if (isNeedToMove.toTop) {
				newWindowPosition[1] = delta.y;
			}
			this.setPosition(newWindowPosition);
		}
	},

	createGhost: function() {
		return this.container.createChild({
			cls: 'x-panel-ghost sb-editor-panel-ghost', cn: {}
		}).setSize(this.getSize());
	}
});

/**
 * Class for main window drag & drop
 * @class Sb.editor.MainWindow.DD
 * @extends Ext.dd.DD
 */
Sb.editor.MainWindow.DD = Ext.extend(Ext.Window.DD, {
	startDrag: function(){
		this.proxy = this.win.ghost();
		var windowSize = this.win.el.getSize();
		var widthOffset = -(windowSize.width - this.win.visibleOffsetX);
		var heightOffset = -(windowSize.height - this.win.visibleOffsetY);
		var headerSize = this.win.header.getSize();
		this.constrainTo(this.win.container, {
			top: -(headerSize.height - this.win.visibleOffsetY),
			bottom: heightOffset,
			left: widthOffset,
			right: widthOffset
		});
	}
});
