/**
 * @class Sb.editor.PagesTreeNodeUI
 * @extends Ext.tree.TreeNodeUI
 */
Sb.editor.PagesTreeNodeUI = Ext.extend(Ext.tree.TreeNodeUI, {
	// override parent implementation
	renderElements: function(n, a, targetNode, bulkRender) {
		this.indentMarkup = n.parentNode ? n.parentNode.ui.getChildIndent() : '';

		var cb = Ext.isBoolean(a.checked),
			nel,
			href = a.href ? a.href : Ext.isGecko ? '' : '#',
			buf = ['<li class="x-tree-node"><div ext:tree-node-id="', n.id, '" class="x-tree-node-el x-tree-node-leaf x-unselectable ', a.cls, '" unselectable="on">',
			'<div class="x-tree-node-el-inner">',
			'<span class="x-tree-node-indent">', this.indentMarkup, '</span>',
			'<img src="', this.emptyIcon, '" class="x-tree-ec-icon x-tree-elbow" />',
			'<img src="', a.icon || this.emptyIcon, '" class="x-tree-node-icon', (a.icon ? " x-tree-node-inline-icon" : ""), (a.iconCls ? " "+a.iconCls : ""), '" unselectable="on" />',
			cb ? ('<input class="x-tree-node-cb" type="checkbox" ' + (a.checked ? 'checked="checked" />' : '/>')) : '',
			'<a hidefocus="on" class="x-tree-node-anchor" href="', href, '" tabIndex="1" ',
			a.hrefTarget ? ' target="' + a.hrefTarget + '"' : '', '><span unselectable="on">',
			Ext.util.Format.htmlEncode(Ext.util.Format.ellipsis(n.text, 35 - n.getDepth() * 2)), '</span></a>',
			'<div class="sb-pages-tree-node-controls">',
			'<div class="sb-pages-tree-node-control-up"></div>',
			'<div class="sb-pages-tree-node-control-right"></div>',
			'<div class="sb-pages-tree-node-control-down"></div>',
			'<div class="sb-pages-tree-node-control-left"></div>',
			'</div>',
			'</div></div>',
			'<ul class="x-tree-node-ct" style="display:none;"></ul>',
			'</li>'].join('');

		if (bulkRender !== true && n.nextSibling && (nel = n.nextSibling.ui.getEl())) {
			this.wrap = Ext.DomHelper.insertHtml("beforeBegin", nel, buf);
		}else{
			this.wrap = Ext.DomHelper.insertHtml("beforeEnd", targetNode, buf);
		}

		this.elNode = this.wrap.childNodes[0];
		this.ctNode = this.wrap.childNodes[1];
		var cs = this.elNode.firstChild.childNodes;
		this.indentNode = cs[0];
		this.ecNode = cs[1];
		this.iconNode = cs[2];
		var index = 3;
		if (cb) {
			this.checkbox = cs[3];
			// fix for IE6
			this.checkbox.defaultChecked = this.checkbox.checked;
			index++;
		}
		this.anchor = cs[index];
		this.textNode = cs[index].firstChild;

		this.ctrls = {};
		this.ctrls.up = Ext.fly(cs[index + 1]).first();
		this.ctrls.right = this.ctrls.up.next();
		this.ctrls.down = this.ctrls.right.next();
		this.ctrls.left = this.ctrls.down.next();
		this.controlsWrapper = this.ctrls.up.parent();

		Ext.iterate(this.ctrls, function(ctrlName, el) {
			Ext.QuickTips.register({
				target: el, text: SbGetLocaleByKey('movePageTooltip')
			});
			el.on('click', this.onControlClick.createDelegate(this, [ctrlName]), this, {stopPropagation: true});
			el.on('dblclick', function (e) {
				e.stopPropagation();
			});
		}, this);

		this.newPos = {
			up: null,
			right: null,
			down: null,
			left: null
		};
	},

	/**
	 * Get drag-n-drop handles, that will allow drag of this node.
	 */
	getDDHandles : function(){
		return [this.ecNode, this.iconNode, this.textNode, this.elNode.firstChild];
	},

	onSelectedChange: function(state) {
		if (state) {
			this.addClass('x-tree-selected');
			this.node.getOwnerTree().body.scrollChildIntoView(this.controlsWrapper);
			this.focus();

			// check left direction
			if (this.node.getDepth() > 1) {
				this.newPos.left = {
					parentNode: this.node.parentNode.parentNode,
					refNode: this.node.parentNode.nextSibling
				};
			} else {
				this.newPos.left = null;
			}

			// check up direction
			if (this.node.previousSibling) {
				this.newPos.up = {
					parentNode: this.node.previousSibling.parentNode,
					refNode: this.node.previousSibling
				};
			} else if (this.node.getDepth() > 1 && this.node.parentNode.previousSibling) {
				this.newPos.up = {
					parentNode: this.node.parentNode.previousSibling,
					refNode: null
				};
			} else {
				this.newPos.up = null;
			}

			// check right direction
			if (this.node.getDepth() < this.node.attributes.maxDepth
				&& this.node.previousSibling && !this.node.childNodes.length) {
				this.newPos.right = {
					parentNode: this.node.previousSibling,
					refNode: null
				};
			} else {
				this.newPos.right = null;
			}

			// check down direction
			if (this.node.nextSibling) {
				this.newPos.down = {
					parentNode: this.node.nextSibling.parentNode,
					refNode: this.node.nextSibling.nextSibling
				};
			} else if (this.node.getDepth() > 1 && this.node.parentNode.nextSibling) {
				this.newPos.down = {
					parentNode: this.node.parentNode.nextSibling,
					refNode: this.node.parentNode.nextSibling.firstChild
				};
			} else {
				this.newPos.down = null;
			}

			Ext.iterate(this.newPos, function(dir, pos) {
				this.ctrls[dir][(null === pos ? 'add' : 'remove') + 'Class']('sb-pages-tree-node-control-' + dir + '-disabled');
			}, this);
		} else {
			this.removeClass('x-tree-selected');
		}
	},

	onControlClick: function(ctrlName) {
		if (this.newPos[ctrlName]) {
			this.newPos[ctrlName].parentNode.expand();
			this.newPos[ctrlName].parentNode.insertBefore(this.node, this.newPos[ctrlName].refNode);
		}
	}
});
