/**
 * @class Sb.editor.SettingsPanel
 * @extends Sb.editor.BaseFormPanel
 * Settings panel
 * @constructor
 * @param {Object} config The config object
 * @xtype settingspanel
 */
Sb.editor.SettingsPanel = Ext.extend(Sb.editor.BaseFormPanel, {

	faviconFile: null,

	// private
	initComponent: function() {
		var siteInfoTab = {
			itemId: 'siteInfo',
			title: SbGetLocaleByKey('siteInfoTabTitle'),
			cls: 'sb-editor-settings-siteinfo-panel',
			autoHeight: true,
			helperMessage: SbGetLocaleByKey('siteInfoTabHelperMsg'),
			layout: 'form',
			listeners: {
				activate: function() {
					//:INFO: workaround for bug #49418
					Ext.getCmp('settingsPanelSiteDescription').syncSize();
					Ext.getCmp('settingsPanelMetaKeywords').syncSize();
				}
			},
			items: [{
				xtype: 'fieldgroup',
				cls: 'sb-editor-box',
				items: [{
					anchor		: '100%',
					xtype		: 'textfield',
					fieldLabel	: SbGetLocaleByKey('siteName'),
					name		: 'siteName',
					listeners	: {valid: Sb.editor.UiContext.registerChanges}
				}, {
					xtype		: 'checkbox',
					hideLabel	: true,
					boxLabel	: SbGetLocaleByKey('useSiteNameInPages'),
					name		: 'pagesUseName',
					listeners	: {check: Sb.editor.UiContext.registerChanges}
				}, {
					xtype		: 'box',
					cls			: 'sb-editor-box-description',
					html		: SbGetLocaleByKey('siteTitleDescription')
				}]
			}, {
				xtype: 'fieldgroup',
				cls: 'sb-editor-box',
				items: [{
					anchor			: '100%',
					xtype			: 'textarea',
					style			: {'overflow-x' : 'hidden'},
					id				: 'settingsPanelSiteDescription',
					fieldLabel		: SbGetLocaleByKey('siteDescription'),
					height			: 45,
					maxLength		: 255,
					maxLengthText	: SbGetLocaleByKey('siteDescriptionMaxLengthExceeded'),
					msgTarget		: 'under',
					name			: 'siteDescription',
					listeners		: {valid: Sb.editor.UiContext.registerChanges}
				}, {
					xtype		: 'box',
					cls			: 'sb-editor-box-description',
					html		: SbGetLocaleByKey('siteDescriptionDesc')
				}]
			}, {
				xtype: 'fieldgroup',
				cls: 'sb-editor-box',
				items: [{
					anchor			: '100%',
					xtype			: 'textarea',
					id				: 'settingsPanelMetaKeywords',
					style			: {'overflow-x' : 'hidden'},
					fieldLabel		: SbGetLocaleByKey('metaKeywords'),
					height			: 45,
					maxLength		: 255,
					maxLengthText	: SbGetLocaleByKey('metaKeywordsMaxLengthExceeded'),
					msgTarget		: 'under',
					name			: 'metaKeywords',
					listeners		: {valid: Sb.editor.UiContext.registerChanges}
				}, {
					xtype		: 'box',
					cls			: 'sb-editor-box-description',
					html		: SbGetLocaleByKey('metaKeywordsDesc')
				}]
			}, {
				layout: 'column',
				cls: 'sb-editor-box sb-editor-box-last',
				border: false,
				defaults: {border: false},
				items: [{
					xtype: 'fieldgroup',
					width: 70,
					cls: 'sb-editor-settings-icon',
					items: {
						xtype		: 'displayfield',
						fieldLabel	: SbGetLocaleByKey('faviconTitle'),
						id			: 'sb-favicon-preview'
					}
				}, {
					columnWidth: 1,
					autoEl: {tag: 'form'},
					layout: 'form',
					labelAlign: 'top',
					id: 'sb-favicon-form',
					items: {
						xtype		: 'fileuploadfield',
						regex		: /(\.ico$)/i,
						msgTarget	: 'sbtitle',
						anchor		: '100%',
						buttonText	: SbGetLocaleByKey('browseButtonText'),
						fieldLabel	: SbGetLocaleByKey('uploadFavicon'),
						emptyText	: SbGetLocaleByKey('clickToBrowseAndUploadFile'),
						name		: 'faviconfile',
						msgTarget	: 'under',
						listeners	: {
							scope: this,
							fileselected: this.onFaviconSelected
						}
					}
				}, {
					xtype	: 'button',
					text	: SbGetLocaleByKey('Remove'),
					iconCls	: 'sb-delete-icon',
					cls		: 'sb-editor-settings-remove-btn',
					scope	: this,
					handler	: function() {
						Sb.editor.UiContext.registerChanges();
						this.faviconFile = 'remove';
						Ext.getDom('sb-favicon-preview').innerHTML = '';
					}
				}]
			}, {
				xtype	: 'box',
				cls		: 'sb-editor-box-description',
				html	: SbGetLocaleByKey('faviconDesc')
			}]
		};

		// It stronly will be component or design element in future. (c)
		var boxCls = 'sb-editor-box';
		var headTpl = '<div class="{0} {5}"><div class="{0}-title"><a href="{4}" id="{1}">{2}</a></div>' +
			'<div class="{0}-description">{3}</div></div>';
		var seoTab = {
			itemId: 'seo',
			title: SbGetLocaleByKey('seoTabTitle'),
			cls: 'sb-editor-settings-seo-panel',
			helperMessage: SbGetLocaleByKey('seoTabHelperMsg'),
			html: '<p>' + SbGetLocaleByKey('seoTabDescription') + '</p>' +
				String.format(headTpl, boxCls, 'sb-ownership', SbGetLocaleByKey('seoOwnershipTitle'),
					SbGetLocaleByKey('seoOwnershipDesc'), '#') +
				String.format(headTpl, boxCls, 'sb-edit-head', SbGetLocaleByKey('seoEditHeadTitle'),
					SbGetLocaleByKey('seoEditHeadDesc'), '#') +
				String.format(headTpl, boxCls, 'sb-google-analitics', SbGetLocaleByKey('seoGoogleAnTitle'),
					SbGetLocaleByKey('seoGoogleAnDesc'), '#') +
				String.format(headTpl, boxCls, 'sb-download-sitemap', SbGetLocaleByKey('seoDownloadSitemapTitle'),
					SbGetLocaleByKey('seoDownloadSitemapDesc'), Sb.Config.baseUrl + '/DownloadSiteMap', 'sb-editor-box-last'),
			listeners: {
				scope: this,
				afterRender: this.seoTabAfterRender
			}
		};

		Ext.apply(this, {
			cls: 'sb-editor-settings-panel',
			title: SbGetLocaleByKey('settingsTitle'),
			helperMessage: SbGetLocaleByKey('settingsDescription'),
			tabTip: SbGetLocaleByKey('settingsDescriptionOnHover'),
			autoHeight: true,
			items: {
				xtype: 'verticaltabpanel',
				tabWidth: 85,
				tabPosition: 'left',
				activeTab: 0,
				border: false,
				cls: 'sb-editor-settings-panel-tabs',
				items: [siteInfoTab, seoTab],
				defaults: {height: 460}
			},
			listeners: {
				scope: this,
				activate: function() {
					this.form.setValues({
						siteName		: Sb.Editor.model.data.title,
						pagesUseName	: Sb.Editor.model.data.pagesUseName,
						siteDescription	: Sb.Editor.model.data.description,
						metaKeywords	: Sb.Editor.model.data.keywords
					});
					this.faviconFile = Sb.Editor.model.data.faviconFile;
					Ext.getDom('sb-favicon-preview').innerHTML = ('favicon.ico' == this.faviconFile)
						? '<img src="' + Sb.Editor.model.getUrl() +	'/favicon.png?' + (new Date).getTime()+ '">'
						: '';
				}
			}
		});

		Sb.editor.SettingsPanel.superclass.initComponent.apply(this, arguments);
		this.bodyCfg.tag = 'div';

		/**
		 * Inner tab panel
		 * @type Ext.TabPanel
		 * @property tabs
		 */
		this.tabs = this.items.item(0);
	},

	ownershipWinOpen: function () {
		var ownershipWin = new Sb.Window({
			width	: 500,
			title	: SbGetLocaleByKey('seoOwnershipTitle'),
			modal	: true,

			items: {
				xtype		: 'form',
				id			: 'sb-ownership-form',
				fileUpload	: true,
				border		: false,
				defaults	: {layout: 'column', border: false},
				items: [{ //row1
					defaults : {border: false},
					items: [{
						xtype		: 'radio',
						name		: 'ownershipType',
						inputValue	: 'emptyFile',
						hideLabel	: true,
						width		: 20,
						checked		: true
					}, {
						columnWidth: 1,
						layout: 'form',
						labelAlign	: 'top',
						items: [{
							xtype		: 'textfield',
							fieldLabel	: SbGetLocaleByKey('createFile'),
							name		: 'emptyFilename',
							anchor		: '100%',
							msgTarget	: 'under'
						}, {
							xtype		: 'label',
							text		: SbGetLocaleByKey('createFileDescription'),
							cls			: 'sb-editor-box-description'
						}]
					}]
				}, { //row2
					defaults : {border: false},
					style: 'margin-top: 15px',
					items: [{
						xtype		: 'radio',
						name		: 'ownershipType',
						id			: 'sb-ownershipType-customFile',
						inputValue	: 'customFile',
						hideLabel	: true,
						width		: 20
					}, {
						columnWidth: 1,
						layout: 'form',
						labelAlign: 'top',
						items: [{
							xtype			: 'fileuploadfield',
							msgTarget		: 'sbtitle',
							anchor			: '100%',
							buttonText		: SbGetLocaleByKey('browseButtonText'),
							fieldLabel		: SbGetLocaleByKey('customFile'),
							emptyText		: SbGetLocaleByKey('clickToBrowseAndUploadFile'),
							labelSeparator	: '',
							name			: 'customFile',
							msgTarget		: 'under',
							listeners		: {
								fileselected: function() {
									Ext.getCmp('sb-ownershipType-customFile').setValue(true);
								}
							}
						}, {
							xtype	: 'label',
							text	: SbGetLocaleByKey('customFileDescription'),
							cls		: 'sb-editor-box-description'
						}]
					}]
				}]
			},
			buttons: [{
				text: SbGetLocaleByKey('ok'),
				handler: function() {
					var form = Ext.getCmp('sb-ownership-form').getForm();
					if (form.findField('ownershipType').getGroupValue() == 'emptyFile' && !form.findField('emptyFilename').getValue()) {
						form.findField('emptyFilename').markInvalid(SbGetLocaleByKey('fieldCantBeEmpty'));
						return;
					}
					if (form.findField('ownershipType').getGroupValue() == 'customFile' && !form.findField('customFile').getValue()) {
						form.findField('customFile').markInvalid(SbGetLocaleByKey('fieldCantBeEmpty'));
						return;
					}
					form.submit({
						url: 'Ajax/Editor/OwnershipVerification',
						waitMsg: SbGetLocaleByKey('creatingFile'),
						success: function(fp, action) {
							Sb.Editor.Ui.mainWindow.statusBar.setMessage(
								SbGetLocaleByKey('verificationFileUploadSuccess')
							);
							ownershipWin.close();
						},
						failure: function() {
							Sb.Editor.Ui.mainWindow.statusBar.setMessageError(
								SbGetLocaleByKey('verificationFileUploadFail')
							);
							ownershipWin.close();
						}
					});
				}
			}, {
				text: SbGetLocaleByKey('cancelButtonTitle'),
				handler: function() {
					ownershipWin.close();
				}
			}]
		});
		ownershipWin.show();
	},

	editHeadWinOpen: function () {
		var editHeadWin = new Sb.Window({
			layout		: 'form',
			width		: 500,
			title		: SbGetLocaleByKey('seoEditHeadTitle'),
			modal		: true,
			labelAlign	: 'top',
			items: {
				xtype	: 'textarea',
				fieldLabel: SbGetLocaleByKey('addHeaderCode'),
				anchor	: '100%',
				name	: 'customHeadArea',
				value	: this.form.findField('customHead')
					? this.form.findField('customHead').getValue()
					: Sb.Editor.model.data.customHead
			},
			buttons: [{
				text: SbGetLocaleByKey('Add'),
				scope: this,
				handler: function() {
					var field;
					if (! (field = this.form.findField('customHead'))) {
						field = new Ext.form.Hidden({name: 'customHead'});
						this.form.add(field);
					}
					field.setValue(editHeadWin.find('customHead')[0].getValue());
					Sb.editor.UiContext.registerChanges();
					editHeadWin.close();
				}
			}, {
				text: SbGetLocaleByKey('cancelButtonTitle'),
				handler: function() {
					editHeadWin.close();
				}
			}]
		});
		editHeadWin.show();
	},

	googleAnWinOpen: function () {
		var googleAnWin = new Sb.Window({
			layout		: 'form',
			width		: 500,
			title		: SbGetLocaleByKey('seoGoogleAnTitle'),
			modal		: true,
			labelAlign	: 'top',
			items: {
				xtype: 'textarea',
				fieldLabel: SbGetLocaleByKey('addGoogleAnCode'),
				anchor	: '100%',
				height	: 200,
				name	: 'customHeadArea',
				value	: this.form.findField('googleAnalyticsCode')
					? this.form.findField('googleAnalyticsCode').getValue()
					: Sb.Editor.model.data.googleAnalyticsCode
			},
			buttons: [{
				text: SbGetLocaleByKey('Add'),
				scope: this,
				handler: function() {
					var field;
					if (! (field = this.form.findField('googleAnalyticsCode'))) {
						field = new Ext.form.Hidden({name: 'googleAnalyticsCode'});
						this.form.add(field);
					}
					field.setValue(googleAnWin.find('googleAnalyticsCode')[0].getValue());
					Sb.editor.UiContext.registerChanges();
					googleAnWin.close();
				}
			}, {
				text: SbGetLocaleByKey('cancelButtonTitle'),
				handler: function() {
					googleAnWin.close();
				}
			}]
		});
		googleAnWin.show();
	},

	seoTabAfterRender: function(cmp) {
		Ext.fly('sb-ownership').on('click',  this.ownershipWinOpen, this);
		Ext.fly('sb-edit-head').on('click',  this.editHeadWinOpen, this);
		Ext.fly('sb-google-analitics').on('click',  this.googleAnWinOpen, this);
	},

	/**
	 * Save handler.
	 * @param {Function} afterSaveCallbacks Handler, that will be run after save
	 */
	saveHandler: function(afterSaveCallbacks) {
		var formValues = this.form.getFieldValues();
		Sb.Editor.model.unserialize({
			title				: formValues.siteName,
			pagesUseName		: formValues.pagesUseName,
			description			: formValues.siteDescription,
			keywords			: formValues.metaKeywords,
			customHead			: formValues.customHead,
			googleAnalyticsCode	: formValues.googleAnalyticsCode,
			faviconFile			: this.faviconFile
		});
		var settingsPanel = this;
		Sb.Editor.model.save({
			success: function() {
				afterSaveCallbacks.success.call();
				Sb.Editor.Ui.mainWindow.statusBar.setMessage(
					SbGetLocaleByKey('messageSiteSettingsSaved')
				);
				if ('update' == Sb.Editor.model.data.faviconFile) {
					Sb.Editor.model.unserialize({
						faviconFile: 'favicon.ico'
					});
				} else if ('remove' == Sb.Editor.model.data.faviconFile) {
					Sb.Editor.model.unserialize({
						faviconFile: null
					});
				}
				settingsPanel.faviconFile = Sb.Editor.model.data.faviconFile;
				if (settingsPanel.uploadForm) {
					settingsPanel.uploadForm.reset();
				}
			},
			failure: afterSaveCallbacks.failure
		});
	},

	/**
	 * Discard handler
	 */
	discardHandler: function() {
		this.faviconFile = Sb.Editor.model.data.faviconFile;
		if (this.uploadForm) {
			this.uploadForm.reset();
		}
	},

	onFaviconSelected: function(field, fieldValue) {
		if ('' === fieldValue) {
			return;
		}
		if (!this.uploadForm) {
			this.uploadForm = new Ext.form.BasicForm('sb-favicon-form', {
				fileUpload: true
			});
			this.uploadForm.add(field);
		}
		Sb.editor.UiContext.registerChanges();
		var settingsPanel = this;
		this.uploadForm.submit({
			url: 'Ajax/Editor/FaviconUpload',
			waitMsg: SbGetLocaleByKey('uploadingImage'),
			success: function(fp, action) {
				Ext.getDom('sb-favicon-preview').innerHTML = '<img src="' + Sb.Editor.model.getUrl() + '/favicon_new.png?'
					+ (new Date).getTime()+ '">';
				settingsPanel.faviconFile = 'update';
			},
			failure: function(fp, action) {
				var message = '';
				if (action.result && action.result.message) {
					message = SbGetLocaleByKey(action.result.message);
				} else {
					message = SbGetLocaleByKey('messageNotIcoFile');
				}
				field.markInvalid(message);
			}
		});
	}
});
Ext.reg('settingspanel', Sb.editor.SettingsPanel);