/**
 * @class Sb.editor.StatusBar
 * @extends Ext.Container
 */
Sb.editor.StatusBar = Ext.extend(Ext.Container, {

	cls: 'sb-editor-statusbar',

	lookupComponent: function(c) {
		if (Ext.isString(c)) {
			c = {text: c};
		}
		c = Ext.applyIf(c || {}, {
			text: '',
			level: Sb.editor.StatusBar.INFO
		});

		c = new Ext.BoxComponent({
			autoEl: {
				cls: this.cls + '-message-wrapper',
				cn: {
					cls: this.cls + '-message ' + this.cls + '-message-' + c.level,
					cn: [{
						cls: this.cls + '-message-control',
						cn: {tag: 'span'}
					}, {
						cls: this.cls + '-message-text',
						html: c.text
					}]
				}
			},
			// Implementing close button behaviour
			onRender: function () {
				Ext.BoxComponent.superclass.onRender.apply(this, arguments);
				this.el.first().setStyle('cursor', 'pointer');
				this.el.first().on('click', function () {
					this.el.slideOut('t', {
						scope: this,
						callback: function() {
							if (!this.isDestroyed){
								this.destroy();
							} else if (Ext.getDom(this.id)) {
								Ext.removeNode(Ext.getDom(this.id));
							}
						}
					});
				}, this);
			}
		});
		return c;
	},

	onRender: function() {
		Sb.editor.StatusBar.superclass.onRender.apply(this, arguments);
		if (Ext.isIE7) {
			this.el.dom.innerHTML = '<span />';
		}
		this.el.addClass('clearfix');
	},

	/**
	 * Sets status message.
	 * @param {String/Object} msg A text string, or a config object containing a <tt>text</tt> and <tt>level</tt> property
	 */
	setMessage: function(msg) {
		this.removeAll();
		this.add(msg);
		this.doLayout();
	},

	/**
	 * Sets status message with level "info"
	 * @param {String} msg A text string
	 */
	setMessageInfo: function(msg) {
		this.setMessage({level: Sb.editor.StatusBar.INFO, text: msg});
	},

	/**
	 * Sets status message with level "warning"
	 * @param {String} msg A text string
	 */
	setMessageWarning: function(msg) {
		this.setMessage({level: Sb.editor.StatusBar.WARNING, text: msg});
	},

	/**
	 * Sets status message with level "error"
	 * @param {String} msg A text string
	 */
	setMessageError: function(msg) {
		this.setMessage({level: Sb.editor.StatusBar.ERROR, text: msg});
	},

	/**
	 * Adds status message and leaves previous messages
	 * @param {String/Object} msg A text string, or a config object containing a <tt>text</tt> and <tt>level</tt> property
	 */
	addMessage: function(msg) {
		this.add(msg);
		this.doLayout();
	},

	/**
	 * Adds status message with level "info"
	 * @param {String} msg A text string
	 */
	addMessageInfo: function(msg) {
		this.addMessage({level: Sb.editor.StatusBar.INFO, text: msg});
	},

	/**
	 * Adds status message with level "warning"
	 * @param {String} msg A text string
	 */
	addMessageWarning: function(msg) {
		this.addMessage({level: Sb.editor.StatusBar.WARNING, text: msg});
	},

	/**
	 * Adds status message with level "error"
	 * @param {String} msg A text string
	 */
	addMessageError: function(msg) {
		this.addMessage({level: Sb.editor.StatusBar.ERROR, text: msg});
	},

	/**
	 * Clear all status messages.
	 */
	clear: function() {
		this.removeAll();
		this.doLayout();
	}
});

/**
 * Info message level (default)
 * @type String
 */
Sb.editor.StatusBar.INFO = 'info';

/**
 * Warning message level
 * @type String
 */
Sb.editor.StatusBar.WARNING = 'warning';

/**
 * Error message level
 * @type String
 */
Sb.editor.StatusBar.ERROR = 'error';