/**
 * @class Sb.editor.StyleBorderPanel
 * @extends Ext.Panel
 * Style border editor panel
 * @constructor
 * @param {Object} config The config object
 * @xtype styleborderpanel
 */
Sb.editor.StyleBorderPanel = Ext.extend(Ext.Panel, {

	/**
	 * Borders on page in selectbox
	 */
	bordersOnPage: 6,
	// private
	initComponent: function() {
		var borderStore = new Ext.data.ArrayStore({
			fields: ['name']
		});
		var externalBorderStore = new Ext.data.ArrayStore({
			fields: ['name']
		});
		var borderTab = {
			xtype: 'container',
			defaults: {border: false},
			items: [{
				layout: 'column',
				defaults: {border: false},
				cls: 'sb-style-borders-innerborders',
				items: [{
					width: 242,
					layout: 'form',
					cls: 'sb-style-border-typeselection',
					items: {
						xtype: 'sbselectbox',
						id: 'borderList',
						fieldLabel: SbGetLocaleByKey('borderContainersWithinSite'),
						triggerAction: 'all',
						store: borderStore,
						mode: 'local',
						valueField: 'name',
						width: 228,
						listWidth: 442,
						lazyInit: false,
						tpl: new Ext.XTemplate(
								'<tpl for=".">',
								'<div id="{name}" class="sb-selectbox-list-item sb-style-border-list-item">',
									'<img style="background: {[this.backgroundWatermarkColor]};"',
									'src="' + Sb.Config.templatesUrl + '/' +
										Sb.Editor.model.data.template.code + '/decorations/border/{name}/images/{name}.png',
									'" alt="{name}">',
								'</div>',
							'</tpl>'
						),
						listClass: 'sb-style-borders-list',
						itemSelector: 'div.sb-selectbox-list-item',
						lastQuery: '',
						onActivate: function() {
							this.setBackgroundColor(Sb.Editor.model.design.data.backgroundColor.watermarkColor);
							var innerBorder = 'border-none';
							Ext.iterate(Sb.Editor.model.design.data.border, function(region, border) {
								if (border != innerBorder) {
									innerBorder = border;
									return false;
								} 
							}, this);
							if (innerBorder == 'border-none') {
								innerBorder = 'border1-black';
							}
							this.setValue(innerBorder);
						},
						setBackgroundColor: function(color) {
							this.tpl.backgroundWatermarkColor = color;
							this.el.select('.sb-selectbox-list-item img').setStyle('background', color);
							Ext.iterate(this.view.getNodes(), function(item) {
								item.firstChild.style.background = color;
							});
						},
						listeners: {
							scope: this,
							select: function(selectBox, record, index) {
								this.findByType('checkboxgroup')[0].eachItem(function(checkBox){
									if (checkBox.getValue()) {
										Sb.Editor.model.design.setDecoration('border', checkBox.region + 'Border', record.get('name'));
									}
								});
								Sb.editor.UiContext.registerChanges();
							},
							afterrender: function (selectBox) {
								var colorBorder = 'black';
								if (!selectBox.store.proxy) {
									Sb.DesignTemplateService.getDecorationVariants('generic', 'border', [], function(result) {
										selectBox.storeRawData = result;

										//color definition boarders
										var pattern = new RegExp('(black|white)', 'i');
										Ext.iterate(Sb.Editor.model.design.data.border, function(border, value) {
											var matches = value.match(pattern);
											if (matches) {
												colorBorder = matches[0];
												selectBox.setValue(value);
												return false;
											}
										}, this);

										var data = [];
										pattern = new RegExp('(.+)-(' + colorBorder + ')', 'i');
										Ext.iterate(selectBox.storeRawData, function (item) {
											if (pattern.test(item)) {
												data.push([item]);
											}
										});
										selectBox.store.proxy = new Ext.ux.data.PagingMemoryProxy(data);
										selectBox.store.load({params: {start: 0, limit: this.bordersOnPage}});

										selectBox.header = selectBox.list.createChild(
											{cls: 'x-combo-list-hd'},
											selectBox.innerList
										);

										selectBox.invertBorderColors = new Ext.Container({
											renderTo: selectBox.header,
											cls: 'sb-invert-colors',
											items: [{
												xtype: 'label',
												html: SbGetLocaleByKey('invertBorderColors'),
												cls: 'sb-invert-colors-title'
											}, {
												xtype: 'radio',
												name: 'invertBorderColors',
												boxLabel: SbGetLocaleByKey('black'),
												value: 'black',
												checked: colorBorder == 'black',
												listeners: {
													scope: this,
													check: function(radio, checked) {
														this.onChangeBorderColor(radio,	selectBox);
													}
												}
											}, {
												xtype: 'radio',
												name: 'invertBorderColors',
												boxLabel: SbGetLocaleByKey('white'),
												value: 'white',
												checked: colorBorder == 'white',
												listeners: {
													scope: this,
													check: function(radio, checked) {
														this.onChangeBorderColor(radio, selectBox);
													}
												}
											}]
										});
										selectBox.assetHeight += selectBox.header.getHeight();

										selectBox.footer = selectBox.list.createChild({cls: 'x-combo-list-ft'});
										selectBox.pageTb = new Ext.PagingToolbar({
											store: selectBox.store,
											pageSize: this.bordersOnPage,
											renderTo: selectBox.footer
										});

										if (Ext.isIE7) {
											selectBox.footer.repaint();
										}

										selectBox.assetHeight += selectBox.footer.getHeight();
									}, this);
								}
							}
						}
					}
				}, {
					columnWidth: 1,
					layout: 'form',
					cls: 'sb-style-border-locationselection11',
					items: {
						xtype: 'checkboxgroup',
						columns: 2,
						fieldLabel: SbGetLocaleByKey('StyleBorderDisplayBorderHere'),
						anchor: '100%',
						defaults: {
							listeners: {
								scope: this,
								check: this.onBorderToContainer
							}
						},
						onActivate: function() {
							this.eachItem(function(item){
								if (item.onActivate) {
									item.onActivate();
								}
							});
						},
						items: [{
							name: 'borderToColumn1',
						    boxLabel: SbGetLocaleByKey('layoutBorderSidebarLeft'),
							region: 'column1',
							onActivate: function() {
								this.region = Sb.Editor.model.design.data.layout.sidebarFirstPosition == 'left' ? 'column1' : 'column2';
								this.setValue(!(Sb.Editor.model.design.data.border[this.region + 'Border'] == 'border-none'));
								var sidebar = this.region == 'column1' ? 'sidebarFirst' : 'sidebarSecond';
								this.el.up('.x-form-item').dom.style.display = Sb.Editor.model.design.data.layout[sidebar] ? 'block' : 'none';
							}
						}, {
							name: 'borderToHeader',
						    boxLabel: SbGetLocaleByKey('layoutBorderHeader'),
							region: 'header',
							onActivate: function() {
								this.setValue(!(Sb.Editor.model.design.data.border.headerBorder == 'border-none'));
							}
						}, {
							name: 'borderToContent',
						    boxLabel: SbGetLocaleByKey('layoutBorderColumn2Content'),
							region: 'content',
							onActivate: function() {
								this.setValue(!(Sb.Editor.model.design.data.border.contentBorder == 'border-none'));
							}
						}, {
							name: 'borderToFooter',
						    boxLabel: SbGetLocaleByKey('layoutBorderFooter'),
							region: 'footer',
							onActivate: function() {
								this.setValue(!(Sb.Editor.model.design.data.border.footerBorder == 'border-none'));
							}
						}, {
							name: 'borderToColumn3',
						    boxLabel: SbGetLocaleByKey('layoutBorderSidebarRight'),
							region: 'column2',
							onActivate: function() {
								this.region = Sb.Editor.model.design.data.layout.sidebarFirstPosition == 'right' ? 'column1' : 'column2';
								this.setValue(!(Sb.Editor.model.design.data.border[this.region + 'Border'] == 'border-none'));
								var sidebar = this.region == 'column1' ? 'sidebarFirst' : 'sidebarSecond';
								this.el.up('.x-form-item').dom.style.display = Sb.Editor.model.design.data.layout[sidebar] ? 'block' : 'none';
							}
						}]
					}
				}]
			}, {
				xtype: 'container',
				layout: 'form',
				cls: 'sb-style-borders-siteborder',
				items: {
					xtype: 'sbselectbox',
					id: 'externalBorderList',
					fieldLabel: SbGetLocaleByKey('borderAroundSite'),
					triggerAction: 'all',
					store: externalBorderStore,
					mode: 'local',
					width: 228,
					valueField: 'name',
					listWidth: 442,
					lazyInit: false,
					tpl: new Ext.XTemplate(
							'<tpl for=".">',
							'<div id="{name}" class="sb-selectbox-list-item sb-style-border-list-item">',
							'<img style="background: {[this.backgroundSiteColor]};"',
								'src="' + Sb.Config.templatesUrl + '/' +
									Sb.Editor.model.data.template.code + '/decorations/external-border/{name}/images/{name}.png',
								'" alt="{name}">',
							'</div>',
						'</tpl>'
					),
					itemSelector: 'div.sb-selectbox-list-item',
					listClass: 'sb-style-borders-list',
					lastQuery: '',
					onActivate: function() {
						this.setBackgroundColor(Sb.Editor.model.design.data.backgroundColor.siteColor);
						this.setValue(Sb.Editor.model.design.data['external-border']['watermarkExternalBorder']);
					},
					setBackgroundColor: function(color) {
						this.tpl.backgroundSiteColor = color;
						this.el.select('.sb-selectbox-list-item img').setStyle('background', color);
						Ext.iterate(this.view.getNodes(), function(item) {
							item.firstChild.style.background = color;
						});
					},
					listeners: {
						scope: this,
						select: function(selectBox, record, index) {
							Sb.Editor.model.design.setDecoration('external-border', 'watermarkExternalBorder', record.get('name'));
							Sb.editor.UiContext.registerChanges();
						},
						afterrender: function(selectBox) {
							var colorBorder = 'black';
							if (!selectBox.store.proxy) {
								Sb.DesignTemplateService.getDecorationVariants('generic', 'external-border', [], function(result) {
									selectBox.storeRawData = result;

									//color definition external-boarder
									var pattern = new RegExp('(black|white)', 'i');
									var value = Sb.Editor.model.design.data['external-border']['watermarkExternalBorder'];
									var matches = value.match(pattern);
									if (matches) {
										colorBorder = matches[0];
										selectBox.setValue(value);
									}

									var data = [];
									pattern = new RegExp('(.+)-(' + colorBorder + '|none)', 'i');
									Ext.iterate(selectBox.storeRawData, function (item) {
										if (pattern.test(item)) {
											data.push([item]);
										}
									});
									selectBox.store.proxy = new Ext.ux.data.PagingMemoryProxy(data);
									selectBox.store.load({params: {start: 0, limit: this.bordersOnPage}});
									selectBox.header = selectBox.list.createChild(
										{cls: 'x-combo-list-hd'},
										selectBox.innerList
									);

									selectBox.invertBorderColors = new Ext.Container({
										renderTo: selectBox.header,
										cls: 'sb-invert-colors',
										items: [{
											xtype: 'label',
											html: SbGetLocaleByKey('invertBorderColors'),
											cls: 'sb-invert-colors-title'
										}, {
											xtype: 'radio',
											name: 'invertExternalBorderColors',
											boxLabel: SbGetLocaleByKey('black'),
											value: 'black',
											checked: colorBorder == 'black',
											listeners: {
												scope: this,
												check: function(radio, checked) {
													this.onChangeBorderColor(radio,	selectBox);
												}
											}
										}, {
											xtype: 'radio',
											name: 'invertExternalBorderColors',
											boxLabel: SbGetLocaleByKey('white'),
											value: 'white',
											checked: colorBorder == 'white',
											listeners: {
												scope: this,
												check: function(radio, checked) {
													this.onChangeBorderColor(radio, selectBox);
												}
											}
										}]
									});
									selectBox.assetHeight += selectBox.header.getHeight();

									selectBox.footer = selectBox.list.createChild({cls: 'x-combo-list-ft'});
									selectBox.pageTb = new Ext.PagingToolbar({
										store: selectBox.store,
										pageSize: this.bordersOnPage,
										renderTo: selectBox.footer
									});

									if (Ext.isIE7) {
										selectBox.footer.repaint();
									}
									selectBox.assetHeight += selectBox.footer.getHeight();
								}, this);
							}
						}
					}
				}
			}]
		};

		Ext.apply(this, {
			title: SbGetLocaleByKey('styleBordersTab'),
			cls: 'sb-editor-style-border-panel',
			defaults: {
				border: false,
				layout: 'form',
				labelAlign: 'top'
			},
			helperMessage: SbGetLocaleByKey('styleBorderTabDescription'),
			items: borderTab,
			listeners: {
				scope: this,
				activate: this.onActivate
			}
		});
		Sb.editor.StyleBorderPanel.superclass.initComponent.apply(this, arguments);
	},

	/**
	 * Change border to selected color
	 * @param {Ext.form.Radio} radio
	 * @param {Sb.form.Selectbox} selectBox
	 */
	onChangeBorderColor: function(radio, selectBox) {
		if (!radio.checked) {
			return;
		}
		var subPattern = radio.value + (radio.name == 'invertExternalBorderColors' ? '|none' : '');
		var data = [], pattern = new RegExp('(.+)-(' + subPattern + ')', 'i');
		Ext.iterate(selectBox.storeRawData, function (item) {
			if (pattern.test(item)) {
				data.push([item]);
			}
		});
		selectBox.store.proxy = new Ext.ux.data.PagingMemoryProxy(data);
		selectBox.pageTb.changePage(Math.ceil((selectBox.pageTb.cursor + selectBox.pageTb.pageSize) / selectBox.pageTb.pageSize));
		selectBox.pageTb.doRefresh();
	},

	/**
	 * Activate values and states of elements for border
	 */
	onActivate: function() {
		var xtypes = ['sbselectbox', 'checkboxgroup'];
		Ext.each(xtypes, function(xtype) {
			Ext.each(this.findByType(xtype), function(item){
				if (item.onActivate) {
					item.onActivate();
				}
			});
		}, this);
	},

	/**
	 * Set Unset border to some container
	 * @param {Ext.form.Checkbox} checkbox
	 * @param {Boolean} checked
	 */
	onBorderToContainer: function(checkbox, checked) {
		var selectedBorder = Ext.getCmp('borderList').getValue();
		if (checkbox.getValue()) {
			Sb.Editor.model.design.setDecoration('border',
				checkbox.region + 'Border',
				selectedBorder);
		} else {
			Sb.Editor.model.design.setDecoration('border', checkbox.region + 'Border', 'border-none');
		}
		Sb.editor.UiContext.registerChanges();
	}
});
Ext.reg('styleborderpanel', Sb.editor.StyleBorderPanel);