/**
 * @class Sb.Editor.Ui
 * <p>This singleton object, which is represent user interface of Sitebuilder.</p>
 * @singleton
 */
Sb.Editor.Ui = function() {
	var widgetsPanel;
	return {
		/**
		 * Checks that any object is dragged at this time
		 */
		isDragActive: function() {
			return Ext.dd.DragDropMgr.dragCurrent != null;
		},

		/**
		 * Main window reference
		 * @type Sb.editor.MainWindow
		 * @property mainWindow
		 */
		mainWindow: null,

		init: function() {
			this.mainWindow = new Sb.editor.MainWindow();
			this.mainWindow.show();

			if (Ext.isIE) {
				var isResized = false;
				if (Ext.isIE7) {
					var rect = document.body.getBoundingClientRect();
					isResized = (rect.right - rect.left) != document.body.clientWidth;
				} else {
					isResized = 96 != screen.deviceXDPI;
				}
				if (isResized) {
					this.mainWindow.setPosition(0, 0);
					Sb.Editor.Ui.mainWindow.statusBar.addMessageWarning(SbGetLocaleByKey('ieResizedWarning'));
				}
			}

			if (window.console && window.console.firebug) {
				Sb.Editor.Ui.mainWindow.statusBar.addMessageWarning(SbGetLocaleByKey('firebugWarning'));
			}

			if (Ext.isIE7) {
				Sb.Editor.Ui.mainWindow.statusBar.addMessageWarning(SbGetLocaleByKey('updateYouIE7toNewVersion'));
			}
		},

		onContextMenu: function(event, element, options) {
			event.stopEvent();

			var items = [];

			if (options.type == 'container') {
				var widgetItems = [];
				Ext.each(['text', '-', 'navigation', 'sharedText', 'header'], function(widget) {
					if (widget == '-') {
						widgetItems.push('-');
						return;
					}
					var type = Sb.WidgetTypeManager.get(widget);
					if (type.isDroppableTo(options.region)) {
						widgetItems.push({
							text: type.getTitle().replace('<br/>', ' '),
							iconCls: 'sb-' + widget.toLowerCase() + '-icon',
							disabled: !type.isAvailable(),
							handler: function() {
								var container = Sb.Editor.siteView.currentPage.widget.widgets.find(function(container) {
									return container.data.region == options.region;
								});
								container.createChild(Sb.WidgetTypeManager.get(widget));
							}
						});
					}
				});
				items.push({
					text: SbGetLocaleByKey('cmAddWidget'),
					iconCls: 'sb-add-widget-icon',
					hideOnClick: false,
					menu: {
						items: widgetItems
					}
				});
			}

			var mainTabs = Sb.Editor.Ui.mainWindow.tabs;

			if (options.type == 'widget' || options.type == 'container') {
				var buttonImg = new Ext.ux.form.FileUploadField({
					autoWidth: true,
					regex: new RegExp('\\.(' + Sb.Config.imageValidExtensions.join('|') + ')$', 'i'),
					buttonOnly: true,
					name: 'image',
					buttonText: SbGetLocaleByKey('cmAddImage'),
					listeners: {
						scope: options.widget,
						fileselected: function(field, fieldValue) {
							var addHandler = function() {
								this.startEdit();
								this.instance.onInit.add(function() {
									this.onImageUpload(field, fieldValue);
								}.createDelegate(this));
							};
							field.ownerCt.ownerCt.doHide();
							if (this.getXType() != 'text_widget' &&
								this.getXType() != 'shared_text_widget') {
								var container = Sb.Editor.siteView.pageWidget.items.find(function(container) {
									return container.model.data.region == options.region;
								});
								container.on('add', function(container, widget) {
									widget.on('afterrender', addHandler, widget);
								}, this, {single: true});
								container.model.createChild(Sb.WidgetTypeManager.get('text'));
							} else {
								addHandler.call(this);
							}
						}
					}
				});
				var form = new Ext.form.FormPanel({
					cls: 'sb-insert-image',
					fileUpload	: true,
					border		: false,
					hideLabels	: true,
					items: buttonImg
				});

				items.push(form);
			} else {
				items.push({
					text: SbGetLocaleByKey('cmStyleColors' + options.type.toUpperCaseFirst()),
					iconCls: 'sb-style-colors-icon',
					handler: function() {
						var styleTab = mainTabs.getComponent('style');
						var place = (options.type == 'container') ? options.region : options.type;
						styleTab.setActiveTab('colors');
						styleTab.form.findField('bgPlace').setValue(place);
						styleTab.onBgPlaceSelect(place);
					}
				});
			}

			if (options.type == 'watermark') {
				items.push({
					text: SbGetLocaleByKey('cmLayoutMargins'),
					iconCls: 'sb-layout-margins-icon',
					handler: function() {
						mainTabs.setActiveTab('layout');
					}
				});
			}

			if (options.type == 'container') {
				if (options.region != 'header' && options.region != 'footer') {
					items.push({
						text: SbGetLocaleByKey('cmLayoutWidth'),
						iconCls: 'sb-layout-width-icon',
						handler: function() {
							mainTabs.setActiveTab('layout');
						}
					});
				}
			}

			// Common menu
			if (items.length) {
				items.push('-');
			}
			items.push({
				text: SbGetLocaleByKey('cmStyleFonts'),
				iconCls: 'sb-style-fonts-icon',
				handler: function() {
					mainTabs.getComponent('style').setActiveTab('fonts');
				}
			});
			items.push({
				text: SbGetLocaleByKey('cmLayout'),
				iconCls: 'sb-layout-icon',
				handler: function() {
					mainTabs.setActiveTab('layout');
				}
			});
			items.push({
				text: SbGetLocaleByKey('cmStyleScheme'),
				iconCls: 'sb-style-scheme-icon',
				handler: function() {
					mainTabs.getComponent('style').setActiveTab('scheme');
				}
			});
			items.push({
				text: SbGetLocaleByKey('cmStyleBorders'),
				iconCls: 'sb-style-borders-icon',
				handler: function() {
					mainTabs.getComponent('style').setActiveTab('styleBorder');
				}
			});
			items.push({
				text: SbGetLocaleByKey('cmPages'),
				iconCls: 'sb-pages-icon',
				handler: function() {
					mainTabs.setActiveTab('pages');
				}
			});
			items.push({
				text: SbGetLocaleByKey('cmSettings'),
				iconCls: 'sb-settings-icon',
				handler: function() {
					mainTabs.setActiveTab('settings');
				}
			});
			items.push('-');
			items.push({
				text: SbGetLocaleByKey('cmHelp'),
				iconCls: 'sb-help-icon',
				handler: function() {
					if (null == options.type) {
						return;
					}
					var articleUrl	= '';
					articleUrl		+= options.type.toUpperCaseFirst();
					if (null != options.region) {
						articleUrl	+= '/' + options.region.toUpperCaseFirst();
					}
					Sb.Editor.openHelp(articleUrl);
				}
			});

			if (!this.menu) {
				this.menu = new Ext.menu.Menu({
					listeners: {
						scope: this,
						hide: function(m) {
							this.isHoverDisabled = false;
						}
					}
				});
			} else {
				this.menu.removeAll();
				if (Ext.isIE7) { // this.menu.removeAll() doesn't remove all items in ie7
					this.menu.el.select('li').each(function(li){
						li.remove();
					});
				}
			}
			this.menu.add(items);
			this.isHoverDisabled = true;
			this.menu.showAt(event.getXY());
		}
	};
}();