/**
 * @class Sb.editor.WidgetsPanel
 * @extends Ext.Panel
 * Widgets panel
 * @constructor
 * @param {Object} config The config object
 * @xtype widgetspanel
 */
Sb.editor.WidgetsPanel = Ext.extend(Ext.Panel, {
	initComponent: function() {
		var widgets = this.items || [];
		Ext.apply(this, {
			title: SbGetLocaleByKey('widgetsTitle'),
			helperMessage: SbGetLocaleByKey('widgetsDescription'),
			tabTip: SbGetLocaleByKey('widgetsDescriptionOnHover'),
			cls: 'sb-editor-widgets-panel',
			autoHeight: true,
			items: {
				layout: 'carousel',
				layoutConfig: {
					scrollButtonPosition: 'split',
					marginScrollButtons: 10
				},
				border: false,
				itemId: 'panel',
				items: widgets,
				lookupComponent: function(config) {
					if ('-' === config) {
						return new Sb.editor.WidgetsPanel.Separator();
					}
					config = (Ext.isString(config)) ? {type: config} : config;
					return new Sb.editor.WidgetsPanel.Button(config);
				},
				listeners: {
					afterrender: function(panel) {
						panel.el.on('mousewheel', function (e) {
							for (var i = 0; i < Math.abs(e.getWheelDelta()) * 4; i++) {
								this.layout['onScroll' + (e.getWheelDelta() > 0 ? 'Left' : 'Right') + 'Click'].call(this.layout);
							}
						}, panel);
					}
				}
			}
		});
		Sb.editor.WidgetsPanel.superclass.initComponent.apply(this, arguments);
	}});
Ext.reg('widgetspanel', Sb.editor.WidgetsPanel);

/**
 * @class Sb.editor.WidgetsPanel.Button
 * @extends Ext.BoxComponent
 */
Sb.editor.WidgetsPanel.Button = Ext.extend(Ext.BoxComponent, {

	overCls: 'sb-widgets-item-over',
	disabledClass: 'sb-widgets-item-disabled',

	constructor: function (config) {
		config.itemId = config.type;

		Sb.editor.WidgetsPanel.Button.superclass.constructor.call(this, config);

		/**
		 * Button`s widget type
		 * @type {Sb.widgetType.Base}
		 * @property widgetType
		 */
		this.type = Sb.WidgetTypeManager.get(config.type);
		Sb.WidgetTypeManager.registerComponent(this, this.type);
	},

	onRender: function(ct, position) {
		this.autoEl = {
			cls: 'sb-widgets-item-wrap',
			cn: {
				cls: 'sb-widgets-item sb-widgets-item-' + this.type.getKey(),
				cn: {
					cls: 'sb-widgets-item-content-wrap',
					id: Ext.id(),
					cn: {
						cls: 'sb-widgets-item-content',
						id: Ext.id(),
						cn: {
							id: Ext.id(),
							tag: 'span',
							html: this.type.getTitle()
						}
					}
				}
			}
		};
		Sb.editor.WidgetsPanel.Button.superclass.onRender.call(this, ct, position);

		Ext.QuickTips.registerSelfAndChildren({
			target: this, text: this.type.getDescription(this.disabled)
		});
	},

	onEnable: function() {
		this.createDD();
		this.dd.unlock();
		Sb.editor.WidgetsPanel.Button.superclass.onEnable.call(this);
	},

	onDisable: function() {
		if (this.dd) {
			this.dd.lock();
		}
		Sb.editor.WidgetsPanel.Button.superclass.onDisable.call(this);
	},

	createDD: function() {
		if (!this.dd) {
			this.dd = new Ext.dd.DragSource(this.el.first(), {
				buttonEl	: this.el,
				ddGroup		: 'widgetButtons',
				dragData	: {widgetType: this.type, widgetButton: this},
				scroll		: false,

				startDrag	: function() {
					Sb.Editor.siteView.onWidgetStartDrag();
					if (Sb.widget.Widget.editCurrent) {
						Sb.widget.Widget.editCurrent.completeEdit.call(Sb.widget.Widget.editCurrent);
					}
					this.proxy.ghost.addClass(this.buttonEl.dom.className);
					this.proxy.update(this.buttonEl.dom.innerHTML);
					Ext.getBody().setStyle('cursor', 'default');
					this.dragData.widgetButton.removeClass(this.dragData.widgetButton.overCls);
				},

				endDrag		: function() {
					if (Sb.widget.ContainerVertical.spacer) {
						Sb.widget.ContainerVertical.spacer.remove();
						delete Sb.widget.ContainerVertical.spacer;
					}
					Ext.getBody().setStyle('cursor', 'auto');
					Sb.Editor.siteView.onWidgetEndDrag();
				}
			});
		}
	},

	afterRender: function() {
		if (!this.disabled) {
			this.createDD();
		}
	}
});

/**
 * @class Sb.editor.WidgetsPanel.Separator
 * @extends Ext.BoxComponent
 */
Sb.editor.WidgetsPanel.Separator = Ext.extend(Ext.BoxComponent, {

	onRender: function(ct, position) {
		this.autoEl = {
			cls: 'sb-widgets-separator',
			cn: {
				tag: 'span'
			}
		};
		Sb.editor.WidgetsPanel.Separator.superclass.onRender.call(this, ct, position);
	}
});