Sb.form.ColorDropDown = Ext.extend(Ext.form.TriggerField, {

	/**
	 * @cfg {Boolean} isShowNoColor Is no color button visible? (defaults to <tt>true</tt>)
	 */
	isShowNoColor: true,

	/**
	 * @cfg {Boolean} isShowColorField Is color field visible? (defaults to <tt>true</tt>)
	 */
	isShowColorField: true,

	/**
	 * @cfg {Boolean} isAutoValueNull Is auto value means null? (defaults to <tt>true</tt>)
	 */
	isAutoValueNull: true,

	constructor: function(config) {
		config = Ext.apply({
			cls			: 'sb-form-color-field',
			itemCls		: 'sb-edit-control',
			noColorText	: SbGetLocaleByKey('noColor'),
			autoValue	: null,
			renderMode	: 'combo',
			editable	: true,
			allowBlank	: false,
			maskRe		: /[\#0-9A-F]/i
		}, config);

		this.regex			= new RegExp('^(\#([0-9A-F]{3}|[0-9A-F]{6})|' + RegExp.escape(config.noColorText) + ')$', 'i');
		Sb.form.ColorDropDown.superclass.constructor.call(this, config);

		this.on('valid', this.onValid, this);
	},

	onValid: function() {
		var color			= this.el.dom.value,
			isNoValue		= color == null || color == '' || color == this.noColorText,
			isRawValueEmpty	= '#000000' == this.rawValue || null == this.rawValue;

		if (isNoValue && this.autoValue) {
			this.menu.picker.setRawValue(this.autoValue);
			return;
		} else if (isNoValue && isRawValueEmpty) {
			return;
		}

		this.menu.picker.setValue(color);
	},

	onResize: function(aw, ah, w, h) {
		Sb.form.ColorDropDown.superclass.onResize.call(this, aw, ah, w, h);
		if(Ext.isNumber(aw)){
			this.el.setWidth(aw - this.el.getMargins('lr') - this.getTriggerWidth());
		}
	},

	initComponent: function() {
		if ('button' == this.renderMode) {
			Ext.apply(this, {
				template: new Ext.Template(
					'<table id="{4}" cellspacing="0" class="x-btn {3}"><tbody class="{1}">',
					'<tr><td class="x-btn-tl"><i>&#160;</i></td><td class="x-btn-tc"></td><td class="x-btn-tr"><i>&#160;</i></td></tr>',
					'<tr><td class="x-btn-ml"><i>&#160;</i></td><td class="x-btn-mc x-btn-mc-colordropdown"><em class="{2}" unselectable="on"><button type="{0}"></button><span><!----></span></em></td><td class="x-btn-mr"><i>&#160;</i></td></tr>',
					'<tr><td class="x-btn-bl"><i>&#160;</i></td><td class="x-btn-bc"></td><td class="x-btn-br"><i>&#160;</i></td></tr>',
					'</tbody></table>'
				)
			});
		}
		Sb.form.ColorDropDown.superclass.initComponent.call(this);
		this.menu = new Sb.form.ColorMenu({
			isShowNoColor	: this.isShowNoColor,
			isShowColorField: this.isShowColorField,
			noColorText		: this.noColorText,
			autoValue		: this.autoValue,
			isAutoValueNull	: this.isAutoValueNull,
			shadow			: false
		});
		this.menu.ownerCt = this;
		this.relayEvents(this.menu, ['change']);
		this.menu.on('show', function() {
			this.isMenuVisible = true;
		}, this);
		this.menu.on('hide', function() {
			this.isMenuVisible = false;
		}, this);
		this.menu.on('beforehide', function() {
			var isHideByTrigger = this.trigger && Ext.EventObject.getTarget() == this.trigger.dom;
			if (isHideByTrigger && this.isMenuVisible) {
				this.isTriggerActive = false;
			}
			return true;
		}, this);
		this.menu.on('change', this.demonstrateSelectedColor, this);
	},

	/**
	 * Can trigger handler execute?
	 *
	 * @type Boolean
	 * @property isTriggerActive
	 */
	isTriggerActive: true,

	/**
	 * Is color menu visible?
	 *
	 * @type Boolean
	 * @property isMenuVisible
	 */
	isMenuVisible: false,

	/**
	 * Show/hide menu
	 *
	 * @param {EventObject} e
	 */
	onTriggerClick: function(e) {
		if (!this.isTriggerActive) {
			this.isTriggerActive = true;
			return;
		}
		if (!this.isMenuVisible) {
			this.menu.show(this.wrap);
		}
	},

	/**
	 * Sets background color to demostrate currently selected color
	 */
	demonstrateSelectedColor: function() {
		var color = this.getValue();
		if ('button' == this.renderMode && this.rendered && this.btnEl) {
			this.btnEl.next().setStyle('background-color', color);
		} else if ('combo' == this.renderMode) {
			this.updateColor(color);
		}
	},

	initValue: Ext.emptyFn,

	/**
	 * Raw color value
	 *
	 * @type String
	 * @property rawValue
	 */
	rawValue: null,

	/**
	 * Updates colorfield styles
	 */
	updateColor: function(color) {
		var isNoValue		= color == null || color == '' || color == this.noColorText,
			isAutoValue		= isNoValue && this.autoValue,
			color			= isAutoValue ? this.autoValue : color,
			backgroundColor	= color,
			colorValue		= color,
			textColor		= '#000000';

		if (isNoValue) {
			if (isAutoValue) {
				textColor 		= Ext.ux.util.Color.isLight(color) ? '#000000' : '#FFFFFF';
				this.rawValue	= this.isAutoValueNull ? null : this.autoValue;
			} else {
				this.rawValue	= null;
				backgroundColor	= '#ffffff';
				colorValue		= this.noColorText;
			}
		} else {
			textColor			= Ext.ux.util.Color.isLight(color) ? '#000000' : '#FFFFFF';
			this.rawValue		= color;
		}

		var fn = function() {
			this.el.setStyle({
				background		: backgroundColor,
				color			: textColor
			});
			this.el.dom.value	= colorValue;
		};

		if (!this.rendered) {
			this.on('render', fn, this, {single: true});
		} else {
			fn.call(this);
		}
	},

	onRender: function() {
		Sb.form.ColorDropDown.superclass.onRender.apply(this, arguments);
		this.demonstrateSelectedColor();
	},

	onDestroy : function() {
		Ext.destroy(this.menu);
		Sb.form.ColorDropDown.superclass.onDestroy.call(this);
	},

	getColor: function() {
		return this.rawValue;
	},

	/**
	 * Change color with HTML syntax #123456 as initial, allowing to reset it on cancel
	 *
	 * @param {String} color The color to set the color picker
	 */
	setInitialValue: function(color) {
		this.setValue(color);
		this.menu.startValue = color;
		this.menu.picker.setRawValue(color);
	},

	/**
	 * Set`s autoValue param to self and all children
	 *
	 * @param {String} color The color to set the all children
	 */
	setAutoValue: function(color) {
		if (color == 'transparent') {
			color = null;
		}
		this.autoValue				= color;
		this.menu.autoValue			= color;
		this.menu.picker.autoValue	= color;
		if (!this.menu.value) {
			this.menu.picker.setRawValue(null, color);
			this.menu.picker.setRawValue(null);
			this.demonstrateSelectedColor();
		}
	},

	/**
	 * Change color with HTML syntax #123456
	 *
	 * @param {String} color The color to set the color picker
	 */
	setValue: function(color) {
		this.updateColor(color);
		this.menu.value = color;
		this.menu.colorField.setValue(color);
	},

	/**
	 * Returns color with HTML syntax #123456
	 *
	 * @return {String}
	 */
	getValue: function() {
		return this.menu.picker.getValue();
	}
});

Sb.form.ColorMenu = Ext.extend(Ext.menu.Menu, {

	/**
	 * @cfg {Boolean} isShowNoColor Is no color button visible? (defaults to <tt>true</tt>)
	 */
	isShowNoColor: true,

	/**
	 * @cfg {Boolean} isShowColorField Is color field visible? (defaults to <tt>true</tt>)
	 */
	isShowColorField: true,

	enableScrolling: false,
	width: 212,

	constructor: function(config) {
		config = Ext.apply({
			noColorText	: SbGetLocaleByKey('noColor'),
			value		: null
		}, config);
		Sb.form.ColorMenu.superclass.constructor.call(this, config);
	},

	onRender : function(ct, position){
		this.keyNav = true;
		Sb.form.ColorMenu.superclass.onRender.call(this, ct, position);
		this.keyNav = false;
	},

	initComponent: function() {
		this.picker		= new Ext.ux.ColorPicker({
			value			: this.value,
			autoValue		: this.autoValue
		});
		this.colorField = new Sb.form.ColorField({
			value			: this.value,
			currentPicker	: this.picker
		});
		this.colorField.on('valid', function(field) {
			this.fireEvent('change', this, field.getValue());
		}, this);

		var panelItems = [];
		if (this.isShowColorField) {
			panelItems.push(this.colorField);
		}
		panelItems.push(this.picker);
		if (this.isShowNoColor) {
			this.noColor = new Ext.Component({
				cls: 'sb-color-dropdown-nocolor',
				html: '<span>' + this.noColorText + '</span>',
				listeners: {
					scope: this,
					afterrender: function(cmp) {
						cmp.el.first().on('click', function() {
							var value = null;
							if (this.autoValue && !this.isAutoValueNull) {
								value = this.autoValue;
							}
							if (this.autoValue) {
								this.picker.setRawValue(value, this.autoValue);
							} else {
								this.picker.setRawValue(value);
							}
							this.fireEvent('change', this, value);
						}, this);
					}
				}
			});
			panelItems.push(this.noColor);
		}
		Ext.apply(this, {
			cls: 'sb-color-dropdown',
			items: [{
				xtype: 'panel',
				items: panelItems,
				buttonAlign: 'center',
				buttons: [{
					text: SbGetLocaleByKey('ok'),
					handler: function() {
						this.hide();
					}.createDelegate(this)
				}, {
					text: SbGetLocaleByKey('cancelButtonTitle'),
					handler: function() {
						this.startValue = (this.startValue) ? this.startValue : null;
						this.picker.setValue(this.startValue);
						this.isCancelled = true;
						this.hide();
					}.createDelegate(this)
				}]
			}]
		});
		Sb.form.ColorMenu.superclass.initComponent.call(this);
		this.relayEvents(this.picker, ['change']);
		this.on('change', this.onChange, this);
		this.on('show', function() {
			if (undefined === this.startValue) {
				this.startValue = this.picker.getValue();
			}
			this.isCancelled = false;
		}, this);
	},

	onChange: function(e, color) {
		if (this.colorField.getValue() != color) {
			this.colorField.setRawValue(color, true);
		}
	},

	doLayout: function(shallow, force) {
		Sb.form.ColorMenu.superclass.doLayout.call(this, shallow, force);
		this.getEl().setZIndex(30000);
	}
});
Ext.reg('colordropdown', Sb.form.ColorDropDown);