Sb.form.FontFamilySizePanel = Ext.extend(Sb.form.FieldGroup, {

	/**
	 * Font-family options
	 * @type Array
	 * @property fontFamilies
	 */
	fontFamilies	: [
		'Andale Mono',	'Arial',	'Arial Black',		'Book Antiqua',	'Comic Sans MS',
		'Courier New',	'Georgia',	'Helvetica',		'Impact',		'Symbol',
		'Tahoma',		'Terminal',	'Times New Roman',	'Trebuchet MS',	'Verdana',
		'Webdings',		'Wingdings'
	],

	/**
	 * Symbol-like font families
	 * @type RegExp
	 * @property reSymbolFamily
	 */
	reSymbolFamily	: /Symbol|Webdings|Wingdings/,

	/**
	 * Font-size options
	 * @type Array
	 * @property fontSizes
	 */
	fontSizes		: [
		'8',	'9',	'10',	'11',	'12',	'14',	'16',	'18',
		'20',	'22',	'24',	'26',	'28',	'36',	'48',	'72'
	],

	constructor		: function(config) {
		config = Ext.apply({
			fontFamilyWidth	: 'auto',
			fontSizeWidth	: 'auto',
			widgetContolCls	: '',
			cls				: 'sb-edit-control-container'
		}, config);

		Sb.form.FontFamilySizePanel.superclass.constructor.call(this, config);

		var fontFamilies	= [];
		Ext.each(this.fontFamilies, function(fontFamily) {
			var styleFont	= (this.reSymbolFamily.test(fontFamily)) ? 'Arial' : fontFamily;
			fontFamilies.push([fontFamily, styleFont]);
		}, this);
		fontFamilies		= new Ext.data.ArrayStore({
			fields			: ['value', 'styleFont'],
			data			: fontFamilies
		});
		var fontFamilyCombo = new Ext.form.ComboBox({
			itemId			: 'FontFamily',
			fieldLabel		: SbGetLocaleByKey('fontFamily'),
			itemCls			: this.widgetContolCls + 'font-family',
			store			: fontFamilies,
			displayField	: 'value',
			valueField		: 'value',
			mode			: 'local',
			width			: this.fontFamilyWidth,
			tpl				:
				'<tpl for=".">' +
					'<div class="x-combo-list-item" style="font-family: {styleFont}">{value}</div>' +
				'</tpl>'
		});
		this.add(fontFamilyCombo);

		var fontSizeCombo = new Ext.form.ComboBox({
			itemId			: 'FontSize',
			fieldLabel		: SbGetLocaleByKey('fontSize'),
			itemCls			: this.widgetContolCls + 'font-size',
			store			: this.fontSizes,
			width			: this.fontSizeWidth,
			editable		: true,
			maskRe			: /[0-9]/
		});
		this.add(fontSizeCombo);
	}
});