/**
 * @class Sb.form.FontSelectBox
 * @extends Ext.form.ComboBox
 * A selectbox control.
 * @constructor
 * Create a new SelectBox.
 * @param {Object} config Configuration options
 * @xtype sbselectbox
 */
Sb.form.FontSelectBox = Ext.extend(Ext.form.ComboBox, {

	storeList: {
		size: new Ext.data.ArrayStore({
			fields: ['value'],
			data: [['8'], ['9'], ['10'], ['11'], ['12'], ['14'], ['16'], ['18'],
			       ['20'], ['22'], ['24'], ['26'], ['28'], ['36'], ['48'], ['72']
			]
		}),
		family: new Ext.data.ArrayStore({
			fields: ['value'],
			data: [['Andale Mono'], ['Arial'], ['Arial Black'], ['Book Antiqua'], ['Comic Sans MS'], ['Courier New'],
				['Georgia'], ['Helvetica'], ['Impact'], ['Symbol'], ['Tahoma'], ['Terminal'], ['Times New Roman'],
				['Trebuchet MS'], ['Verdana'], ['Webdings'], ['Wingdings']]
		})
	},
	triggerAction: 'all',
	mode: 'local',
	valueField: 'value',
	displayField: 'value',
	focusAfterSelectEvent: true,
	constructor : function(config){
		config = config || {};
		fontStoreType = config.fontStoreType || 'size';
		config.store = config.store || this.storeList[fontStoreType];
		config.editable = config.editable || fontStoreType == 'size';
		if (fontStoreType == 'size') {
			config.maskRe = config.maskRe || new RegExp('[0-9]');
		}
		if (fontStoreType == 'family' && !config.tpl) {
			config.tpl = new Ext.XTemplate(
				'<tpl for="."><div class="x-combo-list-item"' +
				'<tpl if="value!=\'Webdings\' && value!=\'Wingdings\'"> style="font-family:{value}"</tpl>' +
				'>{value}</div></tpl>'
			);
		}
		Sb.form.FontSelectBox.superclass.constructor.call(this, config);
	},

	onViewClick : function(doFocus){
		var index = this.view.getSelectedIndexes()[0],
		s = this.store,
		r = s.getAt(index);
		if(r){
			this.onSelect(r, index);
		}else {
			this.collapse();
		}
		if (this.focusAfterSelectEvent) {
			if(doFocus !== false){
				this.el.focus();
			}
		}
		this.focusAfterSelectEvent = true;
	},

	onResize : function(w, h){
		Ext.form.TriggerField.superclass.onResize.call(this, w, h);
		var tw = this.getTriggerWidth();
		if (Ext.isNumber(w)) {
			this.el.setWidth(w - tw);
		}

		if (isNaN(w)) {
			this.wrap.setWidth(this.el.getWidth() + tw);
		}
	}
});
Ext.reg('sbfontselect', Sb.form.FontSelectBox);
