/**
 * @class Sb.form.SelectBox
 * @extends Ext.form.ComboBox
 * A selectbox control.
 * @constructor
 * Create a new SelectBox.
 * @param {Object} config Configuration options
 * @xtype sbselectbox
 */
Sb.form.SelectBox = Ext.extend(Ext.form.ComboBox, {
	defaultAutoCreate: {
		tag: 'div'
	},

	cls: 'sb-selectbox',

	listClass: 'sb-selectbox-list',

	currentValueClass: 'sb-selectbox-current-value',

	constructor: function(config) {
		config.cls = this.cls + ' ' + (config.cls || '');
		config.listClass = this.listClass + ' ' + (config.listClass || '');
		Sb.form.SelectBox.superclass.constructor.call(this, config);
	},

	initComponent: function() {
		Sb.form.SelectBox.superclass.initComponent.call(this);
		this.editable = false;
	},

	bindStore: function(store, initial) {
		Sb.form.SelectBox.superclass.bindStore.apply(this, arguments);

		if (!store || this.store.isCurrentValueBinded) {
			return;
		}

		var currentRecordValue	= null,
			currentSelectedItem	= null;

		this.view.on('selectionchange', function(dataView, selectionEls) {
			var currentRecord		= this.store.getAt(this.view.getSelectedIndexes()[0]);
			if (currentRecord) {
				currentRecordValue	= currentRecord.data[this.valueField];
			}
			if (this.value == currentRecordValue) {
				if (currentSelectedItem) {
					currentSelectedItem.removeClass(this.currentValueClass);
				}
				currentSelectedItem = Ext.get(selectionEls[0]);
				if (currentSelectedItem) {
					currentSelectedItem.addClass(this.currentValueClass);
				}
			}

		}, this);

		this.store.isCurrentValueBinded = true;
	},

	onBeforeLoad: function() {
		if (!this.hasFocus) {
			return;
		}
		this.innerList.update(this.loadingText ?
				'<div class="loading-indicator">'+this.loadingText+'</div>' : '');
		this.selectedIndex = -1;
	},

	onLoad: function() {
		if (!this.hasFocus) {
			return;
		}
		if (this.store.getCount() > 0 || this.listEmptyText) {
			this.expand();
			this.restrictHeight();
			this.selectByValue(this.value, true);
		} else {
			this.collapse();
		}
	},

	setValue: function(value) {
		Sb.form.SelectBox.superclass.setValue.call(this, value);

		if (this.rendered) {
			var recordFoundOnTheStoreByValue = this.findRecord(this.valueField, value);
			if (recordFoundOnTheStoreByValue) {
				this.tpl.overwrite(this.el, recordFoundOnTheStoreByValue.data);
			} else {
				var data = {};
				data[this.valueField] = value;
				this.tpl.overwrite(this.el, data);
			}
		}

		return this;
	},

	getRawValue: function() {
		return this.getValue();
	}

	/**
	 * @cfg {Boolean} editable @hide
	 */
});
Ext.reg('sbselectbox', Sb.form.SelectBox);