/**
 * @class Sb.model.HeaderWidget
 * @extends Sb.model.SharedWidget
 * Class for header model
 * @constructor
 * @param {Object} config The configuration options for the widget model
 */
Sb.model.HeaderWidget = Ext.extend(Sb.model.SharedWidget, {

	/**
	 * @cfg {String} type
	 * defaults to <tt>Header</tt>
	 */

	constructor: function(config) {
		config = Ext.apply({
			type: 'header'
		}, config);

		this.addEvents(
			/**
			 * @event heightchange
			 * Fires after height has changed.
			 */
			'heightchange',
			/**
			 * @event imagechange
			 * Fires after the image and it's properties has changed.
			 */
			'imagechange',
			/**
			 * @event imagefitchange
			 * Fires after the image fit has changed.
			 */
			'imagefitchange',
			/**
			 * @event logochange
			 * Fires after the logo and it's properties has changed.
			 */
			'logochange',
			/**
			 * @event titlechange
			 * Fires after the title and it's properties has changed.
			 */
			'titlechange',
			/**
			 * @event subtitlechange
			 * Fires after the subtitle and it's properties has changed.
			 */
			'subtitlechange'
		);
		Sb.model.HeaderWidget.superclass.constructor.call(this, config);
	},

	/**
	 * Fires on changing of single property change to allow us fire up header widget change events.
	 * @param {String} key
	 */
	onUnserializePropertyChange: function (key) {
		var matches = key.match(/^(height|image|logo|title|subtitle)/);
		if ('imageFit' == key) {
			this.fireEvent('imagefitchange');
		} else if (matches) {
			this.fireEvent(matches[0] + 'change');
		}
	}
});
