/**
 * @class Sb.model.NavigationWidget
 * @extends Sb.model.SharedWidget
 * Class for navigation model
 * @constructor
 * @param {Object} config The configuration options for the widget model
 */
Sb.model.NavigationWidget = Ext.extend(Sb.model.SharedWidget, {

    /**
     * @cfg {String} type
     * defaults to <tt>navigation</tt>
     */

	constructor: function(config) {
		config = Ext.apply({
			type: 'navigation'
		}, config);

		this.addEvents(
			/**
			 * @event levelchange
			 * Fires after the levels/depth has changed.
			 */
			'levelchange',
			/**
			 * @event fontchange
			 * Fires after the link font size/family has changed.
			 */
			'fontchange',
			/**
			 * @event alignmentchange
			 * Fires after the menu alignment has changed.
			 */
			'alignmentchange',
			/**
			 * @event colorchange
			 * Fires after the item colors/decorations has changed.
			 */
			'colorchange'
		);

		Sb.model.NavigationWidget.superclass.constructor.call(this, config);
	},

	/**
	 * Level-independent getter.
	 *
	 * @param {String} propname
	 * @param {Number} level When it`s undefined, it will returns prop for level 1.
	 */
	getByLevel: function (propname, level) {
		if (undefined == level) {
			level = 1;
		}
		return this.data['level' + level + propname];
	},

	/**
	 * Level-independent getter.
	 *
	 * @param {String} propname
	 * @param {String} propvalue
	 * @param {Number} level When it`s undefined, it will set props on all levels 1 and 2.
	 */
	setByLevel: function (propname, propvalue, level) {
		var levels = [level];
		if (undefined == level) {
			levels = [1, 2];
		}
		Ext.each(levels, function (level) {
			this.setProperty('level' + level + propname, propvalue);
		}, this);
	},

	/**
	 * Fires on changing of single property change to allow us fire up navigation widget change events.
	 * @param {String} key
	 */
	onUnserializePropertyChange: function (key) {
		if (key == 'level' || key == 'depth') {
			this.fireEvent('levelchange');
		} else if (/^level[12]Font(Family|Size)$/.test(key)) {
			this.fireEvent('fontchange');
		} else if (/^level[12]Alignment$/.test(key)) {
			this.fireEvent('alignmentchange');
		} else {
			this.fireEvent('colorchange');
		}
	}
});
