/**
 * @class Sb.widget.ContainerVertical
 * @extends Ext.Container
 * @constructor
 * @param {Object} config The config object
 * @xtype container_vertical_widget
 */
Sb.widget.ContainerVertical = Ext.extend(Ext.Container, {
	/**
	 * @cfg {Sb.model.ContainerVerticalWidget} model
	 * The {@link Sb.model.Widget} to bind this Widget to.
	 */

	/**
	 * object already loaded borders
	 */
	loadedBorders: {},

	initComponent: function() {
		this.items = [];
		this.model.widgets.each(function(widget) {
			this.items.push({
				xtype : widget.data.type + '_widget',
				model : widget
			});
		}, this);
		this.settedBorder = '';
		Sb.widget.ContainerVertical.superclass.initComponent.call(this);
	},

	onRender: function(ct) {
		var partial = new Sb.PartialTemplate(Sb.Editor.model.data.template.partials['placeholder'].replace(
				'{content}',
				Sb.Editor.model.data.template.partials['container']
		));
		var background = encodeURIComponent(Sb.Editor.model.design.data.background[this.model.data.region + 'Background']);
		partial.update(ct.dom, {
			'id'	: this.model.data.region,
			'class'	: this.getCssClasses(),
			'backgroundImage': Sb.Config.templatesUrl + '/' + encodeURIComponent(Sb.Editor.model.data.template.code) +
				'/decorations/background/' + background + '/images/' + background + '.png',
			// :TODO: asinko 20100305: temporary dirty hack for compilation container partial templates
			'items'	: [{content: '<div id="container-tmp-content"></div>'}]
		});
		this.el = ct.first();

		this.contentElement = Ext.get('container-tmp-content').parent();
		this.contentElement.update('');
	},

	afterRender: function() {
		Sb.widget.ContainerVertical.superclass.afterRender.call(this);

		this.items.each(function(item) {
			item.render(this.contentElement);
		}, this);

		this.dt = new Ext.dd.DropTarget(this.el.id, {
			copy	: false,
			ddGroup	: 'widgets',

			notifyDrop: function(source, e, data) {
				if (!this.isDroppable(source, e)) {
					return false;
				}
				if (undefined == Sb.Editor.siteView.ddSpacer.position) {
					return;
				}
				this.onDrop(source, data);
				return true;
			}.createDelegate(this),

			notifyOver: function(source, e, data) {
				if (!this.isDroppable(source, e)) {
					Sb.Editor.siteView.ddSpacer.remove();
					return Ext.dd.DropZone.prototype.dropNotAllowed;
				}
				this.notifyOver(source, e.getXY());
				return Ext.dd.DropZone.prototype.dropAllowed;
			}.createDelegate(this),

			notifyOut: function(source, e, data) {
				Sb.Editor.siteView.ddSpacer.remove();
				this.refreshCache();
			}.createDelegate(this)
		});
		this.dt.addToGroup('widgetButtons');
		this.onBackgroundColorChange();
		this.onDesignDecorationChange();

		this.mon(Sb.Editor.model.design, {
			scope: this,
			decorationchange: this.onDesignDecorationChange,
			backgroundcolorchange: this.onBackgroundColorChange
		});
		this.mon(this.model.widgets, {
			scope: this,
			add: this.onModelWidgetAdd,
			remove: this.onModelWidgetRemove
		});
		this.mon(this.el, 'contextmenu', Sb.Editor.Ui.onContextMenu, Sb.Editor.Ui, {
			type: 'container',
			region: this.model.data.region
		});
	},

	onDesignDecorationChange: function () {
		this.el.dom.className = this.getCssClasses();
		this.setBorder();
	},

	/**
	 * apply new styles for borders
	 * @param {String} path
	 * @param {Object} result
	 * @param {String} border
	 */
	setStylesForBorder: function(path, result, border) {
		this.el.down('div.top').setStyle({
			'height': result.height + 'px',
			'padding-left': result.width + 'px',
			'background': 'transparent url(\'' + path +  '/images/' + border + '-top-left.png\') top left no-repeat'
		}).down('div').setStyle({
			'height': result.height + 'px',
			'padding-right': result.width + 'px',
			'background': 'transparent url(\'' + path +  '/images/' + border + '-top-right.png\') top right no-repeat'
		}).down('div').setStyle({
			'height': result.height + 'px',
			'background': 'transparent url(\'' + path +  '/images/' + border + '-top.png\') top left repeat-x'
		});

		this.el.down('div.side').setStyle({
			'padding-left': result.width + 'px',
			'background': 'transparent url(\'' + path +  '/images/' + border + '-left.png\') top left repeat-y'
		}).down('div').setStyle({
			'padding-right': result.width + 'px',
			'background': 'transparent url(\'' + path +  '/images/' + border + '-right.png\') top right repeat-y'
		});

		this.el.down('div.bottom').setStyle({
			'height': result.height + 'px',
			'padding-left': result.width + 'px',
			'background': 'transparent url(\'' + path +  '/images/' + border + '-bottom-left.png\') bottom left no-repeat'
		}).down('div').setStyle({
			'height': result.height + 'px',
			'padding-right': result.width + 'px',
			'background': 'transparent url(\'' + path +  '/images/' + border + '-bottom-right.png\') bottom right no-repeat'
		}).down('div').setStyle({
			'height': result.height + 'px',
			'background': 'transparent url(\'' + path +  '/images/' + border + '-bottom.png\') bottom left repeat-x'
		});
	},

	setBorder: function() {
		var border = Sb.Editor.model.design.data['border'][this.model.data.region + 'Border'];
		if (border != this.settedBorder) {
			this.settedBorder = border;
			var cacheKey = border.replace(/-/g, '', border);
			var path = Sb.Config.templatesUrl + '/' +
				encodeURIComponent(Sb.Editor.model.data.template.code) +
				'/decorations/border/' + encodeURIComponent(border);
			if (!this.loadedBorders[cacheKey]) {
				Sb.DesignTemplateService.getDecorationInfo('generic', 'border', border, function(result) {
					this.loadedBorders[cacheKey] = result;
					this.setStylesForBorder(path, this.loadedBorders[cacheKey], border);
				}.createDelegate(this));
			} else {
				this.setStylesForBorder(path, this.loadedBorders[cacheKey], border);
			}
		}
	},

	/**
	 * Update background color.
	 */
	onBackgroundColorChange: function() {
		this.el.setStyle(
			'background-color',
			Sb.Editor.model.design.data.backgroundColor[this.model.data.region + 'Color']
		);
	},

	onModelWidgetAdd: function(index, widget, key) {
		this.insert(index, Ext.create({
			xtype : widget.data.type + '_widget',
			model : widget
		}));
		this.doLayout();
	},

	onModelWidgetRemove: function(widget) {
		this.items.each(function(item) {
			if (item.model === widget) {
				item.destroy();
			}
		});
	},

	/**
	 * Return css classes of container
	 *
	 * @return {String} Css classes of container
	 */
	getCssClasses: function() {
		var region			= this.model.data.region,
			backgroundKey	= region + 'Background',
			designSettings	= Sb.Editor.model.design.getSettings(),
			cssClasses		= region + ' background';

		if (this.el) {
			cssClasses		= this.el.dom.className
								.replace(/\s+background\d*\-[^\s]+/g, '');	// Clean-up background classes
		} else {
			if (/column\d/.test(region)) {
				cssClasses	+= ' column';
			}
		}

		if (designSettings.background[backgroundKey]) {
			cssClasses		+= ' ' + designSettings.background[backgroundKey];
		}

		return cssClasses;
	},

	getLayoutTarget: function() {
		return this.contentElement;
	},

	/**
	 * Checks if we can drop to target/container
	 *
	 * @param {Ext.dd.DragSource} source
	 * @param {Event} e
	 * @returns {Boolean}
	 */
	isDroppable: function(source, e) {
		if (e.getTarget('.sb-editor-panel')) {
			// Not notify if over our main window
			return false;
		}
		var widgetType = (source.widget) ? source.widget.model.data.type : source.dragData.widgetType;
		if (!Sb.WidgetTypeManager.get(widgetType).isDroppableTo(this.model.data.region)) {
			// Not notify if widget not droppable to this container
			return false;
		}
		return true;
	},

	/**
	 * The function calls continuously while source is being dragged over the container.
	 * @param {Ext.dd.DragSource} source
	 * @param {Array} draggedPosition The xy values like [x, y]
	 */
	notifyOver: function(source, draggedPosition) {
		var isWidget = source.el.hasClass('sb-widget');
		var draggedHeight = 1;

		// find insert position
		var currentWidget, isInsertBefore = false, overSelf = false;
		var widgetsCount = this.items.length;

		for (var newPosition = 0; newPosition < widgetsCount; newPosition++) {
			currentWidget =  this.items.get(newPosition);
			var currentWidgetHeight = currentWidget.el.getHeight();
			var currentWidgetPosition = currentWidget.el.getXY();

			if (isWidget && source.el.id == currentWidget.el.id) {
				overSelf = true;
			} else if (currentWidgetPosition[1] + currentWidgetHeight / 2 > draggedPosition[1] + draggedHeight / 2) {
				isInsertBefore = true;
				break;
			}
		}

		newPosition			= (isInsertBefore && currentWidget ? newPosition : this.items.getCount()) + (overSelf ? -1 : 0);
		var isSamePosition	= (overSelf && Sb.Editor.siteView.ddSpacer.position == newPosition);
		if (!isSamePosition) {
			Sb.Editor.siteView.ddSpacer.position = newPosition;

			var insertPlace, contextElement;
			if (currentWidget) {
				insertPlace = isInsertBefore ? 'beforeBegin' : 'afterEnd';
				contextElement = currentWidget.el.dom;
			} else {
				insertPlace = 'afterBegin';
				contextElement = this.contentElement.dom;
			}
			Sb.Editor.siteView.ddSpacer.insert(insertPlace, contextElement);
			this.refreshCache();
		}
	},

	/**
	 * Calls once when dragged item has been dropped on container
	 * @param {Ext.dd.DragSource} dragSource
	 * @param {Object} data An object containing arbitrary data supplied by the drag source
	 */
	onDrop: function(dragSource, data) {
		var position = Sb.Editor.siteView.ddSpacer.position;
		Sb.Editor.siteView.ddSpacer.remove();
		this.refreshCache();
		if (dragSource.widget) {
			this.model.insert(position, dragSource.widget.model, dragSource.widget.ownerCt.model);
			return;
		}

		this.model.createChild(data.widgetType, position);
	},

	/**
	 * Clean up location cache
	 */
	refreshCache: function() {
		Ext.dd.DragDropMgr.refreshCache({widgetButtons: true});
	}
});

Ext.reg('container_vertical_widget', Sb.widget.ContainerVertical);