/**
 * @class Sb.widget.Header
 * @extends Sb.widget.Widget
 * Class for header widget
 * @constructor
 * @param {Object} config The config object
 * @xtype header_widget
 */
Sb.widget.Header = Ext.extend(Sb.widget.Widget, {

	cls: 'sb-widget-header',

	initComponent: function() {
		this.addEvents(
			"headerloaded"
		);
		Sb.widget.Header.superclass.initComponent.apply(this, arguments);
	},

	onRender: function() {
		Sb.widget.Header.superclass.onRender.apply(this, arguments);

		this.menu.add({
			text: SbGetLocaleByKey('cmWidgetHeaderImage'),
			iconCls: 'sb-image-icon',
			scope: this,
			handler: function() {
				this.startEdit();
				Sb.widget.Header.editPanel.tabs.setActiveTab(0); // Image panel
			}
		});
		this.menu.add({
			text: SbGetLocaleByKey('cmWidgetHeaderTitle'),
			iconCls: 'sb-title-icon',
			scope: this,
			handler: function() {
				this.startEdit();
				Sb.widget.Header.editPanel.tabs.setActiveTab(1); // Title panel
			}
		});
		this.menu.add({
			text: SbGetLocaleByKey('cmWidgetHeaderLogo'),
			iconCls: 'sb-show-icon',
			scope: this,
			handler: function() {
				this.startEdit();
				Sb.widget.Header.editPanel.tabs.setActiveTab(2); // Logo panel
			}
		});

		Ext.DomHelper.append(this.contentElement, {
			tag: 'table',
			cls: 'sb-widget-header-logo',
			cn: [{
				tag: 'tr',
				cn: [{
					tag: 'td',
					cn: {
						tag: 'img',
						id: 'header-logo-image',
						cls: 'logo-image',
						src: ''
					}
				}, {
					tag: 'td',
					cn: {
						cls: 'header-titles',
						cn: [{
							id: 'id-header-title',
							cls: 'header-title'
						}, {
							id: 'id-header-subtitle',
							cls: 'header-subtitle'
						}]
					}
				}]
			}]
		});

		this.onImageChange();
		this.onImageFitChange();
		this.onLogoChange();
		this.onTitleChange();
		this.onSubtitleChange();
	},

	initEvents: function () {
		Sb.widget.Header.superclass.initEvents.call(this);

		this.model.on('heightchange', this.onHeightChange, this);
		this.model.on('imagechange', this.onImageChange, this);
		this.model.on('imagefitchange', this.onImageFitChange, this);
		this.model.on('logochange', this.onLogoChange, this);
		this.model.on('titlechange', this.onTitleChange, this);
		this.model.on('subtitlechange', this.onSubtitleChange, this);
		this.model.on('save', function () {
			Sb.Editor.Ui.mainWindow.statusBar.setMessage(
				SbGetLocaleByKey('messageChangesSaved')
			);
		});
	},

	onHeightChange: function() {
		this.contentElement.setHeight(
			(this.model.data.imageName == 'fallback') ? this.model.data.height : 'auto'
		);

		if (Sb.widget.Widget.editCurrent == this) {
			Sb.widget.Header.editPanel.reanchor();
		}
	},

	onImageChange: function() {
		var data = this.model.data;
		var path = '' == data.imagePath
			? Sb.Config.headersUrl + '/' + data.imageName + '/header.png'
			: Sb.Editor.model.getUrl() + data.imagePath;

		//add timestamp for custom headers to avoid caching
		if (data.imageTime) {
			path += '?' + data.imageTime;
		}

		// destroy image and create new for change image (because of 'onload' bug in ie)
		if (this.imageEl) {
			this.imageEl.un('load');
			Ext.destroy(this.imageEl);
		}
		this.imageEl = this.contentElement.createChild(
			{tag: 'img', cls: 'header-image'}, this.headerContentEl
		);
		if (Sb.widget.Header.imageSize) {
			this.imageEl.setSize(Sb.widget.Header.imageSize);
		}

		this.imageEl.on('load', this.onImageLoad, this);
		this.imageEl.set({'src': path});

		this.onHeightChange();
	},

	onImageLoad: function(){
		this.imageEl.setStyle({
			'top': this.model.data.imagePositionTop + 'px',
			'left': this.model.data.imagePositionLeft + 'px'
		});
		this.onImageFitChange();
		this.fireEvent('headerloaded', this);
	},

	onImageFitChange: function() {
		this.imageEl.setSize('auto', 'auto');
		Sb.widget.Header.imageSize = this.imageEl.getSize();

		this.imageEl.setStyle({
			'width': (this.model.data.imageFit == '1') ? '100%' : 'auto'
		});

		if (Sb.widget.Widget.editCurrent == this) {
			Sb.widget.Header.editPanel.reanchor();
		}
	},

	onLogoChange: function() {
		var data		= this.model.data;
		var hiddenCls	= 'header-logo-hidden';
		var table = this.contentElement.down('table');
		if (data.logoIsHidden == 1) {
			table.addClass(hiddenCls);
		} else {
			table.removeClass(hiddenCls);
		}
		var path = '' == data.logoPath
			? Sb.Config.logosUrl + '/' + data.logoName + '/logo.png'
			: Sb.Editor.model.getUrl() + data.logoPath;
		Ext.get('header-logo-image').set({'src': path});
		table.setStyle({
			'top': data.logoPositionTop + 'px',
			'left': data.logoPositionLeft + 'px'
		});
	},

	onTitleChange: function() {
		var data = this.model.data;
		var title = Ext.get('id-header-title');
		title.update(
			Ext.util.Format
				.htmlEncode(data.title)
				.replace(/\s/g, '&nbsp;')		// NOTE: austimenko: 2010-08-30 Bug in IE7, that normalizes white-space
		);
		title.setStyle({
			'color': data.titleFontColor,
			'font-weight': (data.titleFontBold == 1) ? 'bold' : 'normal',
			'font-size': data.titleFontSize + 'px',
			'font-style': (data.titleFontItalic == 1) ? 'italic' : 'normal',
			'font-family': '"' + data.titleFontFamily + '"',
			'text-decoration': (data.titleFontUnderline == 1) ? 'underline' : 'none'
		});
	},

	onSubtitleChange: function() {
		var data = this.model.data;
		var subTitle = Ext.get('id-header-subtitle');
		subTitle.update(
			Ext.util.Format
				.htmlEncode(data.subtitle)
				.replace(/\s/g, '&nbsp;')		// NOTE: austimenko: 2010-08-30 Bug in IE7, that normalizes white-space
		);
		subTitle.setStyle({
			'color': data.subtitleFontColor,
			'font-weight': (data.subtitleFontBold == 1) ? 'bold' : 'normal',
			'font-size': data.subtitleFontSize + 'px',
			'font-style': (data.subtitleFontItalic == 1) ? 'italic' : 'normal',
			'font-family': '"' + data.subtitleFontFamily + '"',
			'text-decoration': (data.subtitleFontUnderline == 1) ? 'underline' : 'none'
		});
	},

	onStartEdit: function() {
		this.memento = this.model.saveToMemento();
		if (!Sb.widget.Header.editPanel) {
			Sb.widget.Header.editPanel = new Sb.widget.HeaderEditPanel();
		}
		Sb.widget.Header.editPanel.show(this);
	},

	onCompleteEdit: function() {
		this.model.saveIfChanged(this.memento);
	},

	onCancelEdit: function() {
		this.model.restoreFromMemento(this.memento);
	},

	onHideEdit: function() {
		Sb.widget.Header.editPanel.hide();
	},

	onDestroy: function(){
		if (this.rendered) {
			this.model.un('heightchange', this.onHeightChange, this);
			this.model.un('imagechange', this.onImageChange, this);
			this.model.un('imagefitchange', this.onImageFitChange, this);
			this.model.un('logochange', this.onLogoChange, this);
			this.model.un('titlechange', this.onTitleChange, this);
			this.model.un('subtitlechange', this.onSubtitleChange, this);
		}
		Sb.widget.Header.superclass.onDestroy.call(this);
	},

	onDocumentClick: function(event) {
		if (event.getTarget('.sb-color-dropdown')
			|| event.getTarget('.sb-widget-edit-panel-header')
			|| event.getTarget('.x-combo-list')
		) {
			return;
		}

		Sb.widget.Header.superclass.onDocumentClick.call(this, event);
	}
});

Ext.reg('header_widget', Sb.widget.Header);