Sb.widget.HeaderEditPanel = Ext.extend(Sb.widget.WidgetEditPanel, {
	/**
	 * @cfg {Number} imageListPageSize Image per page in images list.
	 */
	imageListPageSize: 10,

	initComponent: function() {
		this.headerImageStore = new Ext.data.JsonStore({
			typeStore: 'library',
			fields: ['name', 'path', 'keywords'],
			baseParams: {
				filter: {}
			}
		});
		this.headerImageCustomStore = new Ext.data.JsonStore({
			typeStore: 'custom',
			fields: ['name', 'time', 'path'],
			listeners: {
				scope: this,
				remove: function(store, record, index) {
					var newData = store.proxy.data;
					newData.splice(index, 1);
					store.proxy = new Ext.ux.data.PagingMemoryProxy(newData);
				}
			}
		});
		var fontStorage = new Ext.data.ArrayStore({
			fields: ['value'],
			data: [['Andale Mono'], ['Arial'], ['Arial Black'], ['Book Antiqua'], ['Comic Sans MS'], ['Courier New'],
				['Georgia'], ['Helvetica'], ['Impact'], ['Symbol'], ['Tahoma'], ['Terminal'], ['Times New Roman'],
				['Trebuchet MS'], ['Verdana'], ['Webdings'], ['Wingdings']]
		});
		var fontSizeStorage = new Ext.data.ArrayStore({
			fields: ['value'],
			data: [['8'], ['9'], ['10'], ['11'], ['12'], ['14'], ['16'], ['18'], ['20'], ['22'], ['24'], ['26'], ['28'],
				['36'], ['48'], ['72']
			]
		});

		Ext.apply(this, {
			width: 584,
			cls: 'sb-widget-edit-panel-header',
			items: {
				autoHeight: true,
				xtype: 'tabpanel',
				items: [{
					autoHeight: true,
					cls: 'sb-widget-header-image-panel',
					title: SbGetLocaleByKey('widgetHeaderImageTabTitle'),
					items: [{
						height: 87,
						layout: 'column',
						defaults: {
							layout: 'form',
							labelAlign: 'top'
						},
						items: [{
							width: 157,
							items: [{
								xtype: 'radiogroup',
								id: 'widgetHeaderImageSelection',
								fieldLabel: SbGetLocaleByKey('widgetHeaderImageSelectionTitle'),
								labelSeparator: '',
								columns: 1,
								defaults: {
									xtype: 'radio',
									name: 'selection'
								},
								items: [{
									boxLabel: SbGetLocaleByKey('widgetHeaderImageSelectionLibraryTitle'),
									inputValue: 'library'
								}, {
									boxLabel: SbGetLocaleByKey('widgetHeaderImageSelectionFileTitle'),
									inputValue: 'file'
								}, {
									boxLabel: SbGetLocaleByKey('widgetHeaderImageSelectionNoTitle'),
									inputValue: 'no'
								}],
								listeners: {
									scope: this,
									change: this.onImageSelectionChange
								}
							}]
						}, {
							width: 238,
							defaults: {
								layout: 'form',
								labelAlign: 'top'
							},
							items: [{
								id: 'widgetHeaderImageListContainer',
								items: {
									xtype: 'sbselectbox',
									id: 'widgetHeaderImageList',
									fieldLabel: SbGetLocaleByKey('widgetHeaderImageListTitle'),
									labelSeparator: '',
									triggerAction: 'all',
									mode: 'local',
									width: 235,
									valueField: 'path',
									cls: 'sb-header-images',
									listClass: 'sb-header-images-list',
									listWidth: 442,
									maxHeight: 338,
									lazyInit: false,
									listEmptyText: SbGetLocaleByKey('widgetHeaderImageListNoMatch'),
									tpl: new Ext.XTemplate(
										'<tpl for=".">' +
										'<div class="sb-selectbox-list-item sb-header-images-list-item" >' +
											'<div>' +
												'<img src="{path}" height="57">' +
												'<div class="sb-selectbox-list-item-delete"><span>X</span></div>' +
											'</div>' +
										'</div></tpl>'
									),
									itemSelector: 'div.sb-selectbox-list-item',
									collapse: function(){
										var dialog = Ext.getCmp('widgetHeaderImageList').dialog;
										if (dialog && dialog.isVisible()) {
											return;
										}
										Sb.form.SelectBox.superclass.collapse.call(this);
									},
									listeners: {
										scope: this,
										select: function(combo, record) {
											if ('file' == Ext.getCmp('widgetHeaderImageSelection').getValue().getGroupValue()) {
												this.widget.model.unserialize({
													imageName: '',
													imagePath: '/' + Sb.Config.headerPath + '/' + record.data.name,
													imageTime: record.data.time
												});
											} else {
												this.widget.model.unserialize({
													imageName: record.data.name,
													imagePath: ''
												});
											}
										},
										afterrender: function(selectbox) {
											selectbox.isClickedDelete = false;
											selectbox.header = selectbox.list.createChild(
												{cls: 'x-combo-list-hd'},
												selectbox.innerList
											);
											selectbox.searchField = new Ext.form.TextField({
												emptyText: SbGetLocaleByKey('widgetHeaderImageListEmptySearch'),
												id: 'widgetHeaderImageListFilter',
												hideMode: 'visibility',
												width: 220,
												renderTo: selectbox.header,
												listeners: {
													scope: this,
													valid: function(field) {
														if (this.hidden) {
															return;
														}
														var words = field.getValue().trim().split(/[~!@#%^&:%_, <>\-\/]/);
														Ext.iterate(words, function(word, index) {
															words[index] = Ext.escapeRe(word);
														});
														var newKeywords = new RegExp('(^|,)(' + words.join('|') + ')', 'ig');
														var currentKeywords = this.headerImageStore.baseParams.filter.keywords;
														if (currentKeywords && currentKeywords.toString() == newKeywords.toString()) {
															return;
														}
														this.headerImageStore.baseParams.filter.keywords = newKeywords;
														this.headerImageStore.load({
															params: {
																start: 0,
																limit: this.imageListPageSize
															}
														});
													}
												}
											});
											selectbox.assetHeight += selectbox.header.getHeight();

											selectbox.footer = selectbox.list.createChild({cls: 'x-combo-list-ft'});
											selectbox.pageTb = new Ext.PagingToolbar({
												store: selectbox.store,
												pageSize: this.imageListPageSize,
												renderTo: selectbox.footer,
												listeners: {
													afterrender: function(pageTb) {
														pageTb.refresh.hide();
													}
												}
											});
											if (Ext.isIE7) {
												selectbox.footer.repaint();
											}
											selectbox.assetHeight += selectbox.footer.getHeight();

											selectbox.view.mon(selectbox.view.getTemplateTarget(), {
												"mouseover": selectbox.view.onMouseOver,
												"mouseout": selectbox.view.onMouseOut,
												scope: selectbox.view
											});

											selectbox.view.on('mouseenter', function(dataView, index, node, evt) {
												var overHeaderName = selectbox.store.data.items[index].data.name;
												if (dataView.store.typeStore == 'library' ||
													selectbox.getValue().indexOf(overHeaderName) >= 0) {
													return;
												}
												var deleteButton = Ext.get(node).down('div').down('div.sb-selectbox-list-item-delete');
												deleteButton.show();
												deleteButton.on('click', function(){
													selectbox.isClickedDelete = true;
												});
											}, this);

											selectbox.view.on('mouseleave', function(dataView, index, node, evt) {
												if (selectbox.store.typeStore == 'library') {
													return;
												}
												var deleteButton = Ext.get(node).down('div').down('div.sb-selectbox-list-item-delete');
												deleteButton.hide();
												deleteButton.removeAllListeners();
											});

											selectbox.on('beforeselect', function(selectbox, record, index) {
												if (selectbox.isClickedDelete) {
													selectbox.isClickedDelete = false;
													selectbox.dialog = Ext.Msg.show({
														title	: SbGetLocaleByKey('widgetHeaderDialogTitleDeleteImage'),
														msg		: SbGetLocaleByKey('widgetHeaderDialogMsgConfirmDeleteImage'),
														closable: false,
														buttons	: Ext.Msg.YESNO,
														scope	: this,
														fn		: function(btn) {
															if (btn == 'yes') {
																this.onDeleteHeader(selectbox, record, index);
															}
														}
													});
													return false;
												}
											}, this);
										}
									}
								}
							}, {
								id: 'widgetHeaderImageHeightContainer',
								layout: 'column',
								items: [{
									xtype: 'box',
									autoEl: 'span',
									cls: 'sb-widget-header-image-height-label',
									html: SbGetLocaleByKey('widgetHeaderImageHeightTitle')
								}, {
									xtype		: 'numberfield',
									id			: 'widgetHeaderImageHeight',
									allowBlank	: false,
									minValue	: 50,
									maxValue	: 500,
									width		: 40,
									fieldLabel	: SbGetLocaleByKey('widgetHeaderImageHeightTitle'),
									listeners: {
										scope: this,
										valid: function(textfield) {
											this.widget.model.unserialize({
												height: textfield.getValue()
											});
										}
									}
								}, {
									xtype: 'box',
									autoEl: 'span',
									cls: 'sb-widget-header-image-height-measure',
									html: SbGetLocaleByKey('px')
								}]
							}]
						}, {
							id: 'sizeColumn',
							width: 177,
							items: [{
								xtype: 'checkbox',
								id: 'widgetHeaderImageFit',
								boxLabel: SbGetLocaleByKey('widgetHeaderImageFitSizeTitle'),
								listeners: {
									scope: this,
									check: function(checkbox, checked) {
										this.widget.model.unserialize({
											imageFit: checked ? 1 : 0
										});
									},
									afterrender: function(checkbox) {
										var fn = Sb.Editor.Ui.mainWindow.setHelperMessage.createDelegate(
											Sb.Editor.Ui.mainWindow, [SbGetLocaleByKey('widgetHeaderImageFitSizeTooltip')]
										);
										checkbox.el.on('mouseenter', fn);
										checkbox.el.on('mouseleave', Sb.Editor.Ui.mainWindow.restoreHelperMessage, Sb.Editor.Ui.mainWindow);

										checkbox.labelEl.on('mouseenter', fn);
										checkbox.labelEl.on('mouseleave', Sb.Editor.Ui.mainWindow.restoreHelperMessage, Sb.Editor.Ui.mainWindow);
									}
								}
							}, {
								xtype: 'displayfield',
								id: 'widgetHeaderImageSizeHint',
								hideLabel: true,
								value: String.format(SbGetLocaleByKey('widgetHeaderImageSizeHint'), 0, 0)
							}]
						}]
					}, {
						id: 'widgetHeaderFileupload',
						cls: 'sb-widget-header-image-upload',
						xtype: 'form',
						fileUpload: true,
						labelAlign: 'top',
						items: {
							xtype: 'fileuploadfield',
							regex: new RegExp('\\.(' + Sb.Config.headerValidExtensions.join('|') + ')$', 'i'),
							msgTarget: 'under',
							anchor: '0',
							buttonText: SbGetLocaleByKey('widgetHeaderUploadButton'),
							fieldLabel: SbGetLocaleByKey('widgetHeaderUploadLabel'),
							emptyText: SbGetLocaleByKey('widgetHeaderUploadEmptyText'),
							labelSeparator: '',
							name: 'headerimage',
							listeners: {
								scope: this,
								fileselected: this.onUploadFile,
								afterrender: function(){
									Ext.getCmp('widgetHeaderFileupload').hide();
								}
							}
						}
					}]
				}, {
					cls: 'sb-widget-header-title-panel',
					autoHeight: true,
					title: SbGetLocaleByKey('widgetHeaderTitleTabTitle'),
					layout: 'column',
					listeners: {
						scope: this,
						activate: function() {
							this.updateTitleFormVisibility();
							this.initTitleTab();
						}
					},
					items: [{
						columnWidth: 0.2,
						xtype: 'radiogroup',
						cls: 'sb-header-title-field-group',
						id: 'widgetHeaderTitleFieldGroup',
						columns: 1,
						defaults: {
							xtype: 'radio',
							name: 'widgetHeaderTitleFieldGroup'
						},
						items: [{
							boxLabel: SbGetLocaleByKey('widgetHeaderTitleFieldTitle'),
							id: 'widgetHeaderTitleFieldTitle',
							value: 'Title'
						}, {
							boxLabel: SbGetLocaleByKey('widgetHeaderTitleFieldSubtitle'),
							id: 'widgetHeaderTitleFieldSubtitle',
							value: 'Subtitle'
						}],
						listeners: {
							scope: this,
							change: this.updateTitleFormVisibility
						}
					}, {
						layout: 'form',
						columnWidth: 0.8,
						items: [
							this.getTitleFormConfigByType(fontStorage, fontSizeStorage, 'Title'),
							this.getTitleFormConfigByType(fontStorage, fontSizeStorage, 'Subtitle')
						]
					}]
				}, {
					title		: SbGetLocaleByKey('widgetHeaderLogoTabTitle'),
					cls			: 'sb-widget-header-logo-panel',
					autoHeight	: true,
					items: {
						xtype	: 'checkbox',
						itemCls	: 'sb-edit-control sb-edit-control-hidelogo',
						boxLabel: SbGetLocaleByKey('widgetHeaderLogoHide'),
						id		: 'hidelogoCheckBox',
						listeners: {
							scope: this,
							check: function(check, checked) {
								this.widget.model.setProperty('logoIsHidden', checked);
							}
						}
					},
					listeners: {
						scope: this,
						activate: function() {
							this.initLogoTab();
						}
					}
				}]
			}
		});

		Sb.widget.HeaderEditPanel.superclass.initComponent.apply(this, arguments);

		this.tabs = this.items.item(0);
		this.tabs.frame = true;
	},

	/**
	 * delete custom header
	 */
	onDeleteHeader: function(selectbox, record, index) {

		Sb.SiteService.deleteHeader(record.get('name'), function(result, response) {
			if (result) {
				selectbox.store.removeAt(index);
				selectbox.pageTb.doRefresh();
				Sb.Editor.Ui.mainWindow.statusBar.setMessageInfo(SbGetLocaleByKey('widgetHeaderImageWasDeleted'));
			} else {
				Sb.Editor.Ui.mainWindow.restore();
				Sb.Editor.Ui.mainWindow.statusBar.setMessageError(SbGetLocaleByKey('widgetHeaderImageDeleteFailed').replace('{headerName}', record.get('name')));
			}
		});
	},

	initLogoTab: function() {
		Ext.getCmp('hidelogoCheckBox').setValue(this.widget.model.data.logoIsHidden);
	},

	//NOTE: austimenko: 28.07.2010: These 3 methods about Position will be reactivated in US #48567
	/**
	 * Generate position tab
	 *
	 * @return {Ext.Panel}
	 */
	createPositionTab: function() {
		/**
		 * Position tab
		 *
		 * @type Ext.Panel
		 */
		this.positionTab = new Ext.Panel({
			title		: SbGetLocaleByKey('widgetHeaderPositionTabTitle'),
			cls			: 'sb-widget-header-position-panel',
			autoHeight	: true
		});
		this.positionTab.on('render', this.renderPositionTab, this, {single: true});
		return this.positionTab;
	},

	/**
	 * Render position tab
	 */
	renderPositionTab: function() {
		this.positionTab.add(
			new Ext.form.Label({
				text	: SbGetLocaleByKey('widgetHeaderPositionSet')
			})
		);

		var slidersGroup = new Ext.Container({
			cls			: 'sb-widget-edit-sliders-group'
		});
		this.positionTab.add(slidersGroup);

		this.horizontalPositionSld = new Ext.form.SliderField();
		slidersGroup.add(this.horizontalPositionSld);
		slidersGroup.add(
			new Ext.form.Label({
				text	: SbGetLocaleByKey('widgetHeaderPositionHorizontal')
			})
		);

		this.verticalPositionSld = new Ext.form.SliderField();
		slidersGroup.add(this.verticalPositionSld);
		slidersGroup.add(
			new Ext.form.Label({
				text	: SbGetLocaleByKey('widgetHeaderPositionVertical')
			})
		);

		this.positionTab.on('afterrender', this.bindPositionTab, this, {single: true});
	},

	bindPositionTab: function() {
		var model		= this.widget.model;
	},

	onImageSelectionChange: function() {
		var selection = Ext.getCmp('widgetHeaderImageSelection').getValue().getGroupValue();
		var imageList = Ext.getCmp('widgetHeaderImageList');

		if ('no' == selection) {
			this.onImageSelectionNo();
		} else {
			Ext.getCmp('widgetHeaderImageHeightContainer').hide();
			Ext.getCmp('widgetHeaderImageListFilter').hide();
			Ext.getCmp('widgetHeaderImageListContainer').show();
			Ext.getCmp('sizeColumn').show();

			if ('file' == selection) {
				this.onImageSelectionFile();
			} else {
				this.onImageSelectionLibrary();
			}

			if (this.widget.model.data.imageName != '') {
				Ext.getCmp('widgetHeaderImageList').setValue(Sb.Config.headersUrl + '/' + this.widget.model.data.imageName + '/preview.png');
			} else {
				Ext.getCmp('widgetHeaderImageList').setValue(
					Sb.Editor.model.getUrl()
					+ this.widget.model.data.imagePath.replace('/image_', '/thumbnail_image_')
					+ '?' + this.widget.model.data.imageTime
				);
			}
		}
	},

	onImageSelectionNo: function() {
		Ext.getCmp('widgetHeaderFileupload').hide();
		Ext.getCmp('widgetHeaderImageListContainer').hide();
		Ext.getCmp('sizeColumn').hide();
		Ext.getCmp('widgetHeaderImageHeightContainer').show();
		Ext.getCmp('widgetHeaderImageHeight').setValue(this.widget.model.data.height);

		this.widget.model.unserialize({
			imageName: 'fallback',
			imagePath:  ''
		});
	},

	onImageSelectionFile: function() {
		var imageList = Ext.getCmp('widgetHeaderImageList');
		Ext.getCmp('widgetHeaderImageListFilter').hide();
		Ext.getCmp('widgetHeaderFileupload').show();
		imageList.bindStore(this.headerImageCustomStore);
		if (!imageList.store.proxy) {
			Sb.SiteService.getHeaders(function(result) {
				Ext.each(result, function(data) {
					data.path = Sb.Editor.model.getUrl() + '/' + Sb.Config.headerPath + '/thumbnail_' + data.name + '?' + data.time;
				});
				imageList.store.proxy = new Ext.ux.data.PagingMemoryProxy(result);
				this.onImageSelectionFile();
			}, this);
			return;
		}

		imageList.store.load({params: {start: 0, limit: this.imageListPageSize}});
	},

	onImageSelectionLibrary: function() {
		var imageList = Ext.getCmp('widgetHeaderImageList');
		Ext.getCmp('widgetHeaderImageListFilter').show();
		Ext.getCmp('widgetHeaderFileupload').hide();
		imageList.bindStore(this.headerImageStore);
		if (!imageList.store.proxy) {
			Sb.SystemService.getHeaders(function(result) {
				Ext.each(result, function(data) {
					data.path = Sb.Config.headersUrl + '/' + data.name + '/preview.png';
				});
				imageList.store.proxy = new Ext.ux.data.PagingMemoryProxy(result);
				this.onImageSelectionLibrary();
			}, this);
			return;
		}

		imageList.store.load({params: {start: 0, limit: this.imageListPageSize}});
	},

	updateTitleFormVisibility: function() {
		var radioGroup = Ext.getCmp('widgetHeaderTitleFieldGroup');
		Ext.getCmp('widgetHeaderTitleProperties').setVisible(
			'Title' == radioGroup.getValue().value
		);
		Ext.getCmp('widgetHeaderSubtitleProperties').setVisible(
			'Subtitle' == radioGroup.getValue().value
		);
	},

	getTitleFormConfigByType: function(fontStorage, fontSizeStorage, type) {
		var fontDecorationConfig = {
			xtype: 'button',
			enableToggle: true ,
			listeners: {
				scope: this,
				toggle: this.onTitleFieldChange
			}
		};

		return {
			layout: 'form',
			id: 'widgetHeader' + type + 'Properties',
			cls: 'sb-header-title-field-properties',
			labelAlign: 'top',
			defaults: {
				labelSeparator: ''
			},
			items: [{
				xtype: 'textfield',
				id: 'widgetHeader' + type + 'Text',
				anchor: '100%',
				labelSeparator: '',
				fieldLabel: SbGetLocaleByKey('widgetHeader' + type + 'TextTitle'),
				width: 160,
				listeners: {
					scope: this,
					valid: this.onTitleFieldChange
				}
			}, {
				xtype: 'combo',
				id: 'widgetHeader' + type + 'FontFamily',
				editable: false,
				width: 120,
				triggerAction: 'all',
				mode: 'local',
				store: fontStorage,
				valueField: 'value',
				displayField: 'value',
				fieldLabel: SbGetLocaleByKey('widgetHeaderFontFamily'),
				tpl: new Ext.XTemplate(
					'<tpl for="."><div class="x-combo-list-item"' +
						'<tpl if="value!=\'Webdings\' && value!=\'Wingdings\'"> style="font-family:{value}"</tpl>' +
					'>{value}</div></tpl>'
				),
				listeners: {
					scope: this,
					select: function(combo, record, index) {
						this.onTitleFieldChange();
					}
				}
			}, {
				xtype: 'combo',
				id: 'widgetHeader' + type + 'FontSize',
				width: 60,
				triggerAction: 'all',
				editable: true,
				maskRe: /[0-9]/,
				mode: 'local',
				store: fontSizeStorage,
				valueField: 'value',
				displayField: 'value',
				fieldLabel: SbGetLocaleByKey('widgetHeaderFontSize'),
				listeners: {
					scope: this,
					select: this.onTitleFieldChange,
					blur: this.onTitleFieldChange,
					specialkey: function(combo, e) {
						if (e.getKey() == e.ENTER) {
							this.onTitleFieldChange();
						}
					}
				}
			}, {
				layout: 'form',
				id: 'widgetHeader' + type + 'FontDecorations',
				cls: 'sb-widget-font-decorations sb-edit-control',
				fieldLabel: SbGetLocaleByKey('widgetHeaderFontDecoration'),
				autoWidth: true,
				defaults: {
					labelSeparator: ''
				},
				items: [{
					xtype: 'colordropdown',
					iconCls: 'x-edit-forecolor',
					id: 'widgetHeader' + type + 'FontColor',
					noColorText: SbGetLocaleByKey('autoColor'),
					isShowColorField: false,
					isAutoValueNull: false,
					hideLabel: true,
					listeners: {
						scope: this,
						change: this.onTitleFieldChange
					}
				}, Ext.apply(
					{
						id: 'widgetHeader' + type + 'FontBold',
						iconCls: 'x-edit-bold'
					},
					fontDecorationConfig
				), Ext.apply(
					{
						id: 'widgetHeader' + type + 'FontItalic',
						iconCls: 'x-edit-italic'
					},
					fontDecorationConfig
				), Ext.apply(
					{
						id: 'widgetHeader' + type + 'FontUnderline',
						iconCls: 'x-edit-underline'
					},
					fontDecorationConfig
				)]
			}]
		};
	},

	afterRender: function() {
		Sb.widget.HeaderEditPanel.superclass.afterRender.call(this);

		var hw = this.tabs.header.dom.offsetWidth;
		var	lis = this.tabs.strip.query('li:not(.x-tab-edge)');
		var	len = lis.length, i;
		for (i = 0; i < len; i++) {
			hw -= lis[i].offsetWidth + 2;
		}
		var delta = Math.floor(hw / len);
		for (i = 0; i < len - 1; i++) {
			lis[i].style.width = (lis[i].offsetWidth + delta) + 'px';
			hw -= delta;
		}
		lis[len - 1].style.width = (lis[len - 1].offsetWidth + hw + 2) + 'px';

		this.widget.on('headerloaded', this.onHeaderLoaded, this);
	},

	onHeaderLoaded: function() {
		if (this.widget.model.data.imageName == 'fallback') {
			Ext.getCmp('widgetHeaderImageSizeHint').hide();
			Ext.getCmp('widgetHeaderImageFit').disable();
			return;
		}

		if (!this.realImg) {
			Ext.DomHelper.append(
				Ext.getBody(),
				{
					cn: [{
						tag: 'img',
						id: 'img-header-image-real-size'
					}]
				}
			);
			this.realImg = Ext.get('img-header-image-real-size');
		}
		this.realImg.set({'src': this.widget.imageEl.getAttribute('src')});
		var headerImgWidth = this.realImg.getWidth();
		var headerImgHeight = this.realImg.getHeight();
		Ext.getCmp('widgetHeaderImageSizeHint').show();
		Ext.getCmp('widgetHeaderImageFit').enable();
		Ext.getCmp('widgetHeaderImageSizeHint').setValue(String.format(
			SbGetLocaleByKey('widgetHeaderImageSizeHint'), headerImgWidth, headerImgHeight
		));
	},

	show: function(widget) {
		Sb.widget.HeaderEditPanel.superclass.show.call(this, widget);

		this.tabs.setActiveTab(0);

		Ext.getCmp('widgetHeaderImageFit').setValue(this.widget.model.data.imageFit == 1);
		Ext.getCmp('widgetHeaderImageSelection').setValue();
		if (this.widget.model.data.imageName == '') {
			Ext.getCmp('widgetHeaderImageSelection').setValue('file');
		} else if (this.widget.model.data.imageName == 'fallback'){
			Ext.getCmp('widgetHeaderImageSelection').setValue('no');
		} else {
			Ext.getCmp('widgetHeaderImageSelection').setValue('library');
		}
		var w = this.widget.contentElement.getWidth();
		Ext.getCmp('widgetHeaderFileupload').el.child('label').update(String.format(
			SbGetLocaleByKey('widgetHeaderUploadLabel'), w
		));
		this.onHeaderLoaded();
		this.initTitleTab();
	},

	initTitleTab: function() {
		Ext.getCmp('widgetHeaderTitleFieldGroup').setValue('widgetHeaderTitleFieldTitle', true);
		Ext.getCmp('widgetHeaderTitleText').setRawValue(this.widget.model.data.title);
		Ext.getCmp('widgetHeaderTitleFontFamily').setValue(this.widget.model.data.titleFontFamily);
		Ext.getCmp('widgetHeaderTitleFontSize').setValue(this.widget.model.data.titleFontSize);
		Ext.getCmp('widgetHeaderTitleFontColor').setInitialValue(this.widget.model.data.titleFontColor);
		Ext.getCmp('widgetHeaderTitleFontColor').setAutoValue('#000000');
		Ext.getCmp('widgetHeaderTitleFontBold').toggle('1' == this.widget.model.data.titleFontBold, true);
		Ext.getCmp('widgetHeaderTitleFontItalic').toggle('1' == this.widget.model.data.titleFontItalic, true);
		Ext.getCmp('widgetHeaderTitleFontUnderline').toggle('1' == this.widget.model.data.titleFontUnderline, true);
		Ext.getCmp('widgetHeaderSubtitleText').setRawValue(this.widget.model.data.subtitle);
		Ext.getCmp('widgetHeaderSubtitleFontFamily').setValue(this.widget.model.data.subtitleFontFamily);
		Ext.getCmp('widgetHeaderSubtitleFontSize').setValue(this.widget.model.data.subtitleFontSize);
		Ext.getCmp('widgetHeaderSubtitleFontColor').setInitialValue(this.widget.model.data.subtitleFontColor);
		Ext.getCmp('widgetHeaderSubtitleFontColor').setAutoValue('#000000');
		Ext.getCmp('widgetHeaderSubtitleFontBold').toggle('1' == this.widget.model.data.subtitleFontBold, true);
		Ext.getCmp('widgetHeaderSubtitleFontItalic').toggle('1' == this.widget.model.data.subtitleFontItalic, true);
		Ext.getCmp('widgetHeaderSubtitleFontUnderline').toggle('1' == this.widget.model.data.subtitleFontUnderline, true);
	},

	onTitleFieldChange: function() {
		var oldHeight = this.widget.contentElement.getHeight();

		this.widget.model.unserialize({
			title: Ext.getCmp('widgetHeaderTitleText').getValue(),
			titleFontFamily: Ext.getCmp('widgetHeaderTitleFontFamily').getValue(),
			titleFontSize: Ext.getCmp('widgetHeaderTitleFontSize').getRawValue(),
			titleFontColor: Ext.getCmp('widgetHeaderTitleFontColor').getValue(),
			titleFontBold: Ext.getCmp('widgetHeaderTitleFontBold').pressed ? '1' : '0',
			titleFontItalic: Ext.getCmp('widgetHeaderTitleFontItalic').pressed ? '1' : '0',
			titleFontUnderline: Ext.getCmp('widgetHeaderTitleFontUnderline').pressed ? '1' : '0',
			subtitle: Ext.getCmp('widgetHeaderSubtitleText').getValue(),
			subtitleFontFamily: Ext.getCmp('widgetHeaderSubtitleFontFamily').getValue(),
			subtitleFontSize: Ext.getCmp('widgetHeaderSubtitleFontSize').getRawValue(),
			subtitleFontColor: Ext.getCmp('widgetHeaderSubtitleFontColor').getValue(),
			subtitleFontBold: Ext.getCmp('widgetHeaderSubtitleFontBold').pressed ? '1' : '0',
			subtitleFontItalic: Ext.getCmp('widgetHeaderSubtitleFontItalic').pressed ? '1' : '0',
			subtitleFontUnderline: Ext.getCmp('widgetHeaderSubtitleFontUnderline').pressed ? '1' : '0'
		});

		if (oldHeight != this.widget.contentElement.getHeight()) {
			Sb.widget.Header.editPanel.reanchor();
		}
	},

	onUploadFile: function(field, fieldValue) {
		var widgetHeaderForm  = Ext.getCmp('widgetHeaderFileupload').getForm();
		if (!widgetHeaderForm.isValid()) {
			field.markInvalid(SbGetLocaleByKey('widgetHeaderUploadErrorTypeInvalid'));
			return;
		}
		widgetHeaderForm.submit({
			url: 'Ajax/Editor/HeaderUpload',
			waitMsg: SbGetLocaleByKey('uploadingImage'),
			success: function(fp, action){
				var imageList = Ext.getCmp('widgetHeaderImageList');
				this.widget.model.unserialize({
					imageName: '',
					imagePath: '/' + Sb.Config.headerPath + '/' + action.result.uploadedFileName,
					imageTime: action.result.time
				});
				imageList.store.proxy.destroy();
				delete imageList.store.proxy;
				this.onImageSelectionChange();
				field.reset();
			}.createDelegate(this),
			failure: function(fp, action){
				field.markInvalid(SbGetLocaleByKey(action.result.message));
			}
		});
	}
});