Sb.widget.NavigationEditPanel = Ext.extend(Sb.widget.WidgetEditPanel, {
	initComponent: function() {
		Ext.apply(this, {
			width: 510,
			autoHeight: true,
			cls: 'sb-widget-edit-panel-navigation',
			items: {
				xtype: 'tabpanel',
				autoHeight: true,
				listeners: {
					scope: this,
					tabchange: function(panel, tab) {
						this.toggleHelperMessage(tab.initialConfig.hasHelperMessage);
					}
				},
				items: [{
					cls: 'sb-widget-navigation-levels-panel',
					hasHelperMessage: true,
					title: SbGetLocaleByKey('widgetNavigationLevelsTabTitle'),
					layout: 'form',
					autoHeight: true,
					labelAlign: 'top',
					items: {
						xtype: 'radiogroup',
						id: 'navigation-level',
						fieldLabel: SbGetLocaleByKey('showMenuLevels'),
						labelSeparator: '',
						defaults: {
							name: 'level'
						},
						items: [{
							boxLabel: SbGetLocaleByKey('firstLevel'),
							itemCls: 'sb-navigation-level-1',
							inputValue: '1-1',
							helperMessage: SbGetLocaleByKey('widgetNavigationLevelMessage11')
						}, {
							boxLabel: SbGetLocaleByKey('secondLevel'),
							itemCls: 'sb-navigation-level-2',
							inputValue: '2-1',
							helperMessage: SbGetLocaleByKey('widgetNavigationLevelMessage21')
						}, {
							boxLabel: SbGetLocaleByKey('firstAndSecondLevel'),
							itemCls: 'sb-navigation-level-1-2',
							inputValue: '1-2',
							helperMessage: SbGetLocaleByKey('widgetNavigationLevelMessage12')
						}],
						listeners: {
							scope: this,
							change: function(group, radio) {
								var value = radio.getGroupValue().split('-');
								this.widget.model.unserialize({
									level: parseInt(value[0]),
									depth: parseInt(value[1])
								});
								this.setHelperMessage(radio.helperMessage);
								this.reanchor();
							}
						}
					}
				},
				this.createMenuItemsTab()
				]
			}
		});

		Sb.widget.NavigationEditPanel.superclass.initComponent.apply(this, arguments);

		this.tabs = this.items.item(0);
		this.tabs.frame = true;
	},

	/**
	 * Generate menu items tab
	 *
	 * @return {Ext.Panel}
	 */
	createMenuItemsTab: function() {
		/**
		 * Menu items tab
		 *
		 * @type Ext.Panel
		 */
		this.menuItemsTab = new Ext.Panel({
			title		: SbGetLocaleByKey('widgetNavigationMenuItemsTabTitle'),
			layout		: 'column',
			cls			: 'sb-widget-navigation-items-panel',
			autoHeight	: true
		});
		this.menuItemsTab.on('render', this.renderMenuItemsTab, this, {single: true});
		return this.menuItemsTab;
	},

	/**
	 * Render menu items tab
	 */
	renderMenuItemsTab: function() {
		/**
		 * Possible link states
		 *
		 * @type Array
		 * @property linkStates
		 */
		this.linkStates		= ['Normal', 'Selected', 'Hovered'];

		/**
		 * Current selected link state
		 *
		 * @type String
		 * @property activeLinkState
		 */
		this.activeLinkState = '';

		/**
		 * Menu item alignments
		 *
		 * @type Array
		 * @property alignments
		 */
		this.alignments		= ['Left', 'Center', 'Right'];

		var smallMargin		= 6;
		var smallButtonWidth= smallMargin * 4;
		var smallCellWidth	= smallMargin * 12;
		var mediumMargin	= smallMargin * 2;
		var mediumCellWidth	= smallCellWidth * 2;

		var tabPanelWidth	= this.width - mediumMargin;
		var colorComboWidth	= smallCellWidth * 1.5;
		var linkStateWidth	= Math.floor(tabPanelWidth * 0.25 - 1);
		var contentWidth	= Math.floor(tabPanelWidth * 0.75 - 1);

		var widgetContolCls = 'sb-edit-control sb-edit-control-';

		// Create link state group
		var stateGroup = new Sb.form.FieldGroup({
			columnWidth		: .25,
			autoWidth		: false
		});
		var stateItems = [];
		Ext.each(this.linkStates, function (linkState, index) {
			var stateItem = new Ext.form.Radio({
				boxLabel		: SbGetLocaleByKey('widgetNavigationLinkState' + linkState),
				inputValue		: linkState,
				name			: 'stateItems'
			});
			stateItem.on('render', function() {
				stateItem.wrap.unselectable();
			}, stateItem, {single: true});
			stateItems.push(stateItem);
		});
		var stateRadioGroup = new Ext.form.RadioGroup({
			items			: stateItems,
			columns			: 1,
			width			: linkStateWidth,
			itemCls			: widgetContolCls + 'link-state',
			fieldLabel		: SbGetLocaleByKey('widgetNavigationLinkState'),
			labelSeparator	: ''
		});
		stateRadioGroup.on('change', function (select, option) {
			var isNormal	= 'Normal' == option.inputValue;
			var action		= (isNormal) ? 'enable' : 'disable';
			fontAlignmentGroup.items.each(function(item) {
				item.ownerCt[action]();
				item[action]();
				if (!item.items) {
					return;
				}
				item.items.each(function(subitem) {
					subitem[action]();
				});
			});
			this.colorDecorationsGroup.layout.setActiveItem(select.items.indexOf(option));
			this.activeLinkState = option.inputValue;
		}, this);
		stateGroup.add(stateRadioGroup);
		this.menuItemsTab.add(stateGroup);

		// Create content group
		var contentGroup = new Sb.form.FieldGroup({
			columnWidth		: .75,
			autoWidth		: false
		});

		// Create content group -> font/alignment group
		var fontAlignmentGroup = new Sb.form.FieldGroup({
			cls				: 'clearfix'
		});
		this.fontFamilySizeGroup = new Sb.form.FontFamilySizePanel({
			fontFamilyWidth	: mediumCellWidth,
			fontSizeWidth	: smallCellWidth,
			widgetContolCls	: widgetContolCls
		});
		fontAlignmentGroup.add(this.fontFamilySizeGroup);

		// Create content group -> font/alignment group -> alignment group
		this.alignmentGroup = new Sb.form.FieldGroup({
			itemId			: 'Alignment',
			layout			: 'column',
			itemCls			: widgetContolCls + 'alignment',
			fieldLabel		: SbGetLocaleByKey('widgetNavigationMenuAlignment')
		});
		Ext.each(this.alignments, function(alignment) {
			var alignmentButton = new Ext.Button({
				itemId		: alignment,
				iconCls		: 'x-edit-alignment-' + alignment.toLowerCase(),
				enableToggle: true
			});
			alignmentButton.on('toggle', function(button, pressed) {
				if (!pressed) {
					return;
				}
				this.alignmentGroup.items.each(function(button2) {
					if (!button2.pressed) {
						return;
					}
					if (button2.id == button.id) {
						return;
					}
					button2.toggle(false);
				});
			}, this);
			this.alignmentGroup.add(alignmentButton);
		}, this);
		fontAlignmentGroup.add(this.alignmentGroup);
		contentGroup.add(fontAlignmentGroup);

		// Create content group -> color/decorations group
		this.colorDecorationsGroup = new Sb.form.FieldGroup({
			layout				: 'card'
		});
		Ext.each(this.linkStates, function(linkState) {
			var panel = new Sb.form.ColorDecorationPanel({
				itemId			: linkState,
				cls				: widgetContolCls + 'color-decorations clearfix',
				colorComboWidth	: colorComboWidth,
				widgetContolCls	: widgetContolCls
			});
			if ('Normal' == linkState) {
				panel.toggle('BackgroundColor');
			}
			this.colorDecorationsGroup.add(panel);
		}, this);
		contentGroup.add(this.colorDecorationsGroup);

		this.menuItemsTab.add(contentGroup);

		this.on('show', this.bindMenuItemsTab, this);
		this.menuItemsTab.on('afterrender', this.bindMenuItemsTab, this, {single: true});
		fontAlignmentGroup.toggle('Alignment', false);
		this.menuItemsTab.on('activate', function () {
			var isShowAlignment = !(this.widget.el.up('#column1') || this.widget.el.up('#column2'));
			fontAlignmentGroup.toggle('Alignment', isShowAlignment);
			stateRadioGroup.setValue(this.linkStates[0]);
		}, this);
	},

	bindMenuItemsTab: function() {
		var model			= this.widget.model,
			palette			= Sb.Editor.model.design.palette,
			region			= this.widget.ownerCt.model.data.region;

		// Font family
		var fontFamily		= this.fontFamilySizeGroup.getComponent('FontFamily');
		var fontFamilyFn	= function() {
			model.setByLevel('FontFamily', this.value);
		};
		fontFamily.setValue(model.getByLevel('FontFamily'));
		fontFamily.on('select', fontFamilyFn);
		this.on('hide', function () {
			fontFamily.un('select', fontFamilyFn);
		}, null, {single: true});

		// Font size
		var fontSize		= this.fontFamilySizeGroup.getComponent('FontSize');
		var fontSizeFn		= function() {
			model.setByLevel('FontSize', this.value);
		};
		var fontSizeFnCustomFont = function(combo, e) {
			if (e.getKey() == e.ENTER) {
				model.setByLevel('FontSize', combo.getRawValue());
			}
		};
		fontSize.setValue(model.getByLevel('FontSize'));
		fontSize.on('select', fontSizeFn);
		fontSize.on('blur', fontSizeFn);
		fontSize.on('specialkey', fontSizeFnCustomFont);
		this.on('hide', function () {
			fontSize.un('select', fontSizeFn);
			fontSize.un('blur', fontSizeFn);
			fontSize.un('specialkey', fontSizeFnCustomFont);
		}, null, {single: true});

		// Alignment
		var alignmentValue	= model.getByLevel('Alignment');
		Ext.each(this.alignments, function(alignmentType) {
			var alignmentBtn	= this.alignmentGroup.getComponent(alignmentType);
			if (alignmentType == alignmentValue) {
				alignmentBtn.toggle(true);
			}
			var alignmentFn		= function() {
				model.setByLevel('Alignment', this.itemId);
			};
			alignmentBtn.on('toggle', alignmentFn);
			this.on('hide', function () {
				alignmentBtn.un('toggle', alignmentFn);
			}, null, {single: true});
		}, this);

		// Link state -> Colors & Decorations
		Ext.each(this.linkStates, function(linkState) {
			// Local vars
			var stateForModel	= ('Normal' == linkState) ? '' : linkState;
			var colorTypes		= ['Color', 'BackgroundColor'];
			var panel			= this.colorDecorationsGroup.getComponent(linkState);

			// Colors
			Ext.each(colorTypes, function(colorType) {
				var modelKey		= colorType + stateForModel,
					paletteKey		= region + 'NavigationLevel1' + modelKey,
					colorCmb		= panel.getComponent(colorType),
					colorFn			= function() {
					model.setByLevel(modelKey, this.getValue());
					if ('BackgroundColor' == modelKey) {
						model.setByLevel('Container' + modelKey, this.getValue());
					}
				};

				colorCmb.setInitialValue(model.getByLevel(modelKey));
				colorCmb.setAutoValue(palette[paletteKey]);
				colorCmb.on('change', colorFn);
				this.on('hide', function () {
					colorCmb.un('change', colorFn);
				}, null, {single: true});
			}, this);

			// Decorations
			Ext.each(panel.decorations, function(decoration) {
				var modelKey		= decoration + stateForModel;
				var decorationBtn	= panel.getComponent('Decoration').getComponent(decoration);
				var decorationFn	= function() {
					model.setByLevel(modelKey, this.pressed);
				};
				decorationBtn.toggle(1 == model.getByLevel(modelKey));
				decorationBtn.on('toggle', decorationFn);
				this.on('hide', function () {
					decorationBtn.un('toggle', decorationFn);
				}, null, {single: true});
			}, this);
		}, this);
	},

	afterRender: function() {
		Sb.widget.NavigationEditPanel.superclass.afterRender.call(this);

		var hw = this.tabs.header.dom.offsetWidth;
		var	lis = this.tabs.strip.query('li:not(.x-tab-edge)');
		var	len = lis.length, i;
		for (i = 0; i < len; i++) {
			hw -= lis[i].offsetWidth + 2;
		}
		var delta = Math.floor(hw / len);
		for (i = 0; i < len - 1; i++) {
			lis[i].style.width = (lis[i].offsetWidth + delta) + 'px';
			hw -= delta;
		}
		lis[len - 1].style.width = (lis[len - 1].offsetWidth + hw + 2) + 'px';

		this.tabs.ft.first().first().createChild({
			cls: 'sb-widget-helper',
			cn: {
				cls: 'sb-widget-helper-wrap',
				cn: {
					cls: 'sb-widget-helper-inner',
					cn: {
						cls: 'sb-widget-helper-body'
					}
				}
			}
		});
		this.tabs.ft.child('.sb-widget-helper:first').setVisibilityMode(Ext.Element.DISPLAY);
	},

	/**
	 * Toggles display of helper message
	 * @param {Boolean} isShow
	 */
	toggleHelperMessage: function(isShow) {
		this.tabs.ft.child('.sb-widget-helper:first')[isShow ? 'show' : 'hide']();
	},

	/**
	 * Sets helper message for tab
	 * @param {String} message
	 */
	setHelperMessage: function(message) {
		this.tabs.ft.child('.sb-widget-helper-body:first').update(message);
	},

	show: function(widget) {
		Sb.widget.NavigationEditPanel.superclass.show.call(this, widget);
		this.tabs.setActiveTab(0);
		Ext.getCmp('navigation-level').setValue(this.widget.model.data.level + '-' + this.widget.model.data.depth);
	}
});