/**
 * @class Sb.widgetType.Shared
 * @extends Sb.widgetType.Base
 */
Sb.widgetType.Shared = Ext.extend(Sb.widgetType.Base, {

	constructor: function(name) {
		/**
		 * Cache of widgets by uuid
		 *
		 * @type Ext.util.MixedCollection
		 * @property cache
		 */
		this.cache = new Ext.util.MixedCollection(false,
			/**
			 * Provide key for collection
			 * @param {Sb.model.Widget} widget
			 * @return {String}
			 */
			function(widget) {
				return widget.data.uuid;
			}
		);

		this.cache.on('add', this.onCacheChange, this);
		this.cache.on('remove', this.onCacheChange, this);

		Sb.widgetType.Shared.superclass.constructor.call(this, name);
	},

	/**
	 * Gets type for creating child
	 * @return {String}
	 */
	getPhpType: function() {
		var reShared = /^shared/;
		if (reShared.test(this.name)) {
			return 'shared_' + this.name.replace(reShared, '').toLowerCase();
		}
		return this.name;
	},

	/**
	 * Fires when number of shared widgets in site changed
	 */
	onCacheChange: Ext.emptyFn,

	/**
	 * Instance factory with cache by uuid
	 *
	 * @param {Object} config The configuration options for the widget model
	 * @return {Sb.model.SharedWidget}
	 */
	createInstance: function(config) {
		if (config.uuid && !this.cache.map[config.uuid]) {
			var widget = Sb.widgetType.Shared.superclass.createInstance.call(this, config);
			this.cache.add(widget);
		}
		return this.cache.map[config.uuid];
	},


	/**
	 * Widget cleaner by type
	 *
	 * @param {Sb.model.Widget} widget
	 */
	removeInstance: function(widget) {
		this.cache.remove(widget);
	},

	/**
	 * Restore widget in cache
	 *
	 * @param {Sb.model.Widget} widget
	 */
	restoreInstance: function(widget) {
		if (widget.data.uuid && !this.cache.map[widget.data.uuid]) {
			this.cache.add(widget);
		}
	}
});