/**
 * Themed checkboxes/radios for Ext 3.0
 * http://www.extjs.com/forum/showthread.php?64689-Themed-checkboxes-radios-for-Ext-3.0
 */
Ext.override(Ext.form.Checkbox, {
	innerCls: 'x-form-checkbox-inner',
	onResize: function() {
		Ext.form.Checkbox.superclass.onResize.apply(this, arguments);
		if (!this.boxLabel) {
			this.innerWrap.alignTo(this.wrap, 'c-c');
		}
	},
	initEvents: function() {
		Ext.form.Checkbox.superclass.initEvents.call(this);
		this.mon(this.el, {
			click: this.onClick,
			change: this.onClick,
			mouseenter: this.onMouseEnter,
			mouseleave: this.onMouseLeave,
			mousedown: this.onMouseDown,
			mouseup: this.onMouseUp,
			scope: this
		});
	},
	onRender: function(ct, position) {
		Ext.form.Checkbox.superclass.onRender.call(this, ct, position);
		if (this.inputValue !== undefined) {
			this.el.dom.value = this.inputValue;
		}
		this.innerWrap = this.el.wrap({
			tag: 'span',
			cls: this.innerCls
		});
		this.wrap = this.innerWrap.wrap({
			cls: 'x-form-check-wrap'
		});
		if (this.boxLabel) {
			this.labelEl = this.wrap.createChild({
				tag: 'label',
				htmlFor: this.el.id,
				cls: 'x-form-cb-label',
				html: this.boxLabel
			});
		} else {
			this.innerWrap.addClass('x-form-check-no-label');
		}
		if (this.checked) {
			this.setValue(true);
		} else {
			this.checked = this.el.dom.checked;
		}
		// Need to repaint for IE, otherwise positioning is broken
		if (Ext.isIE) {
			this.wrap.repaint();
		}

		this.resizeEl = this.positionEl = this.wrap;
	},
	initValue: function() {
		this.originalValue = this.getValue();
	},
	onClick: function() {
		if (Ext.isSafari) {
			this.focus();
		}
		if (this.el.dom.checked != this.checked) {
			this.setValue(this.el.dom.checked);
		}
	},
	setRawValue: function(v) {
		this.setValue(v, false);
	},
	setValue: function(v, isFireEvent) {
		if (undefined == isFireEvent) {
			isFireEvent = true;
		}
		var checked = this.checked ;
		this.checked = (v === true || v === 'true' || v == '1' || String(v).toLowerCase() == 'on');
		if (this.rendered) {
			this.el.dom.checked = this.checked;
			this.el.dom.defaultChecked = this.checked;
			this.innerWrap[this.checked ? 'addClass' : 'removeClass']('x-form-check-checked');
			this.validate();
		}
		if (checked != this.checked) {
			if (isFireEvent) {
				this.fireEvent('check', this, this.checked);
				if (this.handler) {
					this.handler.call(this.scope || this, this, this.checked);
				}
			}
		}
		return this;
	},
	onMouseEnter: function() {
		this.wrap.addClass('x-form-check-over');
	},
	onMouseLeave: function() {
		this.wrap.removeClass('x-form-check-over');
		this.wrap.removeClass('x-form-check-down');
	},
	onMouseDown: function() {
		this.wrap.addClass('x-form-check-down');
	},
	onMouseUp: function() {
		this.wrap.removeClass('x-form-check-down');
	},
	onFocus: function() {
		Ext.form.Checkbox.superclass.onFocus.call(this);
		this.wrap.addClass('x-form-check-focus');
	},
	onBlur: function() {
		Ext.form.Checkbox.superclass.onBlur.call(this);
		this.wrap.removeClass('x-form-check-focus');
	}
});
Ext.override(Ext.form.Radio, {
	innerCls: 'x-form-radio-inner',
	onClick: Ext.form.Radio.superclass.onClick,
	setValue: function(v) {
		Ext.form.Radio.superclass.setValue.call(this, v);
		if (this.rendered && this.checked) {
			var p = this.el.up('form') || Ext.getBody(),
				els = p.select('input[name=' + this.el.dom.name + ']'),
				id = this.el.dom.id;
			els.each(function(el) {
				if (el.dom.id != id) {
					Ext.getCmp(el.dom.id).setValue(false);
				}
			});
		}
		return this;
	}
});

/**
 * Remove show and dragging from Ext JS message boxes
 */
(function() {
	var dlg = Ext.MessageBox.getDialog();
	dlg.header.removeClass('x-window-draggable');
	dlg.header.on('mousedown', function(e) {
		e.stopEvent();
	});
	dlg.on('show', function() {
		dlg.el.disableShadow();
	});
})();

/**
 * msgTarget for form fields
 */
Ext.form.MessageTargets.sbtitle = {
	mark: function(field, msg){
		Ext.form.MessageTargets.title.mark(field, msg);
		if (field.label) {
			field.label.addClass('sb-label-invalid');
		}
	},
	clear: function(field){
		Ext.form.MessageTargets.title.clear(field);
		if (field.label) {
			field.label.removeClass('sb-label-invalid');
		}
	}
};

Ext.override(Ext.Element, {
	/**
	 * At dragndrop time we not need hover reaction
	 */
	hover: function(overFn, outFn, scope, options) {
		var newOverFn = function() {
			if (Sb.Editor && !Sb.Editor.Ui.isDragActive()) {
				overFn.createDelegate(this, arguments)();
			}
		};
		var newOutFn = function() {
			if (Sb.Editor && !Sb.Editor.Ui.isDragActive()) {
				outFn.createDelegate(this, arguments)();
			}
		};
		var me = this;
		me.on('mouseenter', newOverFn, scope || me.dom, options);
		me.on('mouseleave', newOutFn, scope || me.dom, options);
		return me;
	},

	/**
	 * Tries to fit text in one line and truncates it if needed.
	 * If text truncates, then ellipsis is added at the end.
	 * Tags from innerHTML are truncated!
	 */
	ellipsize: function() {
		var endWith = '...';
		this.dom.innerHTML = Ext.util.Format.stripTags(this.dom.innerHTML);
		var currentText = this.dom.innerHTML;
		var currentHeight = this.getHeight();
		this.dom.innerHTML = endWith;
		var neededHeight = this.getHeight();
		this.dom.innerHTML = currentText;
		if (currentHeight <= neededHeight) {
			return;
		}
		var text = this.dom.innerHTML;
		var textEndWith = text.substring(text.length - endWith.length, text.length);
		if (textEndWith != endWith) {
			this.dom.innerHTML += endWith;
		}
		while (this.getHeight() > neededHeight && this.dom.innerHTML.length > endWith.length) {
			text = this.dom.innerHTML;
			text = text.substring(0, text.length - 1 - endWith.length);
			text += endWith;
			this.dom.innerHTML = text;
		}
	}
});

/**
 * Restrict the maximum number of characters that can be entered into the field.
 */
Ext.override(Ext.form.TextField, {
	maxLength: 65535,
	onRender: function(ct, position) {
		Ext.form.TextField.superclass.onRender.call(this, ct, position);
		this.el.set({
			maxLength: this.maxLength
		});
	}
});
Ext.override(Ext.form.TextArea, {
	initEvents: function() {
		Ext.form.TextArea.superclass.initEvents.call(this);
		this.mon(this.el, 'keypress', function(e) {
			if (this.el.dom.value.length > this.maxLength - 1) {
				if (e.ctrlKey) {
					return;
				}
				var k = e.getKey();
				if (Ext.isGecko && (e.isNavKeyPress() || k == e.BACKSPACE || (k == e.DELETE && e.button == -1))) {
					return;
				}
				var cc = String.fromCharCode(e.getCharCode());
				if (!Ext.isGecko && e.isSpecialKey() && !cc) {
					return;
				}
				e.stopEvent();
			}
		}, this);
	}
});

/**
 * String improvements
 */
Ext.apply(String.prototype, {
	camelize: function(isUpperFirst) {
		var parts = this.split(/[-_]+/);
		var len = parts.length;
		var camelized = (isUpperFirst) ? parts[0].toUpperCaseFirst() : parts[0].toLowerCaseFirst();
		for (var i = 1; i < len; i++) {
			camelized += parts[i].toUpperCaseFirst();
		}
		return camelized;
	},

	toUpperCaseFirst: function() {
		if (this.length) {
			return this.charAt(0).toUpperCase() + this.substring(1);
		} else {
			return this;
		}
	},

	toLowerCaseFirst: function() {
		if (this.length) {
			return this.charAt(0).toLowerCase() + this.substring(1);
		} else {
			return this;
		}
	}
});

Ext.apply(RegExp, {
	/**
	 * Add escape prototype feature to RegExp object
	 */
	escape: function(text){
		var sp;
		if(!arguments.callee.sRE){
			sp = ['/','.','*','+','?','|','(',')','[',']','{','}','\\'];
			arguments.callee.sRE = new RegExp('(\\' + sp.join('|\\') + ')', 'g');
		}
		return text.replace(arguments.callee.sRE, '\\$1');
	}
});

/**
 * Promote our usual form`s defaults here
 */
Ext.override(Ext.layout.FormLayout, {
	labelSeparator	: ''
});

/**
 * Promote our usual combo`s defaults here
 */
Ext.override(Ext.form.ComboBox, {
	triggerAction	: 'all',
	shadow			: false,
	editable		: false
});

/**
 * Fix up qtips non-inclusive mechanism
 */
Ext.override(Ext.QuickTip, {
	/**
	 * Register tips on every rendered child node of passed node, as current target mechanism isn`t inclusive
	 * @param {Object} config The config object (not array)
	 */
	registerSelfAndChildren: function(config) {
		var target	= config.target;
		delete config.target;
		if (!target || Ext.isArray(target)) {
			return;
		}
		var cmp	= Ext.getCmp(target.id),
			tgs	= this.targets;
		if (cmp) {
			if (cmp.wrap) {
				tgs[Ext.id(cmp.wrap)] = config;
				cmp.wrap.select('*[id]').each(function(item) {
					tgs[Ext.id(item)] = config;
				});
			} else {
				tgs[Ext.id(cmp.el)] = config;
				cmp.el.select('*[id]').each(function(item) {
					tgs[Ext.id(item)] = config;
				});
			}
		} else {
			var domTarget = Ext.get(target);
			tgs[Ext.id(domTarget)] = config;
			domTarget.select('*[id]').each(function(item) {
				tgs[Ext.id(item)] = config;
			});
		}
	}
});

Ext.apply(Ext.QuickTips, {
	/**
	 * Register tips on every rendered child node of passed node, as current target mechanism isn`t inclusive
	 */
	registerSelfAndChildren: function() {
		var tip = this.getQuickTip();
		tip.registerSelfAndChildren.apply(tip, arguments);
	}
});

/**
 * Insert <wbr> before slashes
 * @param {String} value The string value to format
 * @return {String} String with <wbr>
 */
Ext.util.Format.wbrBySlash = function(value) {
	return value.replace(/\//g, '<img src="' + Ext.BLANK_IMAGE_URL + '" alt="" width="0" height="0" />/');
};