<?php
/**
 * Published site enviroment checker script
 *
 * @author Alexey Belyaev <abelyaev@swsoft.com>
 * @since 2006/07/05
 * @package Web
 * @copyright (c) 2004-2006, SWsoft
 */
function _getPhpInfo () {
	static $result = '';

	if (empty($result)) {
		if (preg_match('/phpinfo/', ini_get('disable_functions'))) {
			$result = 'fail';
		} else {
			ob_start();
			phpinfo(8);
			$result = ob_get_contents();
			ob_end_clean();
		}
	}
	return $result;
}

if(!function_exists('file_put_contents')) {
	function file_put_contents($fileName, $content) {
		$f = fopen($fileName, 'a+');
		if($f) {
		fwrite($f, $content);
                fclose($f);
		return true;
		}
		return false;
	}
}


//Check ioncube
echo "ioncube=ok\n";

echo "os=".getenv('OS')."\n";

// check php version
echo 'phpVersion=' . phpversion() . "\n";

// check php extensions
$phpExtensions = array (
	'mbstring',
	'zlib',
	'session',
);

foreach ($phpExtensions as $name) {
	if (extension_loaded($name)) {
		echo "$name=ok\n";
	} else {
		echo "$name=fail\n";
	}
}

// check dom
if (extension_loaded('dom') || extension_loaded('domxml')) {
	echo "dom=ok\n";
} else {
	echo "dom=fail\n";
}

$domXslt = true;
// check dom xslt support
if (class_exists('XSLTProcessor') || function_exists('domxml_xslt_stylesheet')) {
	echo "domxml_xslt=ok\n";
} else {
	$domXslt = false;
	echo "domxml_xslt=fail\n";
}

if($domXslt == false) {
	// check xslt support
	if (function_exists('xslt_process')) {
		echo "xslt=ok\n";
	} else {
		echo "xslt=fail\n";
	}
}

//check PDO
if(extension_loaded('pdo')) {
	$drivers = PDO::getAvailableDrivers();
	if(in_array('sqlite', $drivers)) {
		echo "PDO sqlite=ok\n";
	} else {
		echo "PDO sqlite=fail\n";
	}
	if(in_array('sqlite2', $drivers)) {
		echo "PDO sqlite2=ok\n";
	} else {
		echo "PDO sqlite2=fail\n";
	}
	if(in_array('mysql', $drivers)) {
		echo "PDO mysql=ok\n";
	} else {
		echo "PDO mysql=fail\n";
	}
} else {
	echo "PDO=fail\n";
}

//check SQLite
if (extension_loaded('sqlite')) {
	//check SQLite version
	$sqliteVer = '';
	if (function_exists('sqlite_libversion')) {
		$sqliteVer = sqlite_libversion();
	} else {
		$info = _getPhpInfo();
		$info = stristr($info, '>SQLite<');
		$info = stristr($info, 'sqlite library');
		if (preg_match('/(\d+)\.(\d+)\.(\d+)/', $info, $match)) {
			$sqliteVer = $match[0];
		} else {
			$sqliteVer = 'unknown';
		}
	}
	echo "sqlite=$sqliteVer\n";
	//check SQLite encoding
	$sqliteEncoding = '';
	if (function_exists('sqlite_libencoding')) {
		$sqliteEncoding = sqlite_libencoding();
	} else {
		$info = _getPhpInfo();
		$info = stristr($info, '>SQLite<');
		$info = stristr($info, 'sqlite encoding');
		if (preg_match('/<\/[^>]*><[^>]*>([a-zA-Z0-9\-]+)[^\.<]+<\/[^>]*>/', $info, $match)) {
			$sqliteEncoding = $match[1];
		} else {
			$sqliteEncoding = 'unknown';
		}
	}
	echo "sqliteEncoding=$sqliteEncoding\n";
} else {
	echo "sqlite=fail\n";
}

//check gd version
if (extension_loaded('gd')) {
	$gdVer = '';
	$gdFunctions = array();
	if (function_exists('gd_info')) {
		$versionInfo = gd_info();
		$info = $versionInfo['GD Version'];
	} else {
		$info = _getPhpInfo();
		$info = stristr($info, 'gd version');
	}
	if (preg_match('/(\d+)\.(\d+)\.(\d+)/', $info, $match) || preg_match('/(\d+)\.(\d+)/', $info, $match)) {
		$gdVer = $match[0];
		//check supported image types
		$types = ImageTypes();
		if ($types & IMG_PNG) {
			echo "gd_png=ok\n";
		} else {
			echo "gd_png=fail\n";
		}
		if (($types & IMG_GIF) && function_exists('imagecreatefromgif')) {
			echo "gd_gif=ok\n";
		} else {
			echo "gd_gif=fail\n";
		}
		if ($types & IMG_JPG) {
			echo "gd_jpg=ok\n";
		} else {
			echo "gd_jpg=fail\n";
		}
		if ($types & IMG_WBMP) {
			echo "gd_wbmp=ok\n";
		} else {
			echo "gd_wbmp=fail\n";
		}
	} else {
		$gdVer = 'unknown';
	}
	echo "gd=$gdVer\n";
} else {
	echo "gd=fail\n";
}

// check mail support
if(function_exists('mail')) {
	echo "mail=ok\n";
} else {
	echo "mail=fail\n";
}

if(function_exists('curl_init')) {
	echo "curl=ok\n";
} else {
	echo "curl=fail\n";
}

ob_start();
phpinfo();
$phpinfo = ob_get_contents();
ob_end_clean();

preg_match('/<td(.*?)Server API <\/td><td(.*?)<\/td>/', $phpinfo, $sapi);
if (is_array($sapi) && !empty($sapi) && strstr($sapi[0], 'Apache')) {
	echo "serverApi=Apache Handler\n";
} else {
	echo "serverApi=CGI\n";
}